<?php
function ib2_read_images($cat = 'backgrounds') {
	$files = array();
	$allowed = array('jpeg', 'jpg', 'png', 'gif', 'bmp', 'svg');
	if ( $dirs = ib2_get_image_folders($cat) ) {
		$base_url = IB2_IMG . ( $cat != 'backgrounds' ? 'graphics/' : '' );
		$base_path = IB2_PATH . 'assets/img/' . ( $cat != 'backgrounds' ? 'graphics/' : '' );
		foreach ( $dirs as $key => $dir ) {
			if ( file_exists($dir) && is_dir($dir) ) {
				if ( $dh = opendir($dir) ) {
					while ( ($file = readdir($dh)) !== false ) {
						if ( $file != "." && $file != ".." ) {
							$parts = explode(".", $file);
							$ext = strtolower(end($parts));

							if ( !in_array($ext, $allowed) ) {
								continue;
							}

							if ( stristr($file, 'oto') || stristr($file, 'testi') ) {
								continue;
							}

							$fullpath = $base_path . $key . '/' . $file;
							list($width, $height) = getimagesize($fullpath);
							if ( $width < 100 ) {
								continue;
							}

							if ( $height != 100 ) {
								$width = floor($width/$height*100);
							}

							$files[] = array(
								'url' => $base_url . $key . '/' . $file,
								'width' => $width
							);
						}
					}
					closedir($dh);
				}
			}
		}
	}
	return $files;
}

function ib2_get_image_folders($cat = 'backgrounds') {
	$path = IB2_PATH . 'assets/img/';
	$graphic_path = $path . 'graphics/';
	$folders = array(
		'arrows' => array(
			'arrow_buttons' => $graphic_path . 'arrow_buttons/',
			'arrows' => $graphic_path . 'arrows/',
		),
		'backgrounds' => array('backgrounds' => $path . 'backgrounds/'),
		'badges' => array(
			'badges' => $graphic_path . 'badges/',
		),
		'buttons' => array(
			'buttons' => $graphic_path . 'buttons/',
			'buy_buttons' => $graphic_path . 'buy_buttons/',
		),
		'buyarea' => array(
			'buy_now_area' => $graphic_path . 'buy_now_area/',
		),
		'creditcards' => array(
			'credit_cards' => $graphic_path . 'credit_cards/',
		),
		'icons' => array(
			'bigicons' => $graphic_path . 'bigicons/',
		),
		'texts' => array(
			'texts' => $graphic_path . 'texts/',
		)
	);

	if ( isset($folders[$cat]) ) {
		return $folders[$cat];
	}
	return false;
}

add_action('wp_ajax_ib2_get_media', 'ib2_get_media_handler');
function ib2_get_media_handler() {
	if ( isset($_GET['cat']) ) {
		$cat = trim(strip_tags($_GET['cat']));
		$response = ib2_read_images($cat);
		wp_send_json($response);
	}
	die();
}

add_action('wp_ajax_ib2_search_media', 'ib2_search_media_handler');
function ib2_search_media_handler() {
	if ( isset($_GET['q']) && isset($_GET['provider']) ) {
		$provider = $_GET['provider'];
		$keyword = trim(strip_tags($_GET['q']));
		$page = intval($_GET['page']);
		$files = array();
		if ( $provider == 'pixabay' ) {
			$apikey = ( !empty($options['pixabay_key']) ? trim(strip_tags($options['pixabay_key'])) : '525704-ef52bfedd2310b5ef8b568742' );
			$pixabay = ib2_pixabay_images('none', $apikey, $keyword, $page);
			if ( $pixabay && isset($pixabay->totalHits) && $pixabay->totalHits > 0 ) {
				foreach ( $pixabay->hits as $image ) {
					$url = ( isset($image->imageURL) ? $image->imageURL : '' );
					if ( empty($url) ) {
						$url = ( isset($image->largeImageURL) ? $image->largeImageURL : $image->webformatURL );
					}
					$width = $image->imageWidth;
					$height = $image->imageHeight;
					if ( $width < 100 ) {
						continue;
					}

					if ( $height != 100 ) {
						$width = floor($width/$height*100);
					}

					$files[] = array(
						'url' => $url,
						'thumb' => ( isset($image->userImageURL) ? $image->userImageURL : $image->previewURL ),
						'width' => $width
					);
				}
			}
		}
		wp_send_json($files);
	}
	die();
}

function ib2_icon_lists() {
	return array(
		'accessibility' => array('fab fa-accessible-icon', 'fas fa-american-sign-language-interpreting', 'fas fa-assistive-listening-systems', 'fas fa-audio-description', 'fas fa-blind', 'fas fa-braille', 'fas fa-closed-captioning', 'far fa-closed-captioning', 'fas fa-deaf', 'fas fa-low-vision', 'fas fa-phone-volume', 'fas fa-question-circle', 'far fa-question-circle', 'fas fa-sign-language', 'fas fa-tty', 'fas fa-universal-access', 'fas fa-wheelchair'),
		'animals' => array('fas fa-cat', 'fas fa-crow', 'fas fa-dog', 'fas fa-dove', 'fas fa-dragon', 'fas fa-feather', 'fas fa-feather-alt', 'fas fa-fish', 'fas fa-frog', 'fas fa-hippo', 'fas fa-horse', 'fas fa-kiwi-bird', 'fas fa-otter', 'fas fa-paw', 'fas fa-spider'),
		'arrows' => array('fas fa-angle-double-down', 'fas fa-angle-double-left', 'fas fa-angle-double-right', 'fas fa-angle-double-up', 'fas fa-angle-down', 'fas fa-angle-left', 'fas fa-angle-right', 'fas fa-angle-up', 'fas fa-arrow-alt-circle-down', 'far fa-arrow-alt-circle-down', 'fas fa-arrow-alt-circle-left', 'far fa-arrow-alt-circle-left', 'fas fa-arrow-alt-circle-right', 'far fa-arrow-alt-circle-right', 'fas fa-arrow-alt-circle-up', 'far fa-arrow-alt-circle-up', 'fas fa-arrow-circle-down', 'fas fa-arrow-circle-left', 'fas fa-arrow-circle-right', 'fas fa-arrow-circle-up', 'fas fa-arrow-down', 'fas fa-arrow-left', 'fas fa-arrow-right', 'fas fa-arrow-up', 'fas fa-arrows-alt', 'fas fa-arrows-alt-h', 'fas fa-arrows-alt-v', 'fas fa-caret-down', 'fas fa-caret-left', 'fas fa-caret-right', 'fas fa-caret-square-down', 'far fa-caret-square-down', 'fas fa-caret-square-left', 'far fa-caret-square-left', 'fas fa-caret-square-right', 'far fa-caret-square-right', 'fas fa-caret-square-up', 'far fa-caret-square-up', 'fas fa-caret-up', 'fas fa-cart-arrow-down', 'fas fa-chart-line', 'fas fa-chevron-circle-down', 'fas fa-chevron-circle-left', 'fas fa-chevron-circle-right', 'fas fa-chevron-circle-up', 'fas fa-chevron-down', 'fas fa-chevron-left', 'fas fa-chevron-right', 'fas fa-chevron-up', 'fas fa-cloud-download-alt', 'fas fa-cloud-upload-alt', 'fas fa-download', 'fas fa-exchange-alt', 'fas fa-expand-arrows-alt', 'fas fa-external-link-alt', 'fas fa-external-link-square-alt', 'fas fa-hand-point-down', 'far fa-hand-point-down', 'fas fa-hand-point-left', 'far fa-hand-point-left', 'fas fa-hand-point-right', 'far fa-hand-point-right', 'fas fa-hand-point-up', 'far fa-hand-point-up', 'fas fa-hand-pointer', 'far fa-hand-pointer', 'fas fa-history', 'fas fa-level-down-alt', 'fas fa-level-up-alt', 'fas fa-location-arrow', 'fas fa-long-arrow-alt-down', 'fas fa-long-arrow-alt-left', 'fas fa-long-arrow-alt-right', 'fas fa-long-arrow-alt-up', 'fas fa-mouse-pointer', 'fas fa-play', 'fas fa-random', 'fas fa-recycle', 'fas fa-redo', 'fas fa-redo-alt', 'fas fa-reply', 'fas fa-reply-all', 'fas fa-retweet', 'fas fa-share', 'fas fa-share-square', 'far fa-share-square', 'fas fa-sign-in-alt', 'fas fa-sign-out-alt', 'fas fa-sort', 'fas fa-sort-alpha-down', 'fas fa-sort-alpha-up', 'fas fa-sort-amount-down', 'fas fa-sort-amount-up', 'fas fa-sort-down', 'fas fa-sort-numeric-down', 'fas fa-sort-numeric-up', 'fas fa-sort-up', 'fas fa-sync', 'fas fa-sync-alt', 'fas fa-text-height', 'fas fa-text-width', 'fas fa-undo', 'fas fa-undo-alt', 'fas fa-upload'),
		'audiovideo' => array('fas fa-audio-description', 'fas fa-backward', 'fas fa-broadcast-tower', 'fas fa-circle', 'far fa-circle', 'fas fa-closed-captioning', 'far fa-closed-captioning', 'fas fa-compress', 'fas fa-eject', 'fas fa-expand', 'fas fa-expand-arrows-alt', 'fas fa-fast-backward', 'fas fa-fast-forward', 'fas fa-file-audio', 'far fa-file-audio', 'fas fa-file-video', 'far fa-file-video', 'fas fa-film', 'fas fa-forward', 'fas fa-headphones', 'fas fa-microphone', 'fas fa-microphone-alt', 'fas fa-microphone-alt-slash', 'fas fa-microphone-slash', 'fas fa-music', 'fas fa-pause', 'fas fa-pause-circle', 'far fa-pause-circle', 'fas fa-phone-volume', 'fas fa-play', 'fas fa-play-circle', 'far fa-play-circle', 'fas fa-podcast', 'fas fa-random', 'fas fa-redo', 'fas fa-redo-alt', 'fas fa-rss', 'fas fa-rss-square', 'fas fa-step-backward', 'fas fa-step-forward', 'fas fa-stop', 'fas fa-stop-circle', 'far fa-stop-circle', 'fas fa-sync', 'fas fa-sync-alt', 'fas fa-undo', 'fas fa-undo-alt', 'fas fa-video', 'fas fa-volume-down', 'fas fa-volume-mute', 'fas fa-volume-off', 'fas fa-volume-up', 'fab fa-youtube'),
		'automotive' => array('fas fa-air-freshener', 'fas fa-ambulance', 'fas fa-bus', 'fas fa-bus-alt', 'fas fa-car', 'fas fa-car-alt', 'fas fa-car-battery', 'fas fa-car-crash', 'fas fa-car-side', 'fas fa-charging-station', 'fas fa-gas-pump', 'fas fa-motorcycle', 'fas fa-oil-can', 'fas fa-shuttle-van', 'fas fa-tachometer-alt', 'fas fa-taxi', 'fas fa-truck', 'fas fa-truck-monster', 'fas fa-truck-pickup'),
		'autumn' => array('fas fa-apple-alt', 'fas fa-campground', 'fas fa-cloud-sun', 'fas fa-drumstick-bite', 'fas fa-football-ball', 'fas fa-hiking', 'fas fa-mountain', 'fas fa-tractor', 'fas fa-tree', 'fas fa-wind', 'fas fa-wine-bottle'),
		'buildings' => array('fas fa-building', 'far fa-building', 'fas fa-church', 'fas fa-gopuram', 'fas fa-home', 'fas fa-hospital', 'far fa-hospital', 'fas fa-hospital-alt', 'fas fa-house-damage', 'fas fa-industry', 'fas fa-mosque', 'fas fa-place-of-worship', 'fas fa-school', 'fas fa-synagogue', 'fas fa-torii-gate', 'fas fa-university', 'fas fa-vihara'),
		'business' => array('fas fa-address-book', 'far fa-address-book', 'fas fa-address-card', 'far fa-address-card', 'fas fa-archive', 'fas fa-balance-scale', 'fas fa-birthday-cake', 'fas fa-book', 'fas fa-briefcase', 'fas fa-building', 'far fa-building', 'fas fa-bullhorn', 'fas fa-bullseye', 'fas fa-business-time', 'fas fa-calculator', 'fas fa-calendar', 'far fa-calendar', 'fas fa-calendar-alt', 'far fa-calendar-alt', 'fas fa-certificate', 'fas fa-chart-area', 'fas fa-chart-bar', 'far fa-chart-bar', 'fas fa-chart-line', 'fas fa-chart-pie', 'fas fa-city', 'fas fa-clipboard', 'far fa-clipboard', 'fas fa-coffee', 'fas fa-columns', 'fas fa-compass', 'far fa-compass', 'fas fa-copy', 'far fa-copy', 'fas fa-copyright', 'far fa-copyright', 'fas fa-cut', 'fas fa-edit', 'far fa-edit', 'fas fa-envelope', 'far fa-envelope', 'fas fa-envelope-open', 'far fa-envelope-open', 'fas fa-envelope-square', 'fas fa-eraser', 'fas fa-fax', 'fas fa-file', 'far fa-file', 'fas fa-file-alt', 'far fa-file-alt', 'fas fa-folder', 'far fa-folder', 'fas fa-folder-minus', 'fas fa-folder-open', 'far fa-folder-open', 'fas fa-folder-plus', 'fas fa-glasses', 'fas fa-globe', 'fas fa-highlighter', 'fas fa-industry', 'fas fa-landmark', 'fas fa-marker', 'fas fa-paperclip', 'fas fa-paste', 'fas fa-pen', 'fas fa-pen-alt', 'fas fa-pen-fancy', 'fas fa-pen-nib', 'fas fa-pen-square', 'fas fa-pencil-alt', 'fas fa-percent', 'fas fa-phone', 'fas fa-phone-slash', 'fas fa-phone-square', 'fas fa-phone-volume', 'fas fa-print', 'fas fa-project-diagram', 'fas fa-registered', 'far fa-registered', 'fas fa-save', 'far fa-save', 'fas fa-sitemap', 'fas fa-socks', 'fas fa-sticky-note', 'far fa-sticky-note', 'fas fa-stream', 'fas fa-table', 'fas fa-tag', 'fas fa-tags', 'fas fa-tasks', 'fas fa-thumbtack', 'fas fa-trademark', 'fas fa-wallet'),
		'charity' => array('fas fa-dollar-sign', 'fas fa-donate', 'fas fa-dove', 'fas fa-gift', 'fas fa-globe', 'fas fa-hand-holding-heart', 'fas fa-hand-holding-usd', 'fas fa-hands-helping', 'fas fa-handshake', 'far fa-handshake', 'fas fa-heart', 'far fa-heart', 'fas fa-leaf', 'fas fa-parachute-box', 'fas fa-piggy-bank', 'fas fa-ribbon', 'fas fa-seedling'),
		'chat' => array('fas fa-comment', 'far fa-comment', 'fas fa-comment-alt', 'far fa-comment-alt', 'fas fa-comment-dots', 'far fa-comment-dots', 'fas fa-comment-slash', 'fas fa-comments', 'far fa-comments', 'fas fa-frown', 'far fa-frown', 'fas fa-meh', 'far fa-meh', 'fas fa-phone', 'fas fa-phone-slash', 'fas fa-poo', 'fas fa-quote-left', 'fas fa-quote-right', 'fas fa-smile', 'far fa-smile', 'fas fa-video', 'fas fa-video-slash'),
		'code' => array('fas fa-archive', 'fas fa-barcode', 'fas fa-bath', 'fas fa-bug', 'fas fa-code', 'fas fa-code-branch', 'fas fa-coffee', 'fas fa-file', 'far fa-file', 'fas fa-file-alt', 'far fa-file-alt', 'fas fa-file-code', 'far fa-file-code', 'fas fa-filter', 'fas fa-fire-extinguisher', 'fas fa-folder', 'far fa-folder', 'fas fa-folder-open', 'far fa-folder-open', 'fas fa-keyboard', 'far fa-keyboard', 'fas fa-microchip', 'fas fa-project-diagram', 'fas fa-qrcode', 'fas fa-shield-alt', 'fas fa-sitemap', 'fas fa-stream', 'fas fa-terminal', 'fas fa-user-secret', 'fas fa-window-close', 'far fa-window-close', 'fas fa-window-maximize', 'far fa-window-maximize', 'fas fa-window-minimize', 'far fa-window-minimize', 'fas fa-window-restore', 'far fa-window-restore'),
		'communication' => array('fas fa-address-book', 'far fa-address-book', 'fas fa-address-card', 'far fa-address-card', 'fas fa-american-sign-language-interpreting', 'fas fa-assistive-listening-systems', 'fas fa-at', 'fas fa-bell', 'far fa-bell', 'fas fa-bell-slash', 'far fa-bell-slash', 'fab fa-bluetooth', 'fab fa-bluetooth-b', 'fas fa-broadcast-tower', 'fas fa-bullhorn', 'fas fa-chalkboard', 'fas fa-comment', 'far fa-comment', 'fas fa-comment-alt', 'far fa-comment-alt', 'fas fa-comments', 'far fa-comments', 'fas fa-envelope', 'far fa-envelope', 'fas fa-envelope-open', 'far fa-envelope-open', 'fas fa-envelope-square', 'fas fa-fax', 'fas fa-inbox', 'fas fa-language', 'fas fa-microphone', 'fas fa-microphone-alt', 'fas fa-microphone-alt-slash', 'fas fa-microphone-slash', 'fas fa-mobile', 'fas fa-mobile-alt', 'fas fa-paper-plane', 'far fa-paper-plane', 'fas fa-phone', 'fas fa-phone-slash', 'fas fa-phone-square', 'fas fa-phone-volume', 'fas fa-rss', 'fas fa-rss-square', 'fas fa-tty', 'fas fa-wifi'),
		'computers' => array('fas fa-database', 'fas fa-desktop', 'fas fa-download', 'fas fa-hdd', 'far fa-hdd', 'fas fa-headphones', 'fas fa-keyboard', 'far fa-keyboard', 'fas fa-laptop', 'fas fa-memory', 'fas fa-microchip', 'fas fa-mobile', 'fas fa-mobile-alt', 'fas fa-plug', 'fas fa-power-off', 'fas fa-print', 'fas fa-save', 'far fa-save', 'fas fa-server', 'fas fa-stream', 'fas fa-tablet', 'fas fa-tablet-alt', 'fas fa-tv', 'fas fa-upload')
	);
}

function ib2_disclaimer_template() {
return <<<EOT
<h2>Disclaimer</h2>
<p>The information contained in this website is for general information purposes only. The information is provided by {%BUSINESS_NAME%} and while we endeavour to keep the information up to date and correct, we make no representations or warranties of any kind, express or implied, about the completeness, accuracy, reliability, suitability or availability with respect to the website or the information, products, services, or related graphics contained on the website for any purpose. Any reliance you place on such information is therefore strictly at your own risk.</p>
<p>In no event will we be liable for any loss or damage including without limitation, indirect or consequential loss or damage, or any loss or damage whatsoever arising from loss of data or profits arising out of, or in connection with, the use of this website.</p>
<p>Through this website you are able to link to other websites which are not under the control of {%BUSINESS_NAME%}. We have no control over the nature, content and availability of those sites. The inclusion of any links does not necessarily imply a recommendation or endorse the views expressed within them.</p>
<p>Every effort is made to keep the website up and running smoothly. However, {%BUSINESS_NAME%} takes no responsibility for, and will not be liable for, the website being temporarily unavailable due to technical issues beyond our control.</p>
EOT;
}

function ib2_terms_template() {
return <<<EOT
<h2>Terms Of Service</h2>
<p>Welcome to our website. If you continue to browse and use this website, you are agreeing to comply with and be bound by the following terms and conditions of use, which together with our privacy policy govern {%BUSINESS_NAME%}'s relationship with you in relation to this website. If you disagree with any part of these terms and conditions, please do not use our website.</p>
<p>The term '{%BUSINESS_NAME%}' or 'us' or 'we' refers to the owner of the website. The term 'you' refers to the user or viewer of our website.</p>
<p>The use of this website is subject to the following terms of use:</p>
<ul>
<li>The content of the pages of this website is for your general information and use only. It is subject to change without notice.</li>
<li>Neither we nor any third parties provide any warranty or guarantee as to the accuracy, timeliness, performance, completeness or suitability of the information and materials found or offered on this website for any particular purpose. You acknowledge that such information and materials may contain inaccuracies or errors and we expressly exclude liability for any such inaccuracies or errors to the fullest extent permitted by law.</li>
<li>Your use of any information or materials on this website is entirely at your own risk, for which we shall not be liable. It shall be your own responsibility to ensure that any products, services or information available through this website meet your specific requirements.</li>
<li>This website contains material which is owned by or licensed to us. This material includes, but is not limited to, the design, layout, look, appearance and graphics. Reproduction is prohibited other than in accordance with the copyright notice, which forms part of these terms and conditions.</li>
<li>All trademarks reproduced in this website, which are not the property of, or licensed to the operator, are acknowledged on the website.</li>
<li>Unauthorised use of this website may give rise to a claim for damages and/or be a criminal offence.</li>
<li>From time to time, this website may also include links to other websites. These links are provided for your convenience to provide further information. They do not signify that we endorse the website(s). We have no responsibility for the content of the linked website(s).</li>
<li>Your use of this website and any dispute arising out of such use of the website is subject to the laws of {%COUNTRY%}.</li>
</ul>
EOT;
}

function ib2_policy_template() {
return <<<EOT
<h2>Privacy Policy</h2>
<p>Your privacy is important to us. It is {%BUSINESS_NAME%}'s policy to respect your privacy regarding any information we may collect from you across our website, <a href="{%URL%}">{%URL%}</a>, and other sites we own and operate.</p>
<p>We only ask for personal information when we truly need it to provide a service to you. We collect it by fair and lawful means, with your knowledge and consent. We also let you know why we're collecting it and how it will be used.</p>
<p>We only retain collected information for as long as necessary to provide you with your requested service. What data we store, we'll protect within commercially acceptable means to prevent loss and theft, as well as unauthorised access, disclosure, copying, use or modification.</p>
<p>We don't share any personally identifying information publicly or with third-parties, except when required to by law.</p>
<p>Our website may link to external sites that are not operated by us. Please be aware that we have no control over the content and practices of these sites, and cannot accept responsibility or liability for their respective privacy policies.</p>
<p>You are free to refuse our request for your personal information, with the understanding that we may be unable to provide you with some of your desired services.</p>
<p>Your continued use of our website will be regarded as acceptance of our practices around privacy and personal information. If you have any questions about how we handle user data and personal information, feel free to contact us.</p>
EOT;
}
