(function (PrimeBuilder, w, d, $) { 
	"use strict";

	/**
	 * Element Action
	 */
	PrimeBuilder.Action = function (prime) {
		this.prime = prime;
		this.txtEd = null;
	};

	PrimeBuilder.Action.prototype = {
		buttons: function(action, btn, event) {
			switch (action) {
				case 'wysiwyg':
					this.textEditor();
					break;

				case 'clone':
					this.clone();
					break;

				case 'delete':
					this.delete();
					break;

				case 'manageform':
					this.manageform();
					break;

				case 'addcolumn':
					this.addcolumn();
					break;

				case 'removecolumn':
					this.removecolumn();
					break;

				case 'editpopup':
					this.editpopup(btn);
					break;

				case 'nextslide':
				case 'prevslide':
				case 'nextquestion':
				case 'prevquestion':
					this.movestep(btn, action);
					break;

				case 'countdownnormal':
				case 'countdownhidden':
					this.switchcountdown(btn, action);
					break;

				case 'addtab':
					this.addtab();
					break;

				case 'removetab':
					this.removetab();
					break;

				case 'addimg':
					this.addimgslide();
					break;

				case 'removeimg':
					this.removeimgslide();
					break;

				case 'addsection':
					this.addsection(btn);
					break;

				case 'template':
					this.changeTemplate(btn);
					break;

				case 'savetemplate':
					this.saveTemplate(btn);
					break;

				case 'saveelem':
					this.saveElement(btn);
					break;

				case 'apiconnect':
					this.apiConnect(btn);
					break;

				case 'legal':
					this.legal();
					break;

				case 'switcheditor':
					this.switcheditor(btn);
					break;

				case 'moveup':
					this.moveUp();
					break;

				case 'movedown':
					this.moveDown();
					break;

				case 'addelement':
					this.addelement(btn);
					break;

				case 'ib2media':
					this.ib2media(btn);
					break;

				case 'addicon':
					this.addicon();
					break;

				case 'removeicon':
					this.removeicon(btn);
					break;

				case 'deletevariant':
					this.deletevariant(btn);
					break;

				case 'visibility':
					this.visibility();
					break;

				case 'alignposleft':
					this.alignpos('left');
					break;

				case 'alignposright':
					this.alignpos('right');
					break;

				case 'alignposcenter':
					this.alignpos('center');
					break;

				case 'permalink':
					this.permalink();
					break;

				case 'restorehistory':
					this.restorehistory();
					break;

				case 'refreshlist':
					this.refreshList(btn);
					break;

				default:
					console.log(action);
			}
		},

		refreshList: function(btn) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				provider = $('#esp-esp-integration-form').val(),
				fields = {
					provider: provider
				};

			btn.button('loading');
			$.ajax({
				type: "POST",
				url: ajaxurl,
				data: {
					action: 'ib2_connect_api',
					fields: fields,
					refresh: true
				},
				success: function(response) {
					if ( response.success ) {
						gui.updateListOptions(response.lists);
						prime.esplists[provider] = response.lists;
					}
					btn.button('reset');
				},
				error: function(jqXHR, textStatus, errorThrown) {
					btn.button('reset');
				}
			});
		},

		restorehistory: function() {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				restored = prime.restored;

			if ( confirm("Are you sure you want to restore previously saved data?\nAll unsaved changes will be lost.") ) {
				$('.loader-txt').text('Restoring Data...');
				$('#editor-loader').removeClass('hidden');
				$('#editor-container').addClass('hidden');
				$.ajax({
					type: "GET",
					url: ajaxurl,
					data: {
						action: 'ib2_restore_history',
						post_id: $('#ib2-post-id').val(),
						variant_id: $('#ib2-variant-id').val(),
						restored: restored
					},
					success: function(response) {
						$('.loader-txt').text('Loading...');
						$('#editor-loader').addClass('hidden');
						$('#editor-container').removeClass('hidden');
						if ( response.success ) {
							$('body').trigger('restore-history', [response.data]);
						} else {
							alert('Cannot find previous history.');
						}
						cache.activeElement = null;
						cache.activeElementType = null;
						cache.hoveredEl = null;

						// remove is-selected class
						iframeDom.find('.is-selected').removeClass('is-selected');
						prime.restored = true;
					}
				});
			}
		},

		alignpos: function(align) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elem = cache.hoveredEl,
				elID = elem.attr('id'),
				type = elem.attr('data-el');

			var flexAlign = 'flex-start';
			if ( type == 'share' ) {
				if ( align == 'right' ) {
					flexAlign = 'flex-end';
				} else if ( align == 'center' ) {
					flexAlign = 'center';
				}
				elem.find('> .el-content').css('justifyContent', flexAlign);
			} else {
				elem.css('textAlign', align);
			}

			// clean highlight
			$('.el-highlight').hide();
			cache.hoveredEl = null;

			// resize selector
			gui.resizeSelector();
			// resize backgrounds
			gui.resizeBackgrounds();
		},

		deletevariant: function(btn) {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				currentVar = prime.currentVariation,
				letter = btn.attr('data-variant'),
				post_id = $('#ib2-post-id').val(),
				tr = btn.closest('tr');

			if ( btn.hasClass('disabled') ) {
				return;
			}

			if ( confirm('Are you sure you want to delete variant ' + prime.utils.ucwords(letter)) ) {
				tr.find('.variant-weights').attr('disabled', true);
				btn.addClass('disabled');
				$.ajax({
					type: "POST",
					url: ajaxurl,
					data: {
						action: 'ib2_delete_variant',
						variant_id: $('#ib2-variant-id').val(),
						post_id: post_id
					},
					success: function(response) {
						if ( response.success ) {
							tr.remove();
							if ( letter == currentVar ) {
								// redirect
								var url = $('#ib2-editor-url').val();
								window.location.href = url;
							}
						} else {
							alert('ERROR: Failed to delete variant.');
							tr.find('.variant-weights').removeAttr('disabled');
							btn.removeClass('disabled');
						}
					}
				});
			}	
		},

		permalink: function() {
			var self = this,
				prime = this.prime,
				post = prime.postData,
				cache = prime.cache,
				gui = prime.gui,
				content = '';

			this.showPopup('change-permalink', {
				title: 'Permalink',
				wide: false,
			});

			var slug = post.post_name,
				permalink = (post.status == 'publish' ) ? post.permalink : post.permalink.replace("{postname}", slug);

			content += '<p class="current-permalink"><a href="' + permalink + '" class="ib2-permalink" target="_blank">' + permalink + '</a></p>';
			content += '<a href="#" role="button" class="btn btn-xs btn-default change-permalink">Change Permalink</a>';
			content += '<form id="permalink-form" style="display: none">';
				content += '<input type="hidden" id="ib2-current-slug" value="' + post.post_name + '">';
				content += '<div class="form-group">';
					content += '<div class="input-group">';
						content += '<span id="base-permalink" class="input-group-addon">' + post.base_permalink + '</span>';
	  					content += '<input type="text" class="form-control" id="ib2-new-slug" value="' + post.post_name + '">';
	  					content += '<span class="input-group-btn">';
	    					content += '<button class="btn btn-default" type="submit">OK</button>';
	  					content += '</span>';
					content += '</div>';
				content += '</div>';
			content += '</form>';

			$('#change-permalink').find('.settings-popup-content').html(content);

			this.centerPopup('change-permalink');
		},

		removeicon: function(btn) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				fieldID = btn.attr('data-id'),
				elID = cache.activeElement,
				type = cache.activeElementType;

			if ( fieldID ) {
				$('#' + fieldID).val('');
				$('#icon-shape-icon-button').parent().find('.icon-preview').find('i').removeAttr('class');
				iframeDom.find('#' + elID).find('.ib2-button').find('i').remove();
				cache.contents[type][elID]['settings']['buttonicon']['iconclass'] = '';
				btn.hide();
			}
		},

		addicon: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				icons = prime.faIcons,
				lastcat = cache.lastIcons || 'accessibility',
				content = '';

			this.showPopup('icon-manager', {
				title: 'Font Awesome Icons',
				wide: false,
				overlay: false
			});

			content += '<hr>';
				content += '<form id="ib2-icon-form" method="get" class="form-inline">';
					content += '<div class="form-group">';
						content += '<select id="ib2-icon-cat" class="form-control">';

						for ( var key in icons ) {
							var selected = lastcat == key ? ' selected="selected"' : '';
							var option = ( key == 'audiovideo' ? 'Audio & Video' : prime.utils.ucwords(key) );
							content += '<option value="' + key + '"' + selected + '>' + option + '</option>';
						}
						content += '</select>';
					content += '</div>';
				content += '</form>';
			content += '<hr>';

			content += '<p class="help-block media-help">Click on icon to insert</p>';
			content += '<div class="insert-elements">';

			icons[lastcat].forEach(function(icon){
				content += '<div class="insert-element">';
					content += '<a href="javascript:void(0)" class="insert-icon-btn" data-icon="' + icon + '">';
					content += '<i class="' + icon + '"></i>';
				content += '</a></div>';
			}, this);
			content += '<div class="clearfix"></div>';
			content += '</div>';

			$('#icon-manager').find('.settings-popup-content').html(content);

			this.centerPopup('icon-manager');
		},

		ib2media: function(btn) {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				cats = {
					arrows: 'Arrows',
					backgrounds: 'Backgrounds',
					badges: 'Badges',
					buttons: 'Buttons',
					buyarea: 'Buy Now Area',
					creditcards: 'Credit Cards',
					icons: 'Icons',
					texts: 'Texts',
					pixabay: 'Pixabay'
				},
				pid = btn.attr('data-id'),
				settingFor = btn.attr('data-for'),
				idx = btn.attr('data-idx'),
				content = '';

			this.showPopup('media-manager', {
				title: 'Browse/Search Image',
				wide: true
			});

			content += '<hr>';
				content += '<form id="ib2-media-form" method="get" class="form-inline">';
					content += '<div class="form-group">';
						content += '<select id="ib2-media-cat" class="form-control" data-id="' + pid + '" data-for="' + settingFor + '" data-idx="' + idx + '">';

						for ( var key in cats ) {
							content += '<option value="' + key + '">' + cats[key] + '</option>';
						}
						content += '</select>';
					content += '</div>';
					content += '<div class="form-group search-image-form hidden">';
						content += '&nbsp;<input type="text" id="media-keyword" class="form-control" placeholder="e.g. enter keywords here" />&nbsp;';
					content += '</div>';
					content += '<button type="submit" id="search-image-btn" class="btn btn-defaults search-image-form hidden" data-loading-text="Searching..." autocomplete="off">Go</button>';
				content += '</form>';
			content += '<hr>';

			content += '<div class="fa-3x text-center media-loader"><i class="fas fa-spinner fa-spin"></i></div>';
			content += '<p class="help-block media-help hidden">Click on image to insert</p>';
			content += '<div class="ib2-media-images flex-images"></div>';

			$('#media-manager').find('.settings-popup-content').html(content);

			this.centerPopup('media-manager');

			if ( !cache.mediaResults.hasOwnProperty($('#ib2-media-cat').val()) ) {
				$.ajax({
					type: "GET",
					url: ajaxurl,
					data: {
						action: 'ib2_get_media',
						mode: 'settings',
						cat: $('#ib2-media-cat').val(),
					},
					success: function(response) {
						cache.mediaResults[$('#ib2-media-cat').val()] = response;
						$('body').trigger("fillmedia", [response, settingFor, idx, pid]);
					}
				});
			} else {
				$('body').trigger("fillmedia", [cache.mediaResults[$('#ib2-media-cat').val()], settingFor, idx, pid]);
			}
		},

		addelement: function(btn) {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				menu = prime.getHtml('elementsMenu'),
				gui = prime.gui,
				content = '',
				pos = ( btn.parent().hasClass('adder-top') ? 'before' : 'after' );

			this.showPopup('element-manager', {
				title: 'Add Element',
				wide: true,
				overlay: false
			});

			content += menu;
			content += '<hr>';
			content += '<h4 class="element-cat-title">Basic Elements</h4>';
			content += '<p class="help-block">Click to add element</p>';

			content += '<div class="element-contents">';
				content += '<div class="basic-elements element-pane active">';
					content += '<div class="insert-elements">';
					prime.elements.forEach(function(element, i){
						var type = element.hasOwnProperty('alias') ? element.alias : element.type;

						if (type != 'wsection' && type != 'page' && type != 'section' && type != 'wbox') {
							content += '<div class="insert-element"><a href="javascript:void(0)" class="insert-element-btn" title="' + element.title + '" data-el="' + type + '" data-pos="' + pos + '">';
								if ( element.hasOwnProperty('textIcon') && element.textIcon != '' ) {
									content += element.textIcon;
								} else {
									content += '<i class="' + element.icon + '"></i>';
								}
							content += '</a></div>';
						}
					});
					content += '<div class="clearfix"></div>';
					content += '</div>';
				content += '</div>';
				//content += '<div class="combo-elements element-pane">Combo elements here</div>';
				content += '<div class="saved-elements element-pane">';
					content += '<div class="ib2-elements-area">';
						content += '<p style="margin-top:40px; text-align:center" class="ib2-element-loader">';
							content += '<i class="fas fa-cog fa-spin fa-3x"></i><br />';
							content += '<em>Loading...</em>';
						content += '</p>';
						content += '<div class="ib2-elements-content" data-pos="' + pos + '"></div>';
					content += '</div>';
				content += '</div>';
			content += '</div>';
			$('#element-manager').find('.settings-popup-content').html(content);

			this.centerPopup('element-manager');
		},

		visibility: function() {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				type = cache.activeElementType,
				elem = iframeDom.find('#' + elID),
				content = '';

			this.showPopup('visibility', {
				title: 'Element Visibility',
				wide: false,
				overlay: false
			});

			var d_checked = elem.hasClass('hidden-lg') && elem.hasClass('hidden-md') ? '' : ' checked="checked"',
				t_checked = elem.hasClass('hidden-sm') ? '' : ' checked="checked"',
				m_checked = elem.hasClass('hidden-xs') ? '' : ' checked="checked"';

			content += '<div class="alert alert-danger alert-dismissible hidden" id="visibility-alert" role="alert">';
  				content += '<button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>';
  				content += 'ERROR: Element must be visible on at least one device.';
			content += '</div>';
			content += '<form id="visibility-form">';
				content += '<div class="checkbox">';
    				content += '<label>';
      					content += '<input type="checkbox" id="desktop-view" value="1"' + d_checked + '> Display Element on Desktop';
    				content += '</label>';
  				content += '</div>';
  				content += '<div class="checkbox">';
    				content += '<label>';
      					content += '<input type="checkbox" id="tablet-view" value="1"' + t_checked + '> Display Element on Tablet';
    				content += '</label>';
  				content += '</div>';
  				content += '<div class="checkbox">';
    				content += '<label>';
      					content += '<input type="checkbox" id="mobile-view" value="1"' + m_checked + '> Display Element on Mobile';
    				content += '</label>';
  				content += '</div>';
			content += '</form>';

			$('#visibility').find('.settings-popup-content').html(content);

			this.centerPopup('visibility');

		},

		moveUp: function() {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				type = cache.activeElementType,
				elem = cache.hoveredEl ? cache.hoveredEl : iframeDom.find('#' + elID),
				prev = elem.prev(),
				canMove = true,
				parent;

			// is this the top element ?
			if ( elem.parent().attr('id') == 'screen-container' && !prev.length ) {
				canMove = false;
			}

			if ( !prev.length ) {
				parent = elem.parent();
				if ( !parent.hasClass('ib2-element') ) {
					parent = parent.closest('.ib2-element');
				}

				if ( parent.length && parent.parent().attr('id') == 'screen-container' ) {
					canMove = false;
				} 
			}

			if ( canMove ) {
				if ( prev.length > 0 ) {
					prev.before(elem);
				} else if ( !prev.length && parent.length > 0 ) {
					parent.before(elem);
				}
			}

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			setTimeout(function(){
				// re-calculate layers
				prime.saveLayers();
				// resize selector
				gui.resizeSelector();
				// resize backgrounds
				gui.resizeBackgrounds();
				// save history
				prime.saveHistory();
			}, 500);
		},

		moveDown: function() {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				type = cache.activeElementType,
				elem = cache.hoveredEl ? cache.hoveredEl : iframeDom.find('#' + elID),
				next = elem.next(),
				canMove = true,
				parent;

			// is this the bottom element ?
			if ( elem.parent().attr('id') == 'screen-container' && !next.length ) {
				canMove = false;
			}

			if ( !next.length ) {
				parent = elem.parent();
				if ( !parent.hasClass('ib2-element') ) {
					parent = parent.closest('.ib2-element');
				}

				if ( parent.length && parent.parent().attr('id') == 'screen-container' ) {
					canMove = false;
				}
			}

			if ( canMove ) {
				if ( next.length > 0 ) {
					next.after(elem);
				} else if ( !next.length && parent.length > 0 ) {
					parent.after(elem);
				}
			}

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			setTimeout(function(){
				// re-calculate layers
				prime.saveLayers();
				// resize selector
				gui.resizeSelector();
				// resize backgrounds
				gui.resizeBackgrounds();
				// save history
				prime.saveHistory();
			}, 500);
		},

		switcheditor: function(btn) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody,
				mode = btn.attr('data-mode');

			if ( mode != cache.editorMode ) {
				btn.closest('ul').find('.active-edit-mode').removeClass('active-edit-mode');
				btn.addClass('active-edit-mode');

				$('#main-editor').removeClass('mobile-edit-mode tablet-edit-mode');
				if ( mode == 'desktop' ) {
					window.FrameWindow.txtMode = 'desktop';
					if ( $('#editor-settings').is(":visible") ) {
						$('#main-editor').addClass('edit-settings-on');
					}
					iframeDom.find('#screen-container').addClass('desktop-screen');
					iframeDom.find('#screen-container').addClass('elements-bar-on');
					iframeDom.find('#elements-bar').show();
				} else {
					window.FrameWindow.txtMode = 'mobile';
					$('#main-editor').removeClass('edit-settings-on');
					$('#main-editor').addClass(mode + '-edit-mode');
					iframeDom.find('#screen-container').removeClass('desktop-screen');
					iframeDom.find('#screen-container').removeClass('elements-bar-on');
					iframeDom.find('#elements-bar').hide();
				}
				cache.editorMode = mode;
			}

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			setTimeout(function(){
				// re-calculate layers
				prime.saveLayers();
				// resize selector
				gui.resizeSelector();
				// resize backgrounds
				gui.resizeBackgrounds();
			}, 500);
		},

		apiConnect: function(btn) {
			var prime = this.prime,
				cache = prime.cache,
				elID = cache.activeElement,
				type = cache.activeElementType,
				fieldID = btn.attr('data-id'),
				provider = $('#' + fieldID).val(),
				content = $('#ib2-api-connect-html').html();

			this.showPopup('api-integration', {
				title: 'API Integration Form',
				wide: false
			});

			$('#api-integration').find('.settings-popup-content').html(content);
			$('#api-provider').val(provider);
			$('#save-api-form').find('.api-field').hide();
			$('#save-api-form').find('.api-' + provider).show();
			this.centerPopup('api-integration');
			cache.popupKeepEl = true;
		},

		legal: function() {
			var prime = this.prime,
				cache = prime.cache,
				elID = cache.activeElement,
				type = cache.activeElementType,
				content = $('#ib2-legal-generator-html').html();

			this.showPopup('legal-generator', {
				title: 'Legal Content Generator',
				wide: false
			});

			$('#legal-generator').find('.settings-popup-content').html(content);
			this.centerPopup('legal-generator');
			cache.popupKeepEl = true;
		},

		saveElement: function() {
			var prime = this.prime,
				cache = prime.cache,
				elID = cache.activeElement,
				type = cache.activeElementType,
				content = $('#ib2-save-element-html').html();

			this.showPopup('save-element', {
				title: 'Save Element',
				wide: false
			});

			$('#save-element').find('.settings-popup-content').html(content);
			$('#save-element-id').val(elID);

			this.centerPopup('save-element');
		},

		saveTemplate: function(btn) {
			var prime = this.prime,
				cache = prime.cache,
				isVariant = btn.attr('data-type') && btn.attr('data-type') == 'variant' ? true : false,
				content = $('#ib2-save-template-html').html();

			this.showPopup('save-template', {
				title: 'Save as Template',
				wide: false
			});

			$('#save-template').find('.settings-popup-content').html(content);

			this.centerPopup('save-template');
		},

		changeTemplate: function(btn) {
			var prime = this.prime,
				cache = prime.cache,
				isVariant = btn.attr('data-type') && btn.attr('data-type') == 'variant' ? true : false,
				content = $('#ib2-templates-html').html(),
				title = isVariant ? 'Create New Variant' : 'Change Template';

			this.showPopup('template-manager', {
				title: title,
				wide: true
			});

			$('#template-manager').find('.settings-popup-content').html(content);
			if ( isVariant ) {
				cache.changeTemplate = false;
				$('#template-manager').find('.change-template-alert').hide();
				$('#template-manager').find('.new-variant-options').show();
				$('#template-manager').find('.ib2-tmpls').hide();
			} else {
				cache.changeTemplate = true;
				$('#template-manager').find('.change-template-alert').show();
				$('#template-manager').find('.new-variant-options').hide();
				$('#template-manager').find('.ib2-tmpls').show();
			}

			this.centerPopup('template-manager');

			// load templates
			prime.loadTemplates('sales', 'Sales Page');
		},

		addsection: function(btn) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elID = cache.hoveredEl.attr('id'),
				type = cache.hoveredEl.attr('data-el'),
				pos = ( btn.parent().hasClass('adder-top') ? 'before' : 'after' ),
				elem = iframeDom.find('.element-type-' + type);

			cache.isAdding = true;
			cache.isDragging = true;

			prime.addElement(iframeDom.find('#' + elID), elem, pos);
		},

		addimgslide: function() {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elID = cache.activeElement,
				type = cache.activeElementType,
				iframeDom = prime.frameBody,
				total = iframeDom.find('#' + elID).find('.carousel-indicators > li').length,
				nextNum = total + 1,
				defaultImg = $('#ib2-slides-src').attr('src'),
				slideId = iframeDom.find('#' + elID).find('.carousel').attr('id'),
				settings = cache.contents[type][elID]['settings']['slides']['slides'],
				html = '';

			iframeDom.find('#' + elID).find('.carousel-indicators').append('<li data-target="#' + slideId + '" data-slide-to="' + total + '"></li>');

			html += '<div class="item">';
			html += '<img src="' + defaultImg + '" alt="Image Slide ' + nextNum + '">';
  			html += '<div class="carousel-caption">';
    		html += '<h3>Image Slide ' + nextNum + '</h3>';
  			html += '</div>';
			html += '</div>';

			iframeDom.find('#' + elID).find('.carousel-inner').append(html);

			settings.push({ title: 'Image Slide ' + nextNum, imageurl: defaultImg, desturl: '' });

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			// re-calculate layers
			prime.saveLayers();
			// resize selector
			gui.resizeSelector();
			// resize backgrounds
			gui.resizeBackgrounds();
		},

		removeimgslide: function() {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				gui = prime.gui,
				elID = cache.activeElement,
				type = cache.activeElementType,
				total = iframeDom.find('#' + elID).find('.carousel-indicators > li').length,
				settings = cache.contents[type][elID]['settings']['slides']['slides'];

			if ( total <= 2 ) {
				alert("Cannot remove the last image slide. You must have at least one slide.");
			} else {
				iframeDom.find('#' + elID).find('.carousel-indicators > li:last-child').remove();
				iframeDom.find('#' + elID).find('.carousel-inner > div:last-child').remove();
			}

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			settings.splice(-1,1);

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			// re-calculate layers
			prime.saveLayers();
			// resize selector
			gui.resizeSelector();
			// resize backgrounds
			gui.resizeBackgrounds();
		},

		addtab: function() {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elID = cache.activeElement,
				type = cache.activeElementType,
				iframeDom = prime.frameBody,
				total = iframeDom.find('#' + elID).find('> ul.ib2-nav-tabs').find('.ib2-tab-title').length,
				newNum = total + 1,
				newID = prime.generateID(9, 'ib2_el_'),
				settings = cache.contents[type][elID]['settings']['tabs']['tabs'];

			iframeDom.find('#' + elID)
				.find('> ul.ib2-nav-tabs')
				.append('<li><a href="#' + elID + '-' + total + '" role="tab"><span class="ib2-tab-title">Tab #' + newNum + '</span></a></li>');

			var tab = '<div class="ib2-tab-pane ib2-section-content" id="' + elID + '-' + total + '">\
							<div id="{id}" class="ib2-content-el ib2-text-el ib2-element is-element" data-el="text">\
								<p>This is your new text content. You can modify this text, add more paragraph, change font style or add images by clicking the edit button.</p>\
							</div>\
						</div>';

			tab = tab.replace(new RegExp('{id}', 'gi'), newID);

			iframeDom.find('#' + elID)
				.find('> .ib2-tab-content')
				.append(tab);

			settings.push({ title: 'Tab #' + newNum });

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			// sortable
			prime.disableSortable();
			prime.sortable();
			// re-calculate layers
			prime.saveLayers();
			// resize selector
			gui.resizeSelector();
			// resize backgrounds
			gui.resizeBackgrounds();
		},

		removetab: function() {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				gui = prime.gui,
				elID = cache.activeElement,
				type = cache.activeElementType,
				total = iframeDom.find('#' + elID).find('> ul.ib2-nav-tabs').find('.ib2-tab-title').length,
				settings = cache.contents[type][elID]['settings']['tabs']['tabs'],
				lastTab = iframeDom.find('#' + elID).find('> ul.ib2-nav-tabs > li:last-child'),
				lastPane = iframeDom.find('#' + elID).find('> .ib2-tab-content > div:last-child'),
				moveActive = false;

			if ( total <= 1 ) {
				alert("Cannot remove the last tab. You must have at least one tab.");
			} else {
				if ( lastPane.hasClass('ib2-tabs-active') ) {
					moveActive = true;
				}
				lastTab.remove();
				lastPane.remove();
			}

			if ( moveActive ) {
				iframeDom.find('#' + elID).find('> ul.ib2-nav-tabs > li:last-child').addClass('ib2-tabs-active');
				iframeDom.find('#' + elID).find('> .ib2-tab-content > div:last-child').addClass('ib2-tabs-active');
			}

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			settings.splice(-1,1);

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			// re-calculate layers
			prime.saveLayers();
			// resize selector
			gui.resizeSelector();
			// resize backgrounds
			gui.resizeBackgrounds();

		},

		switchcountdown: function(btn, action) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elID = cache.activeElement,
				type = cache.activeElementType,
				iframeDom = prime.frameBody;

			if ( action == 'countdownhidden' ) {
				iframeDom.find('#' + elID).find('> div, > span').each(function(){
					if ( !$(this).hasClass('ib2-countdown-reveal') ) {
						$(this).hide();
					}
				});
				iframeDom.find('#' + elID).find('.ib2-countdown-reveal').show();
			} else {
				iframeDom.find('#' + elID).find('> div, > span').each(function(){
					if ( !$(this).hasClass('ib2-countdown-reveal') && !$(this).is(":visible") ) {
						$(this).show();
					}
				});
				iframeDom.find('#' + elID).find('.ib2-countdown-reveal').hide();
			}

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			// resize selector
			gui.resizeSelector();
			// resize backgrounds
			gui.resizeBackgrounds();
		},

		movestep: function(btn, action) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elID = cache.activeElement,
				type = cache.activeElementType,
				iframeDom = prime.frameBody,
				curStep = 1, newStep,
				shouldMove = true,
				totalStep = 1;

			if ( $.inArray(action, ['prevslide', 'nextslide']) > -1 ) {
				curStep = parseInt(iframeDom.find('#' + elID).attr('data-current-slide'));
				newStep = (action == 'prevslide' ? curStep-1 : curStep+1 );
				if ( curStep == 3 && action == 'nextslide' ) {
					newStep = 1;
				}

				if ( curStep == 1 && action == 'prevslide' ) {
					newStep = 3;
				}

				iframeDom.find('#' + elID).find('.ib2-section-slide-' + curStep).css('height', '0px').removeClass('ib2-slide-active').hide();
				iframeDom.find('#' + elID).find('.ib2-section-slide-' + newStep).css('height', 'auto').addClass('ib2-slide-active').show();
				iframeDom.find('#' + elID).attr('data-current-slide', newStep);
				
			} else if ( $.inArray(action, ['prevquestion', 'nextquestion']) > -1 ) {
				var lastQ = 0;
				iframeDom.find('#' + elID).find('.ib2-quiz-page').each(function(i){
					var $this = $(this);
					if ( $this.is(":visible") ) {
						curStep = $this.attr('data-question') ? parseInt($this.attr('data-question')) : 6;
					}
					
					if ( !$this.hasClass('ib2-unused-question') && curStep < 6 ) {
						lastQ++;
					}
					$this.hide();
				});
				
				newStep = (action == 'prevquestion' ? curStep-1 : curStep+1);
				if ( curStep == 1 && action == 'prevquestion' ) {
					newStep = 6;
				}
				if ( curStep == 6 && action == 'nextquestion' ) {
					newStep = 1;
				}

				if ( curStep == 6 && action == 'prevquestion' ) {
					newStep = lastQ;
				}

				var nextID = elID + '-' + newStep;
				if ( (action == 'nextquestion' && iframeDom.find('#' + nextID).hasClass('ib2-unused-question')) || newStep == 6 ) {
					nextID = elID + '-result';
				}

				iframeDom.find('#' + nextID).show();
			}

			// resize selector
			gui.resizeSelector();
			// resize backgrounds
			gui.resizeBackgrounds();
		},

		newPopup: function() {
			var prime = this.prime,
				cache = prime.cache,
				elID = cache.activeElement,
				type = cache.activeElementType,
				iframeDom = prime.frameBody,
				html = $('#ib2-element-popup-html').html(),
				popID = prime.generateID(10, 'ib2_el_') + '-popup',
				imgbtn = $('#ib2-submit-img').attr('src');

			html = html.replace(new RegExp('{id}', 'gi'), popID);
			html = html.replace(new RegExp('{imgbtn}', 'gi'), imgbtn);

			var helper = $(html);
			helper.find('.ib2-element').each(function(i){
				var $this = $(this),
					num = i+1,
					newID = prime.generateID(9, 'ib2_el_'),
					pattern = '{id' + num + '}',
					idRegex = new RegExp(pattern, 'gi');

				html = html.replace(idRegex, newID);
			});

			// append pop up
			iframeDom.find('#screen-container').append(html);

			// save element data
			iframeDom.find('#' + popID).find('.ib2-element').each(function(){
				var $this = $(this),
					id = $this.attr('id'),
					type = $this.attr('data-el');

				prime.newElementData(id, type);
			});

			// save popup data
			prime.newElementData(popID, 'section');

			// save layers
			prime.saveLayers();

			if ( type == 'page' ) {
				cache.contents[type][elID]['settings']['popup']['popid'] = popID;
			} else {
				cache.contents[type][elID]['settings']['buttonaction']['popup'] = popID;
				iframeDom.find('#' + elID).find('a').attr('data-popup', popID);
			}
			return popID;
		},

		editpopup: function(btn) {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				type = cache.activeElementType,
				iframeDom = prime.frameBody,
				isGlobal = btn.attr('data-id') == 'select-popup-edit-page' ? true : false,
				isSlider = btn.attr('data-mode') && btn.attr('data-mode') == 'slider' ? true : false,
				settings = isGlobal ? cache.contents['page']['screen-container']['settings'] : cache.contents[type][elID]['settings'],
				datakey = 'buttonaction',
				data = settings[datakey],
				popupId = isSlider ? 'ib2-bottom-slider-main' : ( isGlobal ? settings['popup']['popid'] : data.popup ),
				winHeight = $(window).height();

			if ( cache.popupMode ) {
				alert('Cannot add/edit another pop up.');
				return;
			}

			if ( !isSlider && btn.attr('data-mode') && btn.attr('data-mode') == 'new' ) {
				popupId = this.newPopup();
			}

			cache.popupMode = true;
			if ( isSlider ) {
				cache.sliderEdit = popupId;
			} else {
				cache.popupId = popupId;
			}

			if ( isGlobal ) {
				cache.activeElement = null;
				cache.activeElementType = null;
				cache.hoveredEl = null;
			}

			// disable sortable
			prime.disableSortable();
			
			// hide normal elements
			iframeDom.find('#screen-container').find('.ib2-section-el').each(function(){
				if ( !$(this).hasClass('ib2-popup') ) {
					$(this).hide();
				}
			});

			// hide wide box
			iframeDom.find('#screen-container').find('.ib2-wide-box').each(function(){
				$(this).hide();
			});

			if ( !iframeDom.find('#screen-container').find('.popup-edit-bg').length ) {
				iframeDom.find('#screen-container').append('<div class="popup-edit-bg"></div>');
			}

			// hide unnecessary settings
			$('.main-editor-btn').hide();
			$('.main-editor-prop').addClass('hidden');
			$('.global-settings-tab').find('.tab-settings-main').removeClass('tab-settings-active');
			$('.publish-page').hide();
			$('#undo-btn').hide();
			$('#redo-btn').hide();
			iframeDom.find('.element-section').hide();

			// hide notification bar
			if ( iframeDom.find('.ib2-notification-bar').length ) {
				iframeDom.find('.ib2-notification-bar').hide();
			}

			// help close picker
			if ( $('.prime-colors').is(":visible") ) {
				prime.picker.pickerClose();
			}

			$('#main-editor').removeClass("edit-settings-on");
			$('#editor-settings').hide();

			// make sure element settings is not hidden
			$('.tab-settings-main').addClass('tab-settings-active');
			$('.active-settings').removeClass('hidden');
			$('.quit-btn').addClass('ib2-close-popup').removeClass('quit-editor-btn');

			// display popup
			iframeDom.find('#' + popupId).show();

			// center popup
			self.centerPopupEdit(iframeDom, popupId, isSlider);
			
			// fix popup class
			iframeDom.find('#' + popupId).find('.ib2-pop-content').addClass('ib2-section-content').css('width', '100%');

			// improve popup close button
			if ( iframeDom.find('#' + popupId).find('.ib2-close-pop').find('img').length ) {
				iframeDom.find('#' + popupId).find('.ib2-close-pop').html('<i class="fas fa-times"></i>');
			}

			// init sortable
			prime.sortable('.ib2-section-content');
		},

		centerPopupEdit: function(iframeDom, popupId, isSlider) {
			var popup = ( isSlider ? iframeDom.find('#' + popupId).find('> .el-content') : iframeDom.find('#' + popupId) );
			var top  = ((iframeDom.find('#screen-container').height() / 2) - (popup.outerHeight() / 2));
			var left = ((iframeDom.find('#screen-container').width() / 2) - (popup.outerWidth() / 2));

			if ( top < 0 ) top = 0;
			if ( left < 0 || isSlider ) left = 0;

			iframeDom.find('#' + popupId).css({
		    	'top' : top,
		    	'left' : left
			});
		},

		closePopupEditor: function(prime, popupId) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody;

			cache.popupMode = false;
			cache.isDragging = true;
			
			if ( cache.sliderEdit ) {
				cache.sliderEdit = false;
			}

			// destroy popup sortable
			prime.disableSortable();

			// clean up
			$('.el-highlight').hide();
			$('.el-selected > .el-resize-handlers').html('');
			$('.el-selected').hide();

			iframeDom.find('.is-selected').removeClass('is-selected');

			// remove pop up settings
			$('#editor-settings > .active-settings > #section-settings').remove();

			// show main editor settings
			$('.main-editor-btn').show();
			$('#undo-btn').show();
			$('#redo-btn').show();
			$('.quit-btn').removeClass('ib2-close-popup').addClass('quit-editor-btn');
			if ( prime.postStatus == 'draft' ) {
				$('.publish-page').show();
			}
			iframeDom.find('.element-section').show();

			// show notification bar
			if ( iframeDom.find('.ib2-notification-bar').length ) {
				iframeDom.find('.ib2-notification-bar').show();
			}

			// remove pop up bg
			iframeDom.find('#screen-container').find('.popup-edit-bg').remove();

			// show wide box
			iframeDom.find('#screen-container').find('.ib2-wide-box').each(function(){
				$(this).show();
			});

			// show normal elements
			iframeDom.find('#screen-container').find('.ib2-section-el').each(function(){
				if ( !$(this).hasClass('ib2-popup') && !$(this).hasClass('ib2-slider-el') ) {
					$(this).show();
				}
			});

			// hide pop up
			iframeDom.find('#' + popupId).hide();

			cache.popupId = null;

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// re-enable sortable
			prime.sortable();

			// resize backgrounds
			gui.resizeBackgrounds();

			// resize selector
			gui.resizeSelector();

			cache.isDragging = false;
		},

		addcolumn: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				type = cache.activeElementType,
				colParent = iframeDom.find('#' + elID).find('.el-cols').eq(0),
				colNum = colParent.find(' > .ib2-section-content').length,
				nextNum = colNum + 1,
				parentWidth = colParent.outerWidth();

			if ( nextNum >= 7 ) {
				alert('You can only add up to 6 columns.');
				return false;
			}

			var newcolId = elID + '-box' + nextNum,
				newWidth = Math.round(parentWidth/nextNum),
				height = colParent.outerHeight(),
				isFlex = ( nextNum % 2 == 0 ? true : false );

			// find heighest col
			if ( colNum > 1 ) {
				var colHeights = [];
				colParent.find(' > .ib2-section-content').each(function(){
					colHeights.push($(this).outerHeight());
				});
				height = Math.max(colHeights);
			}

			if ( newWidth < 100 ) {
				newWidth = 100;
			}

			var siblingCol = colParent.find(' > .ib2-section-content:last-child');
			siblingCol.removeClass('ib2-last-col');

			// adjust width
			colParent.find(' > .ib2-section-content').each(function(i){
				var j = i+1;
				$(this).css('width', newWidth + 'px');
				if ( j % 2 == 0 ) {
					$(this).addClass('ib2-col-resized');
				}
			});

			// append html
			colParent.append('<div id="' + newcolId + '" class="ib2-section-content ib2-section-col ib2-last-col"></div>');
			iframeDom.find('#' + newcolId).css('width', newWidth + 'px');
			if ( isFlex ) {
				iframeDom.find('#' + newcolId).addClass('ib2-col-resized');
			}

			// adjust all column height
			colParent.find(' > .ib2-section-content').css('min-height', height + 'px');

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			// sortable
			prime.disableSortable();
			prime.sortable();
			// re-calculate layers
			prime.saveLayers();
			// resize selector
			gui.resizeSelector();
			// resize backgrounds
			gui.resizeBackgrounds();
		},

		removecolumn: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				type = cache.activeElementType,
				colParent = iframeDom.find('#' + elID).find('.el-cols').eq(0),
				targetCol = colParent.find(' > .ib2-section-content:last-child'),
				colNum = colParent.find(' > .ib2-section-content').length,
				newNum = colNum - 1,
				parentWidth = colParent.get(0).clientWidth;

			if ( colNum == 1 ) {
				alert('There\'s only one column left. You cannot remove any more column.');
				return false;
			}

			var newWidth = Math.round(parentWidth/newNum);

			// remove column
			targetCol.remove();

			// adjust width
			colParent.find(' > .ib2-section-content').each(function(i){
				var j = i+1;
				$(this).css('width', newWidth + 'px');
				if ( j % 2 == 0 ) {
					$(this).addClass('ib2-col-resized');
				}
			});

			// add last col class
			colParent.find(' > .ib2-section-content:last-child').addClass('ib2-last-col');

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			// sortable
			prime.sortable($('.ib2-section-content'));
			// re-calculate layers
			prime.saveLayers();
			// resize selector
			gui.resizeSelector();
			// resize backgrounds
			gui.resizeBackgrounds();
		},

		manageform: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				gui = prime.gui,
				elID = cache.activeElement,
				type = cache.activeElementType,
				data = cache.contents[type][elID],
				setting = data['settings'];

			this.showPopup('form-manager', {
				title: 'Manage Fields',
				wide: true
			});

			// get form data
			var inputs = data.inputs, 
				content = '<form class="form-manager">';

			content += '<table class="form-manager-table table">';
				content += '<thead>';
					content += '<tr>';
						content += '<th><i class="fas fa-arrows-alt-v"></i></th><th>Type</th><th>Field Name/Key</th><th>Text Label</th><th class="text-center">Icon</th><th class="text-center">Required</th><th class="text-center">Hide</th><th>Actions</th>';
					content += '</tr>';
				content += '</thead>';

				content += '<tbody class="form-table-rows">';
					content += this.formTable(inputs);
				content += '</tbody>';

			content += '</table>';
			content += prime.getHtml('fieldForm');
			content += '</form>';

			$('#form-manager').find('.settings-popup-content').html(content);
			$('#form-manager').find('.settings-form-edit').removeClass('hidden');
			$('#form-manager')
				.find('.setting-popup-buttons')
				.html('<button type="button" class="btn btn-success add-new-field"><i class="fa fa-plus"></i> Add New Field</button>&nbsp;<button type="button" class="btn btn-default close-setting-popup">Close</button>');

			this.centerPopup('form-manager');

			$('#form-manager').find('.settings-form-edit').addClass('hidden');

			// sortable
			$('.form-manager-table tbody').sortable({
				update: function(event, ui){
					var newinputs = [],
						origIndex = 0;

					$('.form-manager-table tbody > tr').each(function(f){
						origIndex = parseInt($(this).attr('data-idx'));

						newinputs.push(inputs[origIndex]);
						$(this).attr('data-idx', f);	
					});
					// update input data
					cache.contents[type][elID]['inputs'] = newinputs;

					// rebuild the form
					gui.buildForm(iframeDom.find('#' + elID).find('.form-fields'), cache.contents[type][elID]['inputs'], setting);
				}
			});
		},

		formTable: function(inputs) {
			var content = '',
				imgfolder = $('#ib2-img-folder').val(),
				checked;
			inputs.forEach(function(input, i){
				var iconFile = ( input.icon == 'mail' ? 'env' : input.icon );
				content += '<tr data-idx="' + i + '">';
					content += '<td class="field-sorter"><span class="label label-primary"><i class="fas fa-arrows-alt-v"></i></span></td>';
					content += '<td><span class="label label-warning">' + input.type + '</span></td>';
					content += '<td><code>' + input.name + '</code></td>';
					content += '<td>' + input.placeholder + '</td>';
					content += '<td class="text-center">' + (iconFile != 'none' ? '<img src="' + imgfolder + 'mail-' + iconFile + '-icon.png">' : '<code>none</code>') + '</td>';

					checked = input.required ? 'fas fa-check' : 'fas fa-times';
					content += '<td class="text-center"><i class="' + checked + '"></i></td>';

					checked = input.visible ? 'far fa-eye' : 'fas fa-check';
					content += '<td class="text-center"><i class="' + checked + '"></i></td>';
					content += '<td>\
									<button type="button" class="btn btn-xs btn-default field-edit" title="Edit Field" data-idx="' + i + '"><i class="fa fa-pencil"></i></button>\
									<button type="button" class="btn btn-xs btn-danger field-remove" title="Remove Field" data-idx="' + i + '"><i class="fa fa-trash-o"></i></button>\
								</td>';
				content += '</tr>';
			}, this);
			return content;
		},

		showEditMode: function(id, field, index) {
			var self = this,
				prime = this.prime,
				options = [],
				optionsText = '';

			// switch to regular width
			$('#' + id).removeClass('settings-popup-wide');

			// set required checkbox state
			if ( field.required === true ) {
				$('#' + id).find('.field-required').attr('checked', 'checked');
			} else {
				$('#' + id).find('.field-required').removeAttr('checked');
			}

			// set hide checkbox state
			if ( field.visible !== true ) {
				$('#' + id).find('.field-hide').attr('checked', 'checked');
			} else {
				$('#' + id).find('.field-hide').removeAttr('checked');
			}

			// set and display options
			if ( field.tag == 'select' || field.type == 'radio' ) {
				for ( var key in field.options ) {
					if ( key == field.options[key] ) {
						options.push(field.options[key]);
					} else {
						options.push(key + ':' + field.options[key]);
					}
				}

				if ( options.length > 0 ) {
					optionsText = options.join('\n');
				}
				$('#' + id).find('.field-options').val(optionsText);
				$('#' + id).find('.field-options-prop').removeClass('hidden');
			} else {
				$('#' + id).find('.field-options-prop').addClass('hidden');
			}

			// toggle hide option display
			if ( field.type == 'hidden' ) {
				$('#' + id).find('.field-hide').attr('checked', 'checked');
				$('#' + id).find('.non-hidden-fields').addClass('hidden');
			} else {
				$('#' + id).find('.non-hidden-fields').removeClass('hidden');
			}

			// toggle checkbox option display
			if ( field.type == 'checkbox' ) {
				$('#' + id).find('.checked-by-default').removeClass('hidden');
			} else {
				$('#' + id).find('.checked-by-default').addClass('hidden');
			}

			// set the rest of the fields
			$('#' + id).find('.field-index').val(index);
			$('#' + id).find('.field-type').val(field.type);
			$('#' + id).find('.field-key').val(field.name);
			$('#' + id).find('.field-label').val(field.placeholder);
			$('#' + id).find('.field-value').val(field.value);

			$('#' + id).find('.settings-form-edit').removeClass('hidden');
			$('#' + id).find('.form-manager-table').addClass('hidden');

			// show update and cancel button
			$('#' + id)
				.find('.setting-popup-buttons')
					.html('<button type="button" class="btn btn-primary field-update-btn">Update Field</button>&nbsp;\
							<button type="button" class="btn btn-default field-cancel-btn">Cancel</button>');
		},

		showPopup: function(id, settings) {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				defaults = {
					title: 'Pop-Up',
					header: true,
					footer: true,
					overlay: true,
					removeOnClose: false,
					wide: false
				},
				options = Object.assign({}, defaults, settings),
				html = $.trim(prime.getHtml('basePopup')),
				header = $.trim(prime.getHtml('popupHeader')),
				footer = $.trim(prime.getHtml('popupFooter'));

			html = ( options.header ? html.replace('{header}', header) : html.replace('{header}', '') );
			html = ( options.footer ? html.replace('{footer}', footer) : html.replace('{footer}', '') );
			html = html.replace(new RegExp('{id}', 'gi'), id);
			html = html.replace(new RegExp('{title}', 'gi'), options.title);

			if ( !$('.setting-popup-overlay').length ) {
				$('body').append('<div class="setting-popup-overlay"></div>');
			}

			if ( !$('#' + id).length ) {
				$('body').append(html);
			}

			if ( options.wide ) {
				$('#' + id).addClass('settings-popup-wide');
			}

			if ( options.overlay ) {
				$('.setting-popup-overlay').css('opacity', 0.6);
			} else {
				$('.setting-popup-overlay').css('opacity', 0);
			}

			$('.setting-popup-overlay').show();
			$('#' + id).show();

			cache.currentPopSetting = id;
		},

		centerPopup: function(id) {
			var top  = (($(window).height() / 2) - ($('#' + id).outerHeight() / 2));
			var left = (($(window).width() / 2) - ($('#' + id).outerWidth() / 2));

			if ( top < 0 ) top = 0;
			if ( left < 0 ) left = 0;

			$('#' + id).css({
		    	'top' : top,
		    	'left' : left
			});
		},

		closePopup: function(id, removeOnClose, keepEl) {
			var prime = this.prime,
				iframeDom = prime.frameBody,
				cache = prime.cache;

			keepEl = keepEl || cache.popupKeepEl;
			$('.setting-popup-overlay').hide();
			if ( removeOnClose === true ) {
				$('#' + id).remove();
			} else {
				$('#' + id).hide();
			}

			if ( !keepEl ) {
				// reset active element
				cache.activeElement = null;
				cache.activeElementType = null;
				cache.hoveredEl = null;
				cache.changeTemplate = true;
				cache.currentPopSetting = null;

				// remove is-selected class
				iframeDom.find('.is-selected').removeClass('is-selected');
			}
			cache.popupKeepEl = false;
		},

		delete: function() {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				layers = cache.layers,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				type = cache.activeElementType,
				isContainer = (iframeDom.find('#' + elID).parent().attr('id') == 'screen-container') ? true : false,
				data = null;

			if ( isContainer && iframeDom.find('#screen-container').find("> .ib2-section-el").length == 2 ) {
				alert('Cannot delete section. You need to have at least one section.');
				cache.activeElement = null;
				cache.activeElementType = null;
				cache.hoveredEl = null;

				// remove is-selected class
				iframeDom.find('.is-selected').removeClass('is-selected');
				return;
			}

			// if a container
			if ( isContainer ) {
				for ( var i = 0; i < layers.length; i++ ) {
					if ( layers[i]['id'] == elID ) {
						data = layers[i];
						break;
					}
				}
			} else {
				data = prime.getLayer(window.FrameDocument.getElementById(elID));
			}

			if ( data ) {
				if ( data.hasOwnProperty('child') && data.child !== null && $.isArray(data.child) ) {
					this.deleteElements(data.child);
				}
				this.deleteElement(elID, data.type);
			}

			if ( type == 'wbox' ) {
				this.deleteElement(elID, type);
			}

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			// re-calculate layers
			prime.saveLayers();
			// resize backgrounds
			gui.resizeBackgrounds();
			// save history
			prime.saveHistory();
		},

		deleteElements: function(data) {
			data.forEach(function(child){
				if ( child.hasOwnProperty('child') && child.child !== null && $.isArray(child.child) ) {
					this.deleteElements(child.child);
				}
				this.deleteElement(child.id, child.type);
			}, this);
		},

		deleteElement: function(id, type) {
			var prime = this.prime,
				iframeDom = prime.frameBody,
				cache = prime.cache;

			iframeDom.find('#' + id).remove();
			if ( cache.contents[type].hasOwnProperty(id) ) {
				delete cache.contents[type][id];
			}
		},

		saveCloneData: function(newID, type) {
			var prime = this.prime;
			prime.newElementData(newID, type);
		},

		cloneCss: function(oldID, newID) {
			var prime = this.prime,
				cache = prime.cache,
				iframeHtml = prime.frameHtml;

			if ( iframeHtml.find('#' + oldID + '-css').length ) {
				var css = iframeHtml.find('#' + oldID + '-css').html();
				css = css.replace(new RegExp(oldID, 'gi'), newID);
				iframeHtml.find('head').append('<style id="' + newID + '-css" class="ib2-element-css" type="text/css"></style>');
				iframeHtml.find('#' + newID + '-css').html(css);
				// re-save custom styles
				prime.getCustomStyles();
			}
		},

		cloneChilds: function(parentID) {
			var self = this,
				prime = this.prime,
				iframeDom = prime.frameBody,
				clonedHtml = '';

			if ( iframeDom.find('#' + parentID).find('.ib2-element').length ) {
				iframeDom.find('#' + parentID).find('.ib2-element').each(function(){
					var child = $(this),
						oldID = child.attr('id'),
						childType = child.attr('data-el'),
						freshID = prime.generateID(10, 'ib2_el_');

					clonedHtml = iframeDom.find('#' + parentID).html();
					clonedHtml = clonedHtml.replace(new RegExp(oldID, 'gi'), freshID);
					iframeDom.find('#' + parentID).html(clonedHtml);

					self.cloneCss(oldID, freshID);
					self.saveCloneData(freshID, childType);
				});
			}
		},

		clone: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				type = cache.activeElementType,
				newID = prime.generateID(9, 'ib2_el_'),
				data = null,
				cloned, 
				clonedHtml = '';

			prime.disableSortable();

			// remove is-selected class
			iframeDom.find('.is-selected').removeClass('is-selected');

			if ( iframeDom.find('#' + elID).length ) {
				cloned = iframeDom.find('#' + elID).clone();
				iframeDom.find('#' + elID).after(cloned);
				iframeDom.find('#' + elID).after().attr('id', newID);

				clonedHtml = iframeDom.find('#' + newID).html();
				clonedHtml = clonedHtml.replace(new RegExp(elID, 'gi'), newID);
				iframeDom.find('#' + newID).html(clonedHtml);

				this.cloneCss(elID, newID);
				this.saveCloneData(newID, type);

				this.cloneChilds(newID);
			}

			cache.activeElement = null;
			cache.activeElementType = null;
			cache.hoveredEl = null;

			// re-calculate layers
			prime.saveLayers();

			// resize backgrounds
			gui.resizeBackgrounds();

			// re-enable sortable
			setTimeout(function(){
				prime.sortable();
			}, 50);
		},

		textEditor: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				type = cache.activeElementType;

			// save current text padding
			cache.textPadding = iframeDom.find('#' + elID).css('padding');
			cache.isTexting = elID;

			// widen the text editor
			iframeDom.find('#' + elID).css('padding', '10px');

			// destroy sortable
			prime.disableSortable();

			// load text editor
			window.FrameWindow.loadTinyMce(elID);
		},

		removeTinyMce: function() {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				textID = cache.isTexting;
			if ( textID ) {
				cache.isTexting = null;

				// revert text padding
				iframeDom.find('#' + textID).css('padding', cache.textPadding);
				
				// re-enable sortable
				if ( cache.popupMode ) {
					setTimeout(function(){
						prime.sortable('.ib2-section-content');
					}, 100);
				} else {
					prime.sortable();
				}
			}
		},
	}
})(PrimeBuilder, window, document, jQuery);