(function (PrimeBuilder, w, d, $) { 
	"use strict";

	/**
	 * Settings Actions/Events/Interaction
	 */
	PrimeBuilder.Apply = function (prime) {
		this.prime = prime;
	};

	PrimeBuilder.Apply.prototype = {
		defaults: {
			id: null,
			type: 'text',
			name: 'padding',
			element: null,
			value: null,
			field: null,
			props: null
		},

		switchMode: function(mode, name, setting) {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elID = cache.activeElement,
				type = cache.activeElementType,
				settingData = cache.contents[type][elID]['settings'][name],
				data = {
					id: elID,
					type: type,
					name: name,
					element: prime.getSettingTarget(elID, settingData.pos),
					props: setting.props
				};

			if ( mode == 'all' ) {
				data.value = settingData.all;
				for ( var i = 0; i < setting.fields.length; i++ ) {
					if ( setting.fields[i].mode == 'all' ) {
						data.field = setting.fields[i];
						break;
					}
				}

				this.quickChange(setting.modeAction, data);
			} else {
				setting.fields.forEach(function(field){
					if ( mode == field.mode ) {
						data.value = settingData[mode][field.prop];
						data.field = field;

						this.quickChange(setting.modeAction, data);
					}
				}, this);
			}

			// resize backgrounds
			gui.resizeBackgrounds();

			// update mode data
			cache.contents[type][elID]['settings'][name]['mode'] = mode;
		},

		change: function(data) {
			var prime = this.prime,
				gui = prime.gui;

			data = Object.assign({}, this.defaults, data);
			if ( data.field ) {
				var field = data.field,
					action = field.action;

				if ( action == 'none' ) {
					return;
				}

				this[action](data, true);
				//this.updateBackground(action, data);
			}

			gui.resizeSelector();
			gui.resizeBackgrounds();

			// save history
			prime.saveHistory();
			prime.savingHistory = true;
			setTimeout(function(){
				prime.savingHistory = false;
			}, 3000);
		},

		quickChange: function(action, data, update) {
			var prime = this.prime,
				gui = prime.gui;
			if ( action == 'none' ) {
				return;
			}

			data = Object.assign({}, this.defaults, data);
			this[action](data, update);
			//this.updateBackground(action, data);

			gui.resizeSelector();
			gui.resizeBackgrounds();
		},

		iconstyle: function(data, update) {
			var elem = data.element,
				prop = data.field.prop,
				unit = data.field.unit || '',
				value = data.value + unit;

			if ( prop.indexOf('@') ) {
				var props = prop.split('@');
				prop = props[0];
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}

			elem.parent().css(prop, value);
		},

		bgcolor: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				utils = prime.utils,
				elem = data.element,
				prop = data.field.prop,
				unit = data.field.unit || '',
				value = data.value + unit;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var settings = cache.contents[data.type][data.id]['settings'],
				colors = settings[data.name],
				opacity = settings.hasOwnProperty('opacity') ? settings['opacity']['opacity'] : 1,
				mainColor = colors['backgroundColor'];

			if ( mainColor !== 'transparent' ) {
				var hexColor = utils.rgbToHex(mainColor),
					rgb = utils.hexToRgb(hexColor);

				mainColor = 'rgba(' + rgb.r + ', ' + rgb.g + ', ' + rgb.b + ', ' + opacity + ')';
			}
			// main background color
			elem.css('backgroundColor', mainColor);

			if ( colors['gradient'] == 'yes' ) {
				// process gradient color
				var dir = colors['colordir'] + 'deg',
					color2 = colors['gradColor'],
					pos1 = colors['color1pos'] + "%",
					pos2 = colors['color2pos'] + "%",
					gradient = '';

				mainColor += ' ' + pos1;
				if ( color2 !== 'transparent' ) {
					var hexColor2 = utils.rgbToHex(color2),
						rgb2 = utils.hexToRgb(hexColor2);

					color2 = 'rgba(' + rgb2.r + ', ' + rgb2.g + ', ' + rgb2.b + ', ' + opacity + ') ' + pos2;
				}

				gradient = 'linear-gradient(' + dir + ', ' + mainColor + ', ' + color2 + ')';
				elem.css('backgroundImage', gradient);
			} else {
				elem.css('backgroundImage', 'none');
			}
		},

		style: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				utils = prime.utils,
				iframeDom = prime.frameBody,
				elem = data.element,
				prop = data.field.prop,
				unit = data.field.unit || '',
				value = data.value + unit;

			if ( prop.indexOf('@') ) {
				var props = prop.split('@');
				prop = props[0];
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var settings = cache.contents[data.type][data.id]['settings'];
			if ( prop == 'backgroundColor' && settings.hasOwnProperty('opacity') ) {
				// check opac
				var opacity = settings['opacity']['opacity'] || 1,
					_opac = opacity;
				if ( elem.find('> .is-background').length ) {
					opacity = elem.find('> .is-background').css('opacity');
					if ( opacity != _opac ) {
						cache.contents[data.type][data.id]['settings']['opacity']['opacity'] = opacity;
					}
				}

				if ( opacity < 1 && value != 'transparent' ) {
					var hexColor = utils.rgbToHex(value),
						rgb = utils.hexToRgb(hexColor);

					elem.css('backgroundColor', 'rgba(' + rgb.r + ', ' + rgb.g + ', ' + rgb.b + ', ' + opacity + ')');
				} else {
					elem.css(prop, value);
				}
			} else if ( prop.hasOwnProperty('border') > -1 && prop.hasOwnProperty('radius') < 0 && elem.find('> .is-background').length ) {
				elem.find('> .is-background').css(prop, value);
			} else {
				if ( prop.hasOwnProperty('radius') > -1 && elem.find('> .is-background').length ) {
					elem.find('> .is-background').css(prop, value);
				}
				
				if ( data.type == 'box' && $.inArray(prop, ['width', 'minHeight']) > -1 ) {
					elem.parent().css(prop, value);
					if ( prop == 'minHeight' ) {
						elem.find('> .ib2-box-content').css(prop, value);
					}
				}

				if ( data.type == 'wbox' && prop == 'height' ) {
					iframeDom.find('#' + data.id + '-box').css(prop, value);
				}

				if ( prop == 'width' && cache.contents[data.type][data.id].hasOwnProperty('width') ) {
					cache.contents[data.type][data.id]['width'] = parseFloat(data.value);
				}

				if ( $.inArray(prop, ['height', 'minHeight']) > -1 && cache.contents[data.type][data.id].hasOwnProperty('height') ) {
					cache.contents[data.type][data.id]['height'] = parseFloat(data.value);
				}

				elem.css(prop, value);
			}

			if ( data.type == 'countdown' && $.inArray(prop, ['padding', 'paddingTop']) ) {
				if ( elem.find('.ib2-countdown-reveal').length ) {
					elem.find('.ib2-countdown-reveal').css('paddingTop', value);
				}
			}
		},

		cssvar: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				field = data.field,
				prop = field.prop,
				target = field.cssSelector == 'self' ? elem : elem.find(' ' + field.cssSelector),
				unit = field.unit || '',
				value = data.value + unit;

			var varname = prime.cssVarMap(data.type, field, prop);
			if ( prop.indexOf('backgroundColor') > -1 && $.inArray(data.type, ['button', 'menu', 'form']) > -1 ) {
				var dataName = 'buttonstyle',
					dataKey = 'button-type';

				switch ( data.type ) {
					case 'menu':
						dataName = 'menustyle';
						dataKey = 'style';
						break;

					case 'form': 
						dataName = 'formbutton';
						dataKey = 'submit-color-type';
						break;
				}

				var style = cache.contents[data.type][data.id]['settings'][dataName][dataKey],
					gradient = 'none';

				if ( style == 'glossy' ) {
					var colors = prime.utils.getGradientColors(value);
					if ( colors ) {
						gradient = 'linear-gradient(to bottom, ' + colors[0] + ', ' + colors[1] + ', ' + colors[2] + ')';
					}
				}

				target.each(function(i){
					target.get(i).style.setProperty(varname, value);
					target.get(i).style.setProperty(varname + 'img', gradient);
				});
			} else {
				if ( prop.indexOf('textShadow') > -1 ) {
					value = '1px 1px 0px ' + value;
				}

				target.each(function(i){
					target.get(i).style.setProperty(varname, value);
				});
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		attribute: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				type = data.type,
				attribute = data.field.attrib,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var settings = cache.contents[data.type][data.id]['settings'][data.name];
			if ( type == 'image' ) {
				if ( $.inArray(settings['button-action'], ['popup', 'url']) > -1 ) {
					if ( !elem.is("a") ) {
						elem.find('picture').wrap('<a></a>');
						elem = elem.find('a');
					}
				} else {
					if ( elem.find('picture').parent().is('a') ) {
						elem.find('picture').unwrap();
						return;
					}
				}
			}

			if ( attribute == 'target' ) {
				if ( value == 'yes' ) {
					value = '_blank';
				} else {
					value = '_self';
				}
			}

			if ( attribute == 'href' || attribute == 'src' ) {
				value = ( value != '' ) ? Base64.decode(value) : '';
			}
			elem.attr(attribute, value);
		},

		showimage: function(data, update) {
			var prime = this.prime,
				gui = prime.gui,
				elem = data.element,
				elID = data.id,
				attrib = data.field.attrib,
				attachment = data.attachment || null,
				value = data.value;

			if ( value == '' ) {
				value = data.value = $('#ib2-img-src').attr('src');
			}

			// change style
			var image_url = value.replace(/^(https?|ftp):/, '');
			elem.attr(attrib, image_url);
			elem.parent().find('source').attr('srcset', image_url);

			elem.css('marginTop', 0);
			elem.closest('.el-content').css('height', 'auto');

			// try to resize the image
			if ( elem.get(0) && elem.get(0).naturalWidth ) {
				var currentWidth = elem.outerWidth();
				if ( elem.get(0).naturalWidth < currentWidth ) {
					elem.css('width', elem.get(0).naturalWidth + 'px');
					elem.closest('.el-content').css('width', elem.get(0).naturalWidth + 'px');

					gui.resizeSelector();
				}
			}
	
			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		changetext: function(data, update) {
			var elem = data.element,
				textclass = data.field.textClass,
				value = data.value;

			// change style
			if ( textclass != '' && elem.find('.' + textclass).length ) {
				elem.find('.' + textclass).text(value);
			} else {
				if ( data.type != 'countdown' ) {
					elem.text(value);
				}
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		border: function(data, update) {
			var prime = this.prime,
				gui = prime.gui,
				elem = data.element,
				prop = data.field.prop,
				value = data.value,
				unit = data.field.unit || '';

			if ( data.hasOwnProperty('values') ) {
				value = data.value = data.values[0] + 'px ' + data.values[1] + ' ' + data.values[2];
				if ( elem.find('> .is-background').length ) {
					elem = elem.find('> .is-background');
				}

				elem.css(prop, value);
				
				// update data
				if ( update ) {
					this.updateData(data);
				}
			}
		},

		breakword: function(data, update) {
			var prime = this.prime,
				gui = prime.gui,
				iframeHtml = prime.frameHtml,
				elem = data.element,
				prop = data.field.prop,
				value = data.value;

			var word_wrap = ( value == 'yes' ? 'break-word' : 'normal' );
			var word_break = ( value == 'yes' ? 'break-all' : 'normal' );
			var hyphens = ( value == 'yes' ? 'auto' : 'none' );
			var hyphen_css = "word-wrap: " + word_wrap + " !important; "
				hyphen_css += "word-break: " + word_break + " !important; "
				hyphen_css += "-moz-hyphens: " + hyphens + "; "
				hyphen_css += "-webkit-hyphens: " + hyphens + "; ";
				hyphen_css += "-ms-hyphens: " + hyphens + "; ";
				hyphen_css += "hyphens: " + hyphens + ";";

			var selectors = [
				'.ib2-button',
				'.ib2-section-content p',
				'.ib2-section-content h1',
				'.ib2-section-content h2',
				'.ib2-section-content h3',
				'.ib2-section-content h4',
				'.ib2-section-content h5',
				'.ib2-section-content h6',
				'.ib2-section-content ol',
				'.ib2-section-content ul'
			];

			var selector = selectors.join(","),
				css = selector + " { " + hyphen_css + " }";

			iframeHtml.find('#text-break-word').html(css);
			if ( update ) {
				this.updateData(data);
			}
		},

		formstyle: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			var $el = elem.closest('form');
			if ( value == 'horizontal' ) {
				$el.find('.form-fields').attr('class', 'form-fields col-sm-12 col-md-8 inline-fields');
				$el.find('.button-fields').attr('class', 'button-fields col-sm-12 col-md-4 button-inline');
			} else if ( value == 'semi-horizontal' ) {
				$el.find('.form-fields').attr('class', 'form-fields col-sm-12 col-md-12 inline-fields');
				$el.find('.button-fields').attr('class', 'button-fields col-sm-12 col-md-12 button-halfline');
			} else {
				$el.find('.form-fields').attr('class', 'form-fields col-sm-12 col-md-12');
				$el.find('.button-fields').attr('class', 'button-fields col-sm-12 col-md-12');
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		fieldstyle: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			elem.removeClass('field-normal field-normal-thick field-sharp field-sharp-thick');
			elem.addClass(value);

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		fieldsize: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			elem.removeClass('input-sm input-md input-lg');
			elem.addClass(value);

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		optconfirm: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			if ( value == 'yes' ) {
				elem.find('.form-fields').append('<input type="hidden" name="_confirm" value="1">');
			} else {
				elem.find('.form-fields').find('input[name="_confirm"]').remove();
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		webinar: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var settings = cache.contents[data.type][data.id]['settings'][data.name],
				esp = cache.contents[data.type][data.id]['settings']['formdata'];

			if ( settings['_webinar_key'] != '' && settings['_webinar_key'] != 'none' ) {
				if ( !elem.find('input[name="_webinar_key"]').length ) {
					elem.find('.form-fields').append('<input type="hidden" name="_webinar_key">');
				}
				if ( !elem.find('input[name="_webinar_action"]').length ) {
					elem.find('.form-fields').append('<input type="hidden" name="_webinar_action">');
				}
				elem.find('input[name="_webinar_key"]').attr('value', settings['_webinar_key']);
				elem.find('input[name="_webinar_action"]').attr('value', $('#ib2-webinar-action').val());
			} else {
				elem.find('input[name="_webinar_key"]').remove();

				if ( esp && esp['esp-type'] == 'none' ) {
					elem.find('input[name="_webinar_action"]').remove();
				}
			}
		},

		formredir: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value,
				addurl = false;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var settings = cache.contents[data.type][data.id]['settings'][data.name],
				webinar = cache.contents[data.type][data.id]['settings']['webinar'],
				esp = cache.contents[data.type][data.id]['settings']['formdata'];

			if ( webinar && webinar['_webinar_key'] != '' && webinar['_webinar_key'] != 'none' && settings['_webinar_redirect'] != '' ) {
				addurl = true;
			}

			if ( esp && esp['esp-type'] != 'none' && settings['_webinar_redirect'] != '' ) {
				addurl = true;
			}

			if ( addurl ) {
				if ( !elem.find('input[name="_webinar_redirect"]').length ) {
					elem.find('.form-fields').append('<input type="hidden" name="_webinar_redirect">');
				}
				elem.find('input[name="_webinar_redirect"]').attr('value', Base64.decode(settings['_webinar_redirect']));
			} else {
				elem.find('input[name="_webinar_redirect"]').remove();
			}
		},

		fbopt: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var settings = cache.contents[data.type][data.id]['settings'][data.name];
			if ( settings['facebook-enable'] == 'yes' ) {
				elem.show();
			} else {
				elem.hide();
			}

			elem.find('.ib2-facebook-optin-txt').text(settings['facebook-text']);
			elem.find('.ib2-fb-button').text(settings['facebook-label']);

			if ( settings['facebook-only'] == 'yes' ) {
				elem.parent().find('.form-fields').hide();
				elem.parent().find('.button-fields').hide();
			} else {
				elem.parent().find('.form-fields').show();
				elem.parent().find('.button-fields').show();
			}
		},

		buttonaction: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				type = data.type,
				settings = cache.contents[data.type][data.id]['settings']['buttonaction'],
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			if ( type == 'image' ) {
				if ( $.inArray(settings['button-action'], ['popup', 'url']) > -1 ) {
					if ( !elem.attr('href') ) {
						elem.find('picture').wrap('<a></a>');
						elem = elem.find('a');
					}
				} else {
					if ( elem.find('picture').parent().is('a') ) {
						elem.find('picture').unwrap();
						return;
					}
				}
			}

			elem.removeClass('open-popup');
			if ( settings['button-action'] == 'popup' ) {
				elem.attr('data-popup', settings['popup']).addClass('open-popup');
			} else if ( settings['button-action'] == 'url' ) {
				var url = settings['href'] != '' ? Base64.decode(settings['href']) : '';
				var	target = settings.hasOwnProperty('target') && settings['target'] == 'yes' ? '_blank' : '_self';
				elem.attr('href', url);
				elem.attr('target', target);
			} else {
				elem.attr('href', '');
			}
		},

		buttoncolor: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				cacheData = cache.contents[data.type][data.id]['settings'],
				styleSetting = cacheData.hasOwnProperty('formbutton') ? cacheData['formbutton'] : cacheData['buttonstyle'],
				buttonStyle = styleSetting.hasOwnProperty('submit-color-type') ? styleSetting['submit-color-type'] : styleSetting['button-type'],
				prop = data.field.prop,
				value = data.value;

			// change style
			if ( value == 'transparent' ) {
				elem.css(prop, value);
			} else {
				var css = this.getButtonColorCss(value, buttonStyle);
				elem.css(css);
			}
			
			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		buttonstyle: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				name = data.name,
				colorSetting = cache.contents[data.type][data.id]['settings']['buttoncolor'],
				value = data.value,
				color = colorSetting.backgroundColor,
				css = this.getButtonColorCss(color, value);

			if ( update ) {
				this.updateData(data);
			}

			// normal state
			elem.css(css);

			// hover state
			var cacheData = {
				id: data.id,
				type: data.type,
				name: 'buttoncolor',
				element: elem,
				value: colorSetting['backgroundColor@hover'],
				field: {
					id: 'button-background-hover-color',
					label: 'Background Color',
					type: 'color',
					action: 'cssvar',
					cssSelector: 'self',
					prop: 'backgroundColor@hover'
				},
				props: colorSetting.props
			};
			this.quickChange('cssvar', cacheData);
		},

		formbuttonstyle: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				name = data.name,
				colorSetting = cache.contents[data.type][data.id]['settings']['formbutton'],
				value = data.value,
				color = colorSetting.backgroundColor,
				css = this.getButtonColorCss(color, value);

			if ( update ) {
				this.updateData(data);
			}

			// normal state
			elem.css(css);

			// hover state
			var cacheData = {
				id: data.id,
				type: data.type,
				name: 'formbutton',
				element: elem,
				value: colorSetting['backgroundColor@hover'],
				field: {
					id: 'submit-background-hover-color',
					label: 'Background Color',
					type: 'color',
					action: 'cssvar',
					cssSelector: 'self',
					prop: 'backgroundColor@hover'
				},
				props: colorSetting.props
			};
			this.quickChange('cssvar', cacheData);
		},

		getButtonColorCss: function(color, style) {
			var prime = this.prime,
				utils = prime.utils;

			if ( style == 'glossy' ) {
				var colors = utils.getGradientColors(color);
				return {
					"background-color": color,
					"background-image": (colors ? 'linear-gradient(to bottom, ' + colors[0] + ', ' + colors[1] + ', ' + colors[2] + ')' : 'none'),
				}
			} else {
				return {
					"background-color": color,
					"background-image": 'none',
				}
			}
		},

		css: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				utils = prime.utils,
				elID = data.id,
				type = data.type,
				field = data.field,
				fieldType = field.type,
				iframeHtml = prime.frameHtml,
				name = data.name,
				defSettings = prime.settings,
				setting = cache.contents[data.type][data.id]['settings'][name],
				prop = data.field.prop,
				value = data.value,
				css = '', 
				selectors = [];

			if ( data.hasOwnProperty('values') ) {
				value = data.value = data.values[0] + 'px ' + data.values[1] + ' ' + data.values[2];
			}

			if ( defSettings.hasOwnProperty(name) ) {
				if ( !iframeHtml.find('#' + elID + '-css').length ) {
					iframeHtml.find('head').append('<style id="' + elID + '-css" class="ib2-element-css"></style>');
				}
				defSettings[name]['fields'].forEach(function(field){
					var fieldId = field.type + '-' + field.id + '-' + type;
					if ( $('#' + fieldId).is(":visible") && field.action == 'css' && field.hasOwnProperty('cssSelector') ) {
						if ( $.inArray(field.cssSelector, selectors) < 0 ) {
							selectors.push(field.cssSelector);
						}
					}
				});

				selectors.forEach(function(selector){
					var sel = selector.replace(new RegExp('{elID}', 'g'), '#' + elID);
					css += sel + ' { ';

					defSettings[name]['fields'].forEach(function(field){
						if ( field.hasOwnProperty('cssSelector') && field.cssSelector == selector ) {
							var styleProp = field.prop,
								val = ( styleProp == prop ? value : setting[styleProp] ),
								unit = ( field.hasOwnProperty('unit') ? field.unit : '' );

							if ( styleProp !== 'fontSize' && styleProp.indexOf('FontSize') > -1 ) {
								styleProp = 'fontSize';
							}

							if ( styleProp == 'fontSize' && $.isNumeric(val) ) {
								val = parseInt(val);
								val = (val/10);
								unit = 'rem';
							}

							var important = '';
							if ( styleProp.indexOf('@') > -1 ) {
								var parts = styleProp.split('@');
								styleProp = parts[0];

								if ( typeof parts[1] !== "undefined" && parts[1] == 'hover' && type != 'tabs' ) {
									important = ' !important';
								}
							}

							if ( field.hasOwnProperty('important') && field.important === true ) {
								important = ' !important';
							}

							if ( styleProp == 'textShadow' ) {
								val = '1px 1px 0px ' + val;
							}

							if ( styleProp == 'backgroundColor' && $.inArray(type, ['button', 'countdown']) > -1 ) {
								var styleSetting = cache.contents[data.type][data.id]['settings'][type + 'style'],
									buttonStyle = type == 'button' ? styleSetting['button-type'] : styleSetting['style'].replace('-box', ''),
									buttonCss = this.getButtonColorCss(val, buttonStyle);

								for ( var btnProp in buttonCss ) {
									css += btnProp + ': ' + buttonCss[btnProp] + important + '; ';
								}
							} else {
								if ( styleProp.indexOf('sub-') > -1 ) {
									styleProp = styleProp.replace('sub-', '');
								}
								css += utils.camelToDash(styleProp) + ': ' + val + unit + important + '; ';
							}
						}
					}, this);
					css += '} ';
				}, this);
				iframeHtml.find('#' + elID + '-css').html(css);

				// remove old countdown html
				iframeHtml.find('#' + elID + '-countdown-css').remove();

				// update data
				if ( update ) {
					this.updateData(data);
				}

				// re-save custom styles
				prime.getCustomStyles();
			}
		},

		submitbutton: function(data, update) {
			var prime = this.prime,
				elem = data.element,
				value = data.value;

			if ( value == 'image' ) {
				elem.parent().find('.ib2-form-image').show();
				elem.parent().find('.ib2-form-submit').hide();
			} else {
				elem.parent().find('.ib2-form-image').hide();
				elem.parent().find('.ib2-form-submit').show();
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		submitimage: function(data, update) {
			var attrib = data.field.attrib,
				elem = data.element,
				value = data.value;

			// change style
			elem.parent().find('.ib2-form-image').attr(attrib, value);

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		swapclass: function(data, update) {
			var prime = this.prime,
				utils = prime.utils,
				field = data.field,
				classname = field.classname,
				elem = data.element,
				value = data.value,
				cache = prime.cache,
				setting = cache.contents[data.type][data.id]['settings'][data.name],
				oldClass = setting[classname];

			if ( oldClass != '' && oldClass != 'none' ) {
				elem.removeClass(oldClass);
			}

			if ( value != '' && value != 'none' ) {
				elem.addClass(value);
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		caption: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				setting = cache.contents[data.type][data.id]['settings'][data.name],
				elem = data.element,
				value = data.value;

			if ( !elem.find('figcaption').length && value != '' ) {
				elem.append('<figcaption class="' + data.field.textClass + '"></figcaption>');
			}

			if ( value == '' ) {
				elem.find('figcaption').remove();
				elem.css('backgroundColor', 'transparent');
			} else {
				elem.find('figcaption').text(value);
				elem.css('backgroundColor', setting.backgroundColor);
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		iconshape: function(data, update) {
			var elem = data.element,
				value = data.value;

			elem.removeClass('ib2-icon-rounded ib2-icon-circle');
			if ( value != 'sharp' ) {
				elem.addClass('ib2-icon-' + value);
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		setanchor: function(data, update) {
			var elem = data.element,
				value = data.value;

			elem.attr('data-anchor', value);
			
			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		question: function(data, update) {
			var elem = data.element,
				value = data.value;

			value = data.value = parseInt(value);

			elem.find('> .ib2-quiz-page').addClass('ib2-unused-question').hide();
			for ( var q = 0; q < value; q++ ){
				var question = elem.find('> .ib2-quiz-page').eq(q);
				if ( q == 0 ) {
					question.show();
				}
				question.removeClass('ib2-unused-question');
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		captionstyle: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			if ( !elem.find('figcaption').length || setting['img-caption-txt'] == '' ) {
				value = 'transparent';
			}
			elem.css('backgroundColor', value);
		},

		gdpr: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elID = data.id,
				type = data.type,
				elem = data.element,
				html = $.trim(prime.getHtml('gdpr'));

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			if ( !elem || typeof elem === "undefined" || !elem.length ) {
				if ( setting['gdpr'] == 'yes' ) {
					iframeDom.find('#' + elID).find('.form-fields').after('<div class="eu-checkbox col-md-12">' + html + '</div>');
				}
			}

			if ( setting['gdpr'] == 'no' ) {
				iframeDom.find('#' + elID).find('.eu-checkbox').remove();
			} else {
				if ( setting['eu-text'] == '' ) {
					setting['eu-text'] = 'Yes, I agree to receive transactional and marketing emails from [YOUR BUSINESS]';
				}

				iframeDom.find('#' + elID).find('.eu-text').removeAttr('style');
				iframeDom.find('#' + elID).find('.eu-checkbox').css('color', setting['color']);
				iframeDom.find('#' + elID).find('.eu-text').text($.trim(setting['eu-text']));
			}
		},

		separator: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elem = data.element,
				value = data.value,
				isTop = data.field.prop.indexOf('top') > -1 ? true : false,
				classname = isTop ? 'ib2-top-separator' : 'ib2-bottom-separator',
				prefix = isTop ? 'top-' : '';

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name],
				color = setting[prefix + 'color'] || '#FFFFFF',
				height = setting[prefix + 'height'] || 100,
				flip = setting[prefix + 'flip'] || 'no';
			
			elem.find('svg.' + classname).remove();
			if ( value != 'normal' ) {
				var svg = prime.getHtml(value);
				svg = svg.replace('{posclass}', classname);
				elem.append(svg);
				elem.find('svg.' + classname).get(0).style.setProperty('--shape-height', height + 'px');
				if ( elem.find('svg.' + classname).find('.ib2-main-shape').length ) {
					elem.find('svg.' + classname).find('.ib2-main-shape').get(0).style.setProperty('--shape-color', color);
				}

				if ( isTop ) {
					var transform = flip == 'yes' ? 'scale(-1,-1) translate(-1600,-100)' : 'scale(1,-1) translate(0,-100)';
					elem.find('svg.' + classname).find('.ib2-shape').attr('transform', transform);
				} else {
					if ( flip == 'yes' ) {
						elem.find('svg.' + classname).find('.ib2-shape').attr('transform', 'scale(-1,1) translate(-1600,0)');
					}
				}
			}
		},

		sepcolor: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elem = data.element,
				value = data.value,
				isTop = data.field.prop.indexOf('top') > -1 ? true : false,
				classname = isTop ? 'ib2-top-separator' : 'ib2-bottom-separator';

			// update data
			if ( update ) {
				this.updateData(data);
			}

			if ( elem.find('svg.' + classname).find('.ib2-main-shape').length ) {
				elem.find('svg.' + classname).find('.ib2-main-shape').get(0).style.setProperty('--shape-color', value);
			}
		},

		sepheight: function(data, update) {
			var elem = data.element,
				unit = data.field.unit || '',
				value = data.value + unit,
				isTop = data.field.prop.indexOf('top') > -1 ? true : false,
				classname = isTop ? 'ib2-top-separator' : 'ib2-bottom-separator';

			// update data
			if ( update ) {
				this.updateData(data);
			}
			elem.find('svg.' + classname).get(0).style.setProperty('--shape-height', value);
		},

		sepflip: function(data, update) {
			var elem = data.element,
				value = data.value,
				isTop = data.field.prop.indexOf('top') > -1 ? true : false,
				classname = isTop ? 'ib2-top-separator' : 'ib2-bottom-separator';

			// update data
			if ( update ) {
				this.updateData(data);
			}

			elem.find('svg.' + classname).find('.ib2-shape').each(function(){
				var shape = $(this),
					transform = isTop ? 'scale(-1,-1) translate(-1600,-100)' : 'scale(-1,1) translate(-1600,0)';

				if ( value == 'yes' ) {
					shape.attr('transform', transform);
				} else {
					if ( isTop ) {
						shape.attr('transform', 'scale(1,-1) translate(0,-100)');
					} else {
						shape.removeAttr('transform');
					}
				}
			});
		},

		map: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			var mapURL = 'https://maps.google.com/maps?&z=' + setting['mapzoom'] + '&t=' + setting['maptype'] + '&output=embed';
			if ( setting['address'] !== '' ) {
				mapURL += '&q=' + encodeURIComponent(setting['address']);
			}

			elem.attr('src', mapURL);
		},

		changeicon: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elem = data.element,
				type = data.type,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			if ( data.type == 'icon' ) {
				elem.attr('class', value);
				$('#icon-icon-shape-icon').parent().find('.icon-preview').find('i').attr('class', value);
			} else {
				iframeDom.find('#' + data.id).find('.ib2-button').find('i').remove();
				if ( setting['iconpos'] == 'before' ) {
					iframeDom.find('#' + data.id).find('.ib2-button').prepend('<i class="' + setting['iconclass'] + '"></i>');
				} else {
					iframeDom.find('#' + data.id).find('.ib2-button').append('<i class="' + setting['iconclass'] + '"></i>');
				}
				$('#icon-shape-icon-' + type).parent().find('.icon-preview').find('i').attr('class', setting['iconclass']);
				if ( setting['iconclass'] !== '' ) {
					$('#icon-shape-icon-' + type).parent().find('.btn-remove-icon').show();
				}
			}
		},

		progress: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			elem.text(setting['progress-bar']);
			if ( setting['progress-bar-striped'] == 'yes' ) {
				elem.addClass('progress-bar-striped');
			} else {
				elem.removeClass('progress-bar-striped');
			}

			if ( setting['active'] == 'yes' ) {
				elem.addClass('active');
			} else {
				elem.removeClass('active');
			}
		},

		textshadow: function(data, update) {
			var elem = data.element,
				prop = data.field.prop,
				value = data.value,
				shadow = '1px 1px 0px ' + value;

			elem.css(prop, shadow);

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		shadowtext: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			var shadow = setting['textShadowToggle'] == 'yes' ? '1px 1px ' + setting['textShadowBlur'] + 'px ' + setting['textShadowColor'] : 'none';
			elem.css('textShadow', shadow);
		},

		shadow: function(data, update) {
			var prime = this.prime,
				utils = prime.utils,
				elem = data.element,
				value = data.value,
				unit = data.field.unit || '';

			if ( data.hasOwnProperty('values') ) {
				var values = data.values,
					mode = values[0] == 'initial' ? '' : values[0];

				if ( mode == 'none' ) {
					value = 'none';
				} else {
					var h = values[1] + 'px',
					v = values[2] + 'px',
					blur = values[3] + 'px',
					spread = values[4] + 'px',
					rgb = utils.hexToRgb(values[5]),
					color = 'rgba(' + rgb.r + ', ' + rgb.g + ', ' + rgb.b + ', ' + values[6] + ')';
					value = h + ' ' + v + ' ' + blur + ' ' + spread + ' ' + color + ' ' + mode;
				}
				elem.css('boxShadow', value);

				// update data
				if ( update ) {
					this.updateData(data);
				}
			}
		},

		video: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				elID = data.id,
				value = $.trim(data.value);
			
			// update data
			if ( update ) {
				this.updateData(data);
			}

			if ( value != '' ) {
				try {
					value = window.atob(value);
				} catch (e) {
					// do nothing
				}
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name],
				videos = cache.videos,
				videotype = setting['video-type'],
				videoData = ( videos.hasOwnProperty(elID) ? videos[elID] : {} ),
				vidImage = $('#ib2-vid-src').attr('src'),
				matches, 
				isEmpty = true,
				videoid = '',
				url = '',
				code = '',
				autoplay = ( setting.hasOwnProperty('video-autoplay') && setting['video-autoplay'] !== '' ? setting['video-autoplay'] : 0 ),
				controls = ( setting.hasOwnProperty('video-controls') && setting['video-controls'] !== '' ? setting['video-controls'] : 1 );

			if ( typeof cache.videos[elID] === "undefined" ) {
				cache.videos[elID] = {};
			}

			if ( !cache.videos[elID].hasOwnProperty('autoplay') ) {
				cache.videos[elID]['autoplay'] = autoplay;
			}

			if ( !cache.videos[elID].hasOwnProperty('controls') ) {
				cache.videos[elID]['controls'] = autoplay;
			}

			// setup video
			switch(videotype) {
				case 'youtube':
					if ( value != '' ) {
						matches = value.match(/(?:https?:\/{2})?(?:w{3}\.)?youtu(?:be)?\.(?:com|be)(?:\/watch\?v=|\/)([^\s&]+)/);
						if ( matches ) {
							videoid = matches[1];
							url = '//www.youtube.com/embed/' + videoid + '?wmode=transparent';
							url += '&rel=0&modestbranding=0&showinfo=0&ytid=' + videoid;
							url += '&otopelay=' + autoplay + '&controls=' + controls;
							url += '&enablejsapi=1';
							code = Base64.encode(url);
							isEmpty = false;
						}
					}
					cache.videos[elID][videotype + 'code'] = code;
					cache.videos[elID][videotype + 'id'] = videoid;
					cache.videos[elID]['url'] = Base64.encode(value);
					break;

				case 'vimeo':
					if ( value != '' ) {
						matches = value.match(/(videos|video|channels|\.com)\/([\d]+)/);
						if ( matches ) {
							videoid = matches[2];
							url = '//player.vimeo.com/video/' + videoid + '?wmode=transparent';
							url += '&title=0&byline=0&portrait=0&vmid=' + videoid;
							url += '&otopelay=' + autoplay + '&enablejsapi=1';
							code = Base64.encode(url);
							isEmpty = false;
						}
					}
					cache.videos[elID][videotype + 'code'] = code;
					cache.videos[elID][videotype + 'id'] = videoid;
					cache.videos[elID]['url'] = Base64.encode(value);
					break;

				case 'hosted-mp4':
				case 'hosted-ogg':
				case 'hosted-webm':
				case 'hosted-splash':
					var mp4 = ( setting.hasOwnProperty('video-hosted-mp4') && setting['video-hosted-mp4'] != '' ? encodeURIComponent(Base64.decode(setting['video-hosted-mp4'])) : '' ),
						webm = ( setting.hasOwnProperty('video-hosted-webm') && setting['video-hosted-webm'] != '' ? encodeURIComponent(Base64.decode(setting['video-hosted-webm'])) : '' ),
						ogg = ( setting.hasOwnProperty('video-hosted-ogg') && setting['video-hosted-ogg'] != '' ? encodeURIComponent(Base64.decode(setting['video-hosted-ogg'])) : '' ),
						splash = ( setting.hasOwnProperty('video-hosted-splash') && setting['video-hosted-splash'] != '' ? encodeURIComponent(Base64.decode(setting['video-hosted-splash'])) : '' );

					isEmpty = ( ogg == '' && webm == '' && mp4 == '' ? true : false );
					if ( !isEmpty ) {
						var player = $('#ib2-player').val();
						url = player + '&wmode=transparent';
						url += '&mp4=' + mp4;
						url += '&webm=' + webm;
						url += '&ogg=' + ogg;
						url += '&splash=' + splash;
						url += '&otopelay=' + autoplay + '&controls=' + controls;
						code = Base64.encode(url);
					}

					cache.videos[elID]['hostedcode'] = code;
					cache.videos[elID]['mp4'] = Base64.encode(mp4);
					cache.videos[elID]['webm'] = Base64.encode(webm);
					cache.videos[elID]['ogg'] = Base64.encode(ogg);
					cache.videos[elID]['splash'] = Base64.encode(splash);
					break;

				case 'embed':
					if ( value != '' ) {
						value = value.replace(/(\r\n|\n|\r)/gm,"");
						code = Base64.encode(value);
						isEmpty = false;
					}
					cache.videos[elID][videotype + 'code'] = code;
					break;
			}

			// playback video
			if ( !isEmpty ) {
				if ( videotype == 'embed' ) {
					elem.html('<div class="embed-responsive embed-responsive-16by9">' + value + '</div>');
					elem.find('iframe, embed').addClass('embed-responsive-item');
				} else {
					var output = 
					'<div class="embed-responsive embed-responsive-16by9">\
						<iframe class="embed-responsive-item ib2-video-frame" src="' + url + '" scrolling="no" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>\
					</div>';
					elem.html(output);
				}
			} else {
				elem.html('<img src="' + vidImage + '" class="img-responsive">');
			}
		},

		videotoggle: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				videos = cache.videos,
				elem = data.element,
				field = data.field,
				elID = data.id,
				value = data.value,
				videoType = value,
				videoData = ( videos.hasOwnProperty(elID) ? videos[elID] : {} ),
				vidImage = $('#ib2-vid-src').attr('src'),
				fileKey = field.file.replace('video-', '');

			if ( typeof cache.videos[elID] === "undefined" ) {
				cache.videos[elID] = {};
			}

			if ( field.type == 'checkbox' ) {
				if ( field.file == 'video-autoplay' ) {
					cache.videos[elID]['autoplay'] = data.value = ( value == 'yes' ? 1 : 0 );
				} else if ( field.file == 'video-controls' ) {
					cache.videos[elID]['controls'] = data.value = ( value == 'yes' ? 0 : 1 );
				}
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
			
			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			videoType = cache.videos[elID]['type'] = setting['video-type'];

			var processHosted = true;
			if ( videoType == 'hosted' ) {
				if ( setting['video-hosted-mp4'] === '' && setting['video-hosted-webm'] === '' && setting['video-hosted-ogg'] === '' ) {
					processHosted = false;
				}
			}

			if ( processHosted || setting['video-' + videoType] !== '' ) {
				var code, re1, re2;
				if ( videoData.hasOwnProperty(videoType + 'code') && videoData[videoType + 'code'] != '' ) {
					try {
						code = window.atob(videoData[videoType + 'code']);
					} catch(e) {
						code = videoData[videoType + 'code'];
					}

					re1 = new RegExp("([?&])otopelay=.*?(&|$)", "i");
					re2 = new RegExp("([?&])controls=.*?(&|$)", "i");

					code = code.replace(re1, '$1' + 'otopelay' + "=" + setting['video-autoplay'] + '$2');
					code = ( videoType == 'youtube' || 'hosted' ) ? code.replace(re2, '$1' + 'controls' + "=" + setting['video-controls'] + '$2') : code;
				
					if ( videoType == 'embed' ) {
						elem.find(' > .el-content').html(code);
						elem.find(' > .el-content').find('iframe, embed').addClass('embed-responsive-item');
					} else {
						var output = 
						'<div class="embed-responsive embed-responsive-16by9">\
							<iframe class="embed-responsive-item" src="' + code + '" scrolling="no" style="pointer-events: none" allow="autoplay; encrypted-media" allowfullscreen></iframe>\
						</div>';

						elem.html(output);
					}
				} else {
					var videoUrl = ( videoType == 'hosted' ? setting['video-hosted-mp4'] : setting['video-' + videoType] );
					try {
						videoUrl = window.atob(videoUrl);
					} catch(e) {
						videoUrl = videoUrl;
					}

					var cacheData = {
						id: data.id,
						type: data.type,
						name: data.name,
						element: elem,
						value: videoUrl,
						field: field,
					};
					this.quickChange('video', cacheData);
				}
			} else {
				elem.html('<img src="' + vidImage + '" class="img-responsive">');
			}
		},

		backgroundimg: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elem = data.element,
				elID = data.id,
				type = data.type,
				prop = data.field.prop,
				value = $.trim(data.value),
				props = data.props,
				imgprops = ['backgroundAttachment', 'backgroundPosition', 'backgroundRepeat', 'backgroundSize'],
				styles = {};

			// update data
			if ( update ) {
				this.updateData(data);
			}

			if ( value !== '' ) {
				var value = value.replace(/^(https?|ftp):/, '');
				if ( elem.find(" > .is-background").length ) {
					// change the background only
					elem.find(" > .is-background").css(prop, "url('" + value + "')");
				} else {
					var setting = cache.contents[data.type][data.id]['settings'][data.name];

					// disable border on main element
					elem.css('border', 'none');

					// create bg img placeholder
					elem.prepend('<div id="' + elID + '-box-img" class="is-background"></div>');
					styles = {
						position: 'absolute',
						top: 0,
						left: 0,
						maxWidth: '100%',
						width: '100%',
						height: elem.css('height'),
					};

					for ( var key in setting ) {
						if ( key == 'backgroundImage' ) {
							value = value.replace(/^(https?|ftp):/, '');
							styles['backgroundImage'] = "url('" + value + "')";
						} else {
							if ( $.inArray(key, imgprops) > -1 ) {
								styles[key] = setting[key] || '';
							}
						}
					}

					// border data
					if ( cache.contents[data.type][data.id]['settings'].hasOwnProperty('border') ) {
						var borderSetting = cache.contents[data.type][data.id]['settings']['border'];
						if ( borderSetting['mode'] == 'single' && borderSetting.hasOwnProperty('single') ) {
							for ( var key in borderSetting['single'] ) {
								styles[key] = borderSetting['single'][key];
							}
						} else {
							styles['border'] = borderSetting['all'];
						}
					}

					// corner data
					if ( cache.contents[data.type][data.id]['settings'].hasOwnProperty('corner') ) {
						var cornerSetting = cache.contents[data.type][data.id]['settings']['corner'];
						if ( cornerSetting['mode'] == 'single' && cornerSetting.hasOwnProperty('single') ) {
							for ( var key in cornerSetting['single'] ) {
								styles[key] = cornerSetting['single'][key];
							}
						} else {
							styles['borderRadius'] = cornerSetting['all'];
						}
					}
					elem.find(" > .is-background").css(styles);
				}
			} else {
				// return border
				// border data
				if ( cache.contents[data.type][data.id]['settings'].hasOwnProperty('border') ) {
					var borderSetting = cache.contents[data.type][data.id]['settings']['border'],
						borderStyles = {};
					if ( borderSetting['mode'] == 'single' && borderSetting.hasOwnProperty('single') ) {
						for ( var key in borderSetting['single'] ) {
							borderStyles[key] = borderSetting['single'][key];
						}
					} else {
						borderStyles['border'] = borderSetting['all'];
					}
					elem.css(borderStyles);
				}

				// remove background placeholder
				elem.find(" > .is-background").remove();
				
				// resize selector
				gui.resizeSelector();
				// resize all backgrounds
				gui.resizeBackgrounds();
			}
		},

		backgroundimgprop: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				prop = data.field.prop,
				unit = data.field.unit || '',
				value = data.value + unit;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			if ( prop == 'backgroundAttachment' ) {
				if ( value == 'fixed' ) {
					elem.find(" > .is-background").css('backgroundPosition', 'center top');
				} else {
					elem.find(" > .is-background").css('backgroundPosition', setting['backgroundPosition']);
				}
			}

			// change style
			elem.find(" > .is-background").css(prop, value);
		},

		backgroundvid: function(data, update) {
			var prime = this.prime,
				gui = prime.gui,
				cache = prime.cache,
				type = data.type,
				value = data.value,
				elem = data.element,
				file = data.field.file,
				videos = {};

			// update data
			if ( update ) {
				this.updateData(data);
			}

			value = value ? value.replace(/^(https?|ftp):/, '') : '';
			videos[file] = value;

			// display video
			gui.backgroundVideo(elem, videos);
		},

		togglevideo: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elem = data.element,
				value = data.value;

			// display video
			if ( value == 'yes' ) {
				var vids = ['mp4', 'ogg', 'webm'],
					setting = cache.contents[data.type][data.id]['settings'][data.name],
					videos = {};

				if ( setting ) {
					for ( var key in setting ) {
						if ( $.inArray(key, vids) > -1 ) {
							videos[key] = ( typeof setting[key] === 'string' ? setting[key] : '' );
						}
					}
					gui.backgroundVideo(elem, videos);
				}
			} else {
				elem.find(" > .ib2-bgvideo-param").remove();
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		sectiontoggle: function(data, update) {
			var prime = this.prime,
				iframeDom = prime.frameBody,
				gui = prime.gui,
				elem = data.element,
				cache = prime.cache,
				elID = data.id,
				value = data.value,
				content = '',
				width = prime.pagewidth;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			// grab content
			content = elem.find(".el-cols").eq(0).html(); 
			// remove content
			elem.find(".el-cols").eq(0).html('');

			if ( value == 'section' ) {
				elem.css({ 
					'width': width + 'px',
					'margin-left': 'auto',
					'margin-right': 'auto'
				});

				elem.removeClass('ib2-wsection-el');
				elem.attr('data-el', value);
				elem.find(' > .el-content > .el-content-inner').remove();
				elem.find(' > .el-content').addClass('el-cols');
				elem.find(' > .el-content').append(content);
			} else {
				elem.css({ 
					'width': '100%',
					'margin-left': '0px',
					'margin-right': 'opx'
				});
				elem.addClass('ib2-wsection-el');
				elem.attr('data-el', value);
				elem.find(' > .el-content').removeClass('el-cols');
				elem.find(' > .el-content').append('<div class="el-content-inner" style="width: ' + width + 'px; margin:0px auto"></div>');
				elem.find(' > .el-content > .el-content-inner').html('<div class="el-cols">' + content + '</div>');
			}

			if ( iframeDom.find('#' + elID + '-box-img').length ) {
				iframeDom.find('#' + elID + '-box-img').attr('id', 'old-background');
				var bgimg = iframeDom.find('#old-background').clone();
				bgimg.attr('id', elID + '-box-img');
				bgimg.prependTo(elem.find(' > .el-content'));
				iframeDom.find('#old-background').remove();
			}

			var newSettings = cache.contents[data.type][elID],
				newType = ( data.type == 'section' ) ? 'wsection' : 'section';


			delete cache.contents[data.type][elID];

			newSettings.type = newType;
			cache.contents[newType][elID] = newSettings;

			cache.activeElementType = newType;

			prime.disableSortable();
			prime.sortable();

			// resize
			gui.resizeSelector();
			gui.resizeBackgrounds();
		},

		countdownstyle: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elem = data.element,
				iframeHtml = prime.frameHtml,
				iframeDom = prime.frameBody,
				setting = cache.contents[data.type][data.id]['settings'][data.name],
				textSetting = cache.contents[data.type][data.id]['settings']['countdowntexts'],
				elID = data.id,
				value = data.value,
				format = prime.getHtml('normalCountdown'),
				customCss = false;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			iframeHtml.find('#' + elID + '-countdown-css').remove();
			iframeHtml.find('#' + elID + '-css').remove();
			iframeDom.find('#' + elID).addClass('ib2-countdown-style');
			if ( value == 'circular' ) {
				format = prime.getHtml('circularCountdown');
				format = format.replace(new RegExp('{id}', 'gi'), elID);
				iframeDom.find('#' + elID).html(format);

				// set stroke
				var radius = 54,
					circumference = 2 * Math.PI * radius;

				iframeDom.find('#' + elID).find('.clock-anim').css('stroke-dasharray', circumference);
				iframeDom.find('#' + elID).find('.clock-anim').css('stroke-dashoffset', circumference);
				iframeDom.find('#' + elID).addClass('ib2-circular-countdown');
				iframeDom.find('#' + elID).removeClass('ib2-countdown-shadow');
			} else {
				format = format.replace(new RegExp('{id}', 'gi'), elID);
				iframeDom.find('#' + elID).html(format);

				iframeDom.find('#' + elID).removeClass('ib2-circular-countdown');
				iframeDom.find('#' + elID).removeClass('ib2-countdown-shadow');
				if ( value == 'text' ) {
					iframeDom.find('#' + elID).removeClass('ib2-countdown-style');
					if ( !iframeDom.find('#' + elID).find('.c-before-text').length ) {
						iframeDom.find('#' + elID).prepend('<span class="c-before-text"></span> ');
					}

					if ( typeof textSetting['c-before-text'] !== "undefined" && textSetting['c-before-text'] !== null && textSetting['c-before-text'] != '' ) {
						iframeDom.find('#' + elID).find('.c-before-text').text(textSetting['c-before-text']);
					}

					if ( !iframeDom.find('#' + elID).find('.c-after-text').length ) {
						iframeDom.find('#' + elID).append('<span class="c-after-text"></span> ');
					}

					if ( typeof textSetting['c-after-text'] !== "undefined" && textSetting['c-after-text'] !== null && textSetting['c-after-text'] != '' ) {
						iframeDom.find('#' + elID).find('.c-after-text').text(textSetting['c-after-text']);
					}
				} else if ( value == 'glossy-box' ) {
					iframeDom.find('#' + elID).addClass('ib2-countdown-shadow');
					customCss = true;
				} else if ( value == 'flat-box' ) {
					customCss = true;
				}
			}

			// clean up css first
			iframeHtml.find('#' + elID + '-css').remove();
			if ( cache.cssRules.hasOwnProperty(elID) ) {
				delete cache.cssRules[elID];
			}

			if ( customCss ) {
				this.css(data, false);
			}

			data.name = 'countdown';
			this.countdowndate(data, false);
		},

		countdowndate: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elem = data.element,
				iframeDom = prime.frameBody,
				elID = data.id,
				value = data.value,
				updateType = iframeDom.find('#' + elID).find('.clock-item').length ? 'circular' : 'normal',
				finalDate, time;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			if ( setting.mode == 'date' ) {
				var offset = setting['tz'],
					hour = ("0" + setting['time-hour']).slice(-2),
					min = ("0" + setting['time-min']).slice(-2),
					newdate = setting['exact-date'] + ' ' + hour + ':' + min,
					origTime = moment.tz(newdate, offset),
					utc = origTime.clone().tz("UTC"),
					localTime = moment.utc(utc).toDate();

				finalDate = moment(localTime).format("YYYY/MM/DD HH:mm:ss");
			} else {
				var day = parseInt(setting['time-day']) * 24 * 60 * 60 * 1000,
					hour = parseInt(setting['time-hour']) * 60 * 60 * 1000,
					min = parseInt(setting['time-min']) * 60 * 1000,
					dateValue = day + hour + min;
				
				finalDate = new Date().valueOf() + dateValue;
			}

			// update countdown
			iframeDom.find('#' + elID).countdown(finalDate)
				.on('update.countdown', function(event) {
					time = {
						days: event.offset.totalDays, 
						hours: event.offset.hours, 
						minutes: event.offset.minutes, 
						seconds: event.offset.seconds
					};
					
					gui.countdownUpdate(iframeDom, elID, time, updateType);
				});
		},

		countexpire: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elem = data.element,
				iframeDom = prime.frameBody,
				setting = cache.contents[data.type][data.id]['settings'][data.name],
				elID = data.id,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			$('#el-btn-countdown-normal').hide();
			$('#el-btn-countdown-hidden').hide();
			if ( value == 'reveal' ) {
				$('#el-btn-countdown-normal').show().removeClass('disabled');
				$('#el-btn-countdown-hidden').show().addClass('disabled');
				if ( !iframeDom.find('#' + elID).find('.ib2-countdown-reveal').length ) {
					var textSetting = cache.contents['page']['screen-container']['settings']['pagetypo'],
						html = '<div id="' + elID + '-content" class="ib2-section-content ib2-countdown-reveal" style="margin: 0px auto; max-width:100%; min-height: 120px;"></div>';

					iframeDom.find('#' + elID).append(html);

					var txtElem = iframeDom.find('.element-type-text'),
						pos = 'afterbegin';

					cache.isAdding = true;
					cache.isDragging = true;

					prime.addElement(iframeDom.find('#' + elID + '-content'), txtElem, pos);

					var paddingTop = iframeDom.find('#' + elID).css('paddingTop'),
						css = {
							paddingTop: paddingTop,
							fontSize: textSetting['fontSize'] + 'px',
								fontFace: textSetting['fontFace'],
								color: textSetting['color'],
								textShadow: 'none'
							},
						content = 'This is the content section that will be revealed after the countdown\'s end. You can modify this text, add more text, or change the style by clicking the pencil button.';

					iframeDom.find('#' + elID + '-content').css(css);
					iframeDom.find('#' + elID + '-content').find('.ib2-text-el').eq(0).find('p').text(content);
				}

				iframeDom.find('#' + elID).find('> div, > span').each(function(){
					if ( !$(this).hasClass('ib2-countdown-reveal') ) {
						$(this).hide();
					}
				});
				iframeDom.find('#' + elID).find('.ib2-countdown-reveal').show();
				// sortable
				prime.disableSortable();
				prime.sortable();
			} else {
				iframeDom.find('#' + elID).find('> div, > span').each(function(){
					if ( !$(this).hasClass('ib2-countdown-reveal') && !$(this).is(":visible") ) {
						$(this).show();
					}
				});
				iframeDom.find('#' + elID).find('.ib2-countdown-reveal').remove();
			}
		},

		opacity: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				utils = prime.utils,
				elem = data.element,
				type = data.type,
				value = data.value,
				currentBgColor = cache.contents[data.type][data.id]['settings']['backgroundColor'] || null,
				boxed = ['section', 'wsection', 'box', 'columns', 'wbox', 'video'];

			// update data
			if ( update ) {
				this.updateData(data);
			}

			if ( $.inArray(type, boxed) < 0 ) {
				elem.css('opacity', value);
				return;
			}

			// background image opacity
			if ( elem.find(" > .is-background").length ) {
				elem.find(" > .is-background").css('opacity', value);
			}

			// background video opacity
			if ( elem.find(" > .ib2-bgvideo-param").length ) {
				elem.find(" > .ib2-bgvideo-param").css('opacity', value);
			}

			// background color
			var cacheData = {
				id: data.id,
				type: data.type,
				name: ( type == 'video' ? 'backgroundcolor' : 'background'),
				element: elem,
				value: currentBgColor,
				field: {
					id: 'color',
					label: 'Color',
					type: 'color',
					action: 'bgcolor',
					prop: 'backgroundColor'
				}
			};
			this.quickChange('bgcolor', cacheData);
		},

		comment: function(data, update) {
			var prime = this.prime,
				elem = data.element,
				value = data.value;

			elem.attr('data-comment', value);
			elem.find('.comment-system').text(prime.utils.ucwords(value));
			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		shortcode: function(data, update) {
			var prime = this.prime,
				elem = data.element,
				value = data.value;

			elem.find('.current-shortcode').text(value);
			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		html: function(data, update) {
			var prime = this.prime,
				elem = data.element,
				value = data.value;

			elem.find('.el-content').html(value);
			if ( value && elem.find('.el-content').find('script').length ) {
				var script_remove = 0;
				elem.find('.el-content').find('script').each(function(){
					var scr = $(this);
					if ( !scr.attr('src') || scr.attr('src') === '' ) {
						scr.remove();
						script_remove++;
					}
				});

				if ( script_remove > 0 ) {
					value = data.value = elem.find('.el-content').html();
					$('#textarea-html-content-code').val(value);
				}
			}
			
			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		switchesp: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var settings = cache.contents[data.type][data.id]['settings'][data.name],
				webinar = cache.contents[data.type][data.id]['settings']['webinar'];
			if ( value !== 'html' ) {
				if ( !elem.find('input[name="_esp"]').length ) {
					elem.find('.form-fields').append('<input name="_esp" type="hidden">');
				}

				if ( !elem.find('input[name="_lid"]').length ) {
					elem.find('.form-fields').append('<input name="_lid" type="hidden">');
				}

				elem.find('input[name="_esp"]').val(value);
				elem.find('input[name="_lid"]').val('none');

				if ( !elem.find('input[name="_webinar_action"]').length ) {
					elem.find('.form-fields').append('<input type="hidden" name="_webinar_action">');
				}
				elem.find('input[name="_webinar_action"]').attr('value', $('#ib2-webinar-action').val());

			} else {
				elem.find('input[name="_esp"]').remove();
				elem.find('input[name="_lid"]').remove();
				if ( webinar && (webinar['_webinar_key'] == 'none' || webinar['_webinar_key'] === null) ) {
					elem.find('input[name="_webinar_action"]').remove();
				}
			}
		},

		switchlist: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var settings = cache.contents[data.type][data.id]['settings'][data.name];
			if ( !elem.find('input[name="_esp"]').length ) {
				elem.find('.form-fields').append('<input name="_esp" type="hidden">');
				elem.find('input[name="_esp"]').val(settings['esp-type']);
			}

			if ( !elem.find('input[name="_lid"]').length ) {
				elem.find('.form-fields').append('<input name="_lid" type="hidden">');
			}

			elem.find('input[name="_lid"]').val(value);
		},

		pagewidth: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				settings = cache.contents['page']['screen-container'],
				elem = data.element,
				value = data.value;

			// update containers
			iframeDom.find('#screen-container > .ib2-section-el').each(function(){
				var $el = $(this),
					type = $el.attr('data-el');

				if ( type == 'wsection' ) {
					$el.find('> .el-content > .el-content-inner').css('width', value + 'px');
					if ( $el.find('> .el-content > .el-content-inner > .el-cols > .ib2-section-content').length == 1 ) {
						$el.find('> .el-content > .el-content-inner > .el-cols > .ib2-section-content').css('width', value + 'px');
					}
				} else {
					$el.css('width', value + 'px');
					if ( $el.find('> .el-content > .ib2-section-content').length == 1 ) {
						$el.find('> .el-content > .ib2-section-content').css('width', value + 'px');
					}
				}
			});

			// update other vars
			settings['width'] = value;
			prime.pagewidth = value;

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		loadmenu: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value,
				menu = prime.utils.slugify(value);

			if ( value == 'none' ) {
				alert('ERROR: Please select a menu.');
				return;
			}

			if ( !$('#ib2-menu-' + menu + '-html').length ) {
				alert('ERROR: The menu you selected have no item.');
				return;
			}

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings']['menustyle'],
				html = $('#ib2-menu-' + menu + '-html').html(),
				style = setting['style'];

			html = html.replace('ID-nav', data.id + '-nav');
			elem.find('> nav').html(html);
			elem.find('nav > ul').addClass('ib2-navi ib2-navi-' + style);
		},

		menustyle: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				elem = data.element,
				value = data.value;

			elem.removeClass('ib2-navi-plain ib2-navi-plain-pipe ib2-navi-flat-box ib2-navi-glossy');
			elem.addClass('ib2-navi-' + value);

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			gui.setNavCss(value, prime.getSettingTarget(data.id, 'nav'), setting);
		},

		socialshare: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			for ( var key in setting ) {
				if ( key == 'pos' ) {
					continue;
				}

				var social = key.replace('share-', '');
				if ( setting[key] == 'yes' ) {
					elem.find('.ib2-' + social + '-share').show();
				} else {
					elem.find('.ib2-' + social + '-share').hide();
				}
			}
		},

		carousel: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elID = data.id,
				type = data.type,
				elem = data.element,
				value = data.value,
				num = data.num,
				key = data.key;

			if ( update ) {
				cache.contents[type][elID]['settings'][data.name][data.name][num][key] = value;
			}

			var settings = cache.contents[type][elID]['settings'][data.name][data.name];
			if ( settings.length ) {
				var html = '';
				elem.find('.carousel-inner > .item').each(function(i){
					if ( settings[i]['imageurl'] == '' ) {
						settings[i]['imageurl'] = $('#ib2-slides-src').attr('src');
					}

					var activeClass = ( $(this).hasClass('active') ? ' active' : '' ),
						img = settings[i]['imageurl'],
						title = settings[i]['title'],
						url = settings[i]['desturl'];

					html += '<div class="item' + activeClass + '">';
					if ( url !== '' ) html += '<a href="' + url + '">';
						html += '<img src="' + img + '" alt="' + title + '">';
						if ( url !== '' ) html += '</a>';
						if ( title !== '' ) {
  						html += '<div class="carousel-caption">' + title + '</div>';
					}
					html += '</div>';
				});
				elem.find('.carousel-inner').html(html);
			}
		},

		infobar: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			if ( iframeDom.find('#screen-container').find('.ib2-notification-bar').length ) {
				iframeDom.find('#screen-container').find('.ib2-notification-bar').remove();
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			if ( setting.active == 'yes' ) {
				var html = PrimeBuilder.Html.getInfobarHtml(),
					top = iframeDom.find('#screen-container').offset().top,
					width = iframeDom.find('#screen-container').outerWidth();

				iframeDom.find('#screen-container').append(html);

				var barHeight = iframeDom.find('#screen-container').find('.ib2-notification-bar > .ib2-notification-text').outerHeight();
				iframeDom.find('#screen-container').css('padding-top', barHeight + 'px');

				// style and text
				iframeDom.find('.ib2-notification-bar > .ib2-notification-text')
					.css({
						'font-family': setting.fontFamily,
						'background-color': setting.backgroundColor,
						'border-color': setting.border,
						'color': setting.color
					});

				iframeDom.find('.ib2-notification-bar > .ib2-notification-text').find('.att-text').text(setting.text);
				iframeDom.find('.ib2-notification-bar > .ib2-notification-text > a.att-url')
					.attr('href', setting.url)
					.text(setting.anchor)
					.css('color', setting.color);

				if ( !setting.hasOwnProperty('backgroundColor@active') ) {
					setting['backgroundColor@active'] = cache.contents[data.type][data.id]['settings'][data.name]['backgroundColor@active'] = '#3c78d8';
				}
				iframeDom.find('.ib2-notification-bar > .ib2-notification-text > a.att-url').css('backgroundColor', setting['backgroundColor@active']);

			} else {
				iframeDom.find('#screen-container').css('padding-top', '0px');
				iframeDom.find('#screen-container').find('.ib2-notification-bar').remove();
			}
		},

		animation: function(data, update) {
			var prime = this.prime,
				elem = data.element,
				value = data.value,
				anim = prime.animationMap(value);

			elem.addClass('animated ' + anim);
			setTimeout(function(){
				elem.removeClass('animated ' + anim);
			}, 2000);

			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		tab: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elID = data.id,
				type = data.type,
				elem = data.element,
				value = data.value,
				num = data.num,
				key = data.key;

			if ( update ) {
				cache.contents[type][elID]['settings'][data.name][data.name][num][key] = value;
			}

			var settings = cache.contents[type][elID]['settings'][data.name][data.name];

			iframeDom.find('#' + elID).find('> ul.ib2-nav-tabs > li').eq(num).find('.ib2-tab-title').text(value);
		},

		datesetup: function(data, update) {
			var prime = this.prime,
				cache = prime.cache,
				elID = data.id,
				type = data.type,
				elem = data.element,
				value = data.value;

			// update data
			if ( update ) {
				this.updateData(data);
			}

			var settings = cache.contents[type][elID]['settings'][data.name];
			var newDate = moment().tz(settings['tz']);
			elem.html(newDate.format(settings['format']));
		},

		savedata: function(data, update) {
			// update data
			if ( update ) {
				this.updateData(data);
			}
		},

		updateData: function(data) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				field = data.field,
				mode = field.mode || 'none',
				value = data.value,
				useFile = ( field.hasOwnProperty('file') ? true : false ),
				useText = ( field.hasOwnProperty('textClass') ? true : false );

			value = $.isNumeric(value) ? parseFloat(value) : value;
			var dataKey = useFile ? field.file : ( useText ? field.textClass : ( field.attrib || field.prop ) );
			if ( typeof dataKey === 'undefined' && field.hasOwnProperty('classname') ) {
				dataKey = field.classname;
			}

			if ( typeof dataKey === 'undefined' && field.hasOwnProperty('typename') ) {
				dataKey = field.typename;
			}

			if ( typeof cache.contents[data.type][data.id]['settings'][data.name] === "undefined" ) {
				var element = prime.getElementData(data.type, prime.getElementAlias(iframeDom.find('#' + data.id))),
					pos = 'outer';
				if ( element && element.hasOwnProperty('options') ) {
					element.options.forEach(function(option){
						if ( option.name == data.name ) {
							pos = option.target;
						}
					});
				}
				cache.contents[data.type][data.id]['settings'][data.name] = { 'pos': pos };
			}

			var setting = cache.contents[data.type][data.id]['settings'][data.name];
			if ( mode !== 'none' && setting.hasOwnProperty(mode) ) {
				if ( mode == 'single' )
					cache.contents[data.type][data.id]['settings'][data.name][mode][dataKey] = value;
				else if ( mode == 'all' )
					cache.contents[data.type][data.id]['settings'][data.name][mode] = value;
				else
					cache.contents[data.type][data.id]['settings'][data.name][dataKey] = value;
			} else {
				cache.contents[data.type][data.id]['settings'][data.name][dataKey] = value;
			}
			//console.log(cache.contents[data.type][data.id]['settings'][data.name]);
		},

		updateBackground: function(action, data) {
			var prime = this.prime,
				elID = data.id,
				type = data.type,
				iframeDom = prime.frameBody;

			if ( action.indexOf('background') == -1 && iframeDom.find('#' + elID + '-box-img').length ) {
				var selector = window.FrameDocument.getElementById(elID + '-box-img');
				if ( selector ) {
					var css = {
						maxWidth: '100%',
						width: '100%',
						height: iframeDom.find('#' + elID + '-box-img').parent().css('height'),
					};

					var parentStyles = prime.getStyles(selector.parentNode);
					if ( parentStyles ) {
						for ( var key in parentStyles ) {
							if ( key.indexOf('border') > -1 ) {
								css[key] = parentStyles[key];
							}
						}
						iframeDom.find('#' + elID + '-box-img').css(css);
					}
				}
			}
		}
	};

})(PrimeBuilder, window, document, jQuery);