(function (PrimeBuilder, window, document, $) { 
	"use strict";

	/**
	 * Editor GUI
	 */
	PrimeBuilder.Gui = function (prime) {
		this.prime = prime;
		this.buttons = null;
		this.fbuttons = null;
		this.resizable = false;
		this.fileFrame = null;
	};

	PrimeBuilder.Gui.prototype = {
		/**
		 * Default element button object
		 * @type {Object}
		 */
		defaultButton: { 
				id: 'id', 
				icon: 'fa fa-cog', 
				title: 'Button', 
				action: null,
				color: 'primary',
				classes: ''
		},

		/**
		 * List of default element buttons
		 * @type {Array}
		 */
		defaultButtons: [
			{ id: 'moveup', icon: 'fas fa-arrow-up', title: 'Move Element Up', action: 'moveup' },
			{ id: 'movedown', icon: 'fas fa-arrow-down', title: 'Move Element Down', action: 'movedown' },
			{ id: 'visibility', icon: 'fas fa-eye', title: 'Manage Visibility', action: 'visibility' },
			{ id: 'clone', icon: 'fa fa-copy', title: 'Clone Element', action: 'clone' },
			{ id: 'save', icon: 'fas fa-save', title: 'Save Element', action: 'saveelem' },
			{ id: 'delete', icon: 'fa fa-trash-o', title: 'Delete Element', color: 'danger', action: 'delete' }
		],

		setup: function() {
			var prime = this.prime;

			prime.loadHtml('toolbar', document.querySelector('#editor-container'), 'afterbegin');
			prime.loadHtml('devices', document.querySelector('.prime-toolbar'), 'beforeend');
			//prime.loadHtml('footer', document.querySelector('#editor-container'), 'beforeend');
			prime.loadHtml('settingsWrapper', document.querySelector('#editor-container'));
			prime.loadHtml('elementsWrapper', window.FrameDocument.querySelector('body'));
			prime.loadHtml('highlight', document.querySelector('#iframe-tools'));
			prime.loadHtml('selected', document.querySelector('#iframe-tools'));

			// load editor settings
			prime.loadHtml('settings', document.querySelector('#editor-settings'));

			// exit url
			$('.quit-btn').attr('href', $('#ib2-dashboard-url').val());

			// adjust height
			//this.adjustEditorSize();
		},

		adjustEditorSize: function() {
			var prime = this.prime,
				iframeDom = prime.frameBody,
				wh = $(window).outerHeight(), 
				eh = (wh-80) - 50;

			// adjust elements bar height
			iframeDom.find('#elements-bar > .elements-icons').css('height', eh + 'px');

			// adjust page container height
			if ( iframeDom.find('#screen-container').outerHeight() < wh ) {
				iframeDom.find('#screen-container').css('min-height', wh + 'px');
			}
		},

		updateListOptions: function(lists) {
			var options = '';
			if ( lists ) {
				options += '<option value="none"> -- select list/campaign/sequence -- </option>';
				for ( var key in lists ) {
					options += '<option value="' + key + '">' + lists[key] + '</option>';
				}
			} else {
				options += '<option value="none"> -- service not connected -- </option>';
			}
			$('#list-esp-list-form').html(options);
		},

		backgroundVideo: function(elem, videos) {
			if ( !elem.find("> .ib2-bgvideo-param").length ) {
				if ( elem.find('> .is-background').length ) {
					elem.find('> .is-background').after('<div class="ib2-bgvideo-param"></div>');
				} else {
					elem.prepend('<div class="ib2-bgvideo-param"></div>');
				}
			}

			elem.find("> .ib2-bgvideo-param").removeClass('youtube-bg-video');
			if ( !elem.find("> .ib2-bgvideo-param > video").length ) {
				elem.find("> .ib2-bgvideo-param").prepend('<video class="is-video-bg" autoplay muted loop></video>');
			}

			if ( typeof videos !== "undefined" ) {
				for ( var key in videos ) {
					if ( videos[key] !== '' ) {
						var vidUrl = videos[key].replace(/^(https?|ftp):/, '');
						if ( elem.find("> .ib2-bgvideo-param > video").find('source[type*="' + key + '"]').length ) {
							elem.find("> .ib2-bgvideo-param > video").find('source[type*="' + key + '"]').attr('src', vidUrl);
						} else {
							elem.find("> .ib2-bgvideo-param > video")
								.append('<source src="' + vidUrl + '" type="video/' + key + '">');
						}
					} else {
						if ( elem.find("> .ib2-bgvideo-param > video").find('source[type*="' + key + '"]').length ) {
							elem.find("> .ib2-bgvideo-param > video").find('source[type*="' + key + '"]').remove();
						}
					}
				}
			}

			if ( !elem.find("> .ib2-bgvideo-param > video > source").length ) {
				elem.find("> .ib2-bgvideo-param").remove();
			}
		},

		setNavCss: function(style, baseEl, setting) {
			var prime = this.prime,
				utils = prime.utils;

			baseEl.find('> li > a').each(function(i){
				var target = baseEl.find('> li > a').get(i);
				target.style.setProperty('--menu-color', setting['color']);
				target.style.setProperty('--menu-hover-color', setting['color@hover']);
			});
			
			baseEl.find('ul li a').each(function(i){
				var target = baseEl.find('ul li a').get(i);
				target.style.setProperty('--sub-menu-color', setting['sub-color']);
				target.style.setProperty('--sub-menu-hover-color', setting['sub-color@hover']);
			});

			if ( style == 'flat-box' ) {
				baseEl.each(function(i){
					var target = baseEl.get(i);
					target.style.setProperty('--menu-bg', setting['backgroundColor']);
					target.style.setProperty('--menu-bgimg', 'none');
				});
				baseEl.find('> li').each(function(i){
					var target = baseEl.find('> li').get(i);
					target.style.setProperty('--menu-hover-bg', setting['backgroundColor@hover']);
					target.style.setProperty('--menu-hover-bgimg', 'none');
				});
			} else if ( style == 'glossy' ) {
				var colors = utils.getGradientColors(setting['backgroundColor']);
				baseEl.each(function(i){
					var target = baseEl.get(i);
					target.style.setProperty('--menu-bg', setting['backgroundColor']);
					if ( colors ) {
						target.style.setProperty('--menu-bgimg', 'linear-gradient(to bottom, ' + colors[0] + ', ' + colors[1] + ', ' + colors[2] + ')');
					}
				});
				colors = utils.getGradientColors(setting['backgroundColor@hover']);
				baseEl.find('> li').each(function(i){
					var target = baseEl.find('> li').get(i);
					target.style.setProperty('--menu-hover-bg', setting['backgroundColor@hover']);
					if ( colors ) {
						target.style.setProperty('--menu-hover-bgimg', 'linear-gradient(to bottom, ' + colors[0] + ', ' + colors[1] + ', ' + colors[2] + ')');
					}
				});
			}
			
			baseEl.find('ul').each(function(i){
				var target = baseEl.find('ul').get(i);
				target.style.setProperty('--sub-menu-bg', setting['sub-backgroundColor']);
			});

			baseEl.find('ul li').each(function(i){
				var target = baseEl.find('ul li').get(i);
				target.style.setProperty('--sub-menu-hover-bg', setting['sub-backgroundColor@hover']);
			});
		},

		buildCss: function(cssRules, skipPage) {
			var prime = this.prime,
				cache = prime.cache,
				settings = prime.settings,
				apply = prime.apply,
				page = cache.contents['page']['screen-container']['settings'],
				iframeHtml = prime.frameHtml;

			skipPage = skipPage || false;
			// build global css
			if ( page && !skipPage ) {
				// remove existing css
				$('.ib2-element-css').remove();

				if ( page.hasOwnProperty('background') && settings.hasOwnProperty('background') ) {
					settings.background.fields.forEach(function(field){
						var dataKey = (field.hasOwnProperty('file') ? field['file'] : field['prop']);
						// apply change and update data
						var cacheData = {
							id: 'screen-container',
							type: ['page'],
							name: 'background',
							element: prime.frameBody.find('#screen-container'),
							value: page['background'][dataKey],
							field: field,
							props: ( settings.background.hasOwnProperty('props') ? settings.background.props : null )
						};

						apply.quickChange(field.action, cacheData);
					});
				}

				if ( page.hasOwnProperty('pagetypo') && settings.hasOwnProperty('pagetypo') ) {
					settings.background.fields.forEach(function(field){
						var dataKey = (field.hasOwnProperty('file') ? field['file'] : field['prop']);
						// apply change and update data
						var cacheData = {
							id: 'screen-container',
							type: ['page'],
							name: 'pagetypo',
							element: prime.frameBody.find('#screen-container'),
							value: page['background'][dataKey],
							field: field,
							props: ( settings.pagetypo.hasOwnProperty('props') ? settings.pagetypo.props : null )
						};

						apply.quickChange(field.action, cacheData);
					});
				}
			}

			if ( cssRules && typeof cssRules === "object" ) {
				var output = '';
				for ( var selector in cssRules ) {
					output += '<style type="text/css" class="ib2-element-css" id="' + selector + '-css">\n';
					cssRules[selector].forEach(function(style){
						var cssdata = style.hasOwnProperty('hover') ? style.hover : ( style.normal || null );
						if ( cssdata ) {
							output += style.selector + ' { ' + cssdata.cssText + ' }\n';
						}
					});
					output += '</style>\n';
				}
				iframeHtml.find('head').append(output);
			}
		},

		buildForm: function(el, inputs, setting) {
			if ( inputs.length > 0 ) {
				var style = setting['formstyle'];

				// clear fields
				el.find('.ib2-field-group').each(function(){
					$(this).remove();
				});

				inputs.forEach(function(field){
					var output = '', input = '', selected, checked;

					if ( field.visible === true || field.type == 'hidden' ) {
						if ( field.type != 'hidden' && field.type != 'checkbox' && field.type != 'radio' ) {
							output += '<div class="ib2-field-group form-group">';
						} else if ( field.type == 'checkbox' ) {
							output += '<div class="ib2-field-group ' + field.type + '">';
						}

						if ( field.type == 'select' ) {
							output += '<select name="' + field.name + '" placeholder="' + field.placeholder + '" class="form-control ' + style.fieldclass + ' ' + style.fieldstyleclass + '"';
							output += ( field.required == true ? ' required' : '' );
							output += '>';
							for ( var val in field.options ) {
								selected = ( val == field.value ? ' selected="selected"' : '' );
								output += '<option value="' + val + '"' + selected + '>' + field.options[val] + '</option>';
							}
							output += '</select>';
						} else if ( field.type == 'checkbox' ) {
							checked = ( field.checked === true ? ' checked="checked"' : '' );
							output += '<label><input type="checkbox" name="' + field.name + '" value="' + field.value + '"' + checked + '> ' + field.placeholder + '</label>';
						} else if ( field.type == 'radio' ) {
							checked = ( field.checked === true ? ' checked="checked"' : '' );
							for ( var val in field.options ) {
								checked = ( field.value === val ? ' checked="checked"' : '' );
								output += '<div class="ib2-field-group radio">';
								output += '<label><input type="radio" name="' + field.name + '" value="' + field.value + '"' + checked + '> ' + field.placeholder + '</label>';
								output += '</div>';
							}
						} else if ( field.type == 'textarea' ) {
							output += '<textarea name="' + field.name + '" placeholder="' + field.placeholder + '" rows="3" class="form-control ' + style.fieldclass + ' ' + style.fieldstyleclass + '"';
							output += ( field.required == true ? ' required' : '' );
							output += '>';
							output += field.value;
							output += '</textarea>';
						} else {
							var iconclass = ( field.icon !== 'none' ? 'field-' + field.icon + '-icon' : '' );
							output += '<input type="' + field.type + '" name="' + field.name + '" placeholder="' + field.placeholder + '" value="' + field.value + '"';
							output += ( field.type != 'hidden' ? ' class="form-control ' + style.fieldclass + ' ' + style.fieldstyleclass + ' ' + iconclass + '"' : '' );
							output += ( field.required == true ? ' required' : '' );
							output += ' />';
						}

						if ( field.type != 'hidden' && field.type != 'radio' ) {
							output += '</div>';
						}
					
						$(output).appendTo(el)
							.find('input[type="text"], input[type="email"], select, textarea')
							.css({
								backgroundColor: style.backgroundColor,
								borderColor: style.borderColor 
							});
					}
				});
			}
		},

		displayMedia: function(type, multiple, settingFor, idx, pid) {
			var prime = this.prime,
				interact = this.interact,
				utils = prime.utils,
				self = this;

			multiple = multiple || false;
			// If the media frame already exists, reopen it.
			if ( typeof self.fileFrame !== "undefined" && self.fileFrame !== null ) {
				self.fileFrame.close();
			}

			var options = {
				frame: 'select',
				title: 'Upload ' + utils.ucwords(type),
				library: {
	                type: type
	            },
				button: {
					text: 'Insert ' + utils.ucwords(type),
				},
				multiple: multiple
			};

			// create wp media
			self.fileFrame = wp.media.frames.file_frame = wp.media(options);
			
			// listen to select event
			self.fileFrame.on('select', function(){
				var attachment = self.fileFrame.state().get('selection').first().toJSON();
				$('body').trigger("mediaselected", [attachment, settingFor, idx, pid]);
			});

			// Finally, open the modal
			self.fileFrame.open();
		},

		displaySettings: function(type, alias, settings) {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				elID = cache.activeElement,
				opened = false,
				dynamics = ['slides', 'tabs', 'image', 'button', 'form', 'text'];

			// clean up first
			if ( $.inArray(type, dynamics) > -1 && $('#editor-settings > .active-settings > #' + type + '-settings').length ) {
				$('#editor-settings > .active-settings > #' + type + '-settings').remove();
			}

			if ( cache.popupMode && type == 'section' ) {
				$('#editor-settings > .active-settings > #' + type + '-settings').remove();
			}

			$('#editor-settings > .active-settings > .element-settings').hide();
			// check if settings for this element was previously opened
			if ( $('#editor-settings > .active-settings > #' + type + '-settings').length ) {
				$('#editor-settings > .active-settings > #' + type + '-settings').show();
				opened = true;
			} else {
				$('#editor-settings > .active-settings').append('<div id="' + type + '-settings" class="element-settings"></div>');
			}

			if ( !opened ) {
				// set up title
				this.settingTitle(type, alias);

				// setup fields
				settings.forEach(function(set) {
					// build header
					this.settingHeader(type, set);
					// build tabs
					this.settingTabs(type, set);
					// build fields
					this.settingFields(type, set);
				}, this);
			}

			// hide some section if necessary
			var parent = $('#editor-settings > .active-settings > #' + type + '-settings');
			settings.forEach(function(set, i) {
				if ( set.hasOwnProperty('condition') ) {
					switch (set.condition) {
						case 'normal-only':
							if ( cache.popupMode !== true ) {
								parent.find('.settings-header').eq(i).show();
								parent.find('.settings-fields').eq(i).show();
							} else {
								parent.find('.settings-header').eq(i).hide();
								parent.find('.settings-fields').eq(i).hide();
							}
							break;

						case 'popup-only':
							if ( cache.popupMode === true ) {
								parent.find('.settings-header').eq(i).show();
								parent.find('.settings-fields').eq(i).show();
							} else {
								parent.find('.settings-header').eq(i).hide();
								parent.find('.settings-fields').eq(i).hide();
							}
							break;
					}
				}
			}, this);

			// display layers and variants
			var variantsHtml = PrimeBuilder.Html.variantSettings(this.prime.variants);
			if ( $('#editor-settings > .split-settings > .global-fields').html() == '' ) {
				$('#editor-settings > .split-settings > .global-fields').html(variantsHtml);
			}
		},

		settingTitle: function(type, alias) {
			var data = this.prime.getElementData(type, alias),
				cache = this.prime.cache,
				icon = cache.popupMode && type == 'section' ? 'fas fa-fire' : data.icon,
				title = cache.popupMode && type == 'section' ? 'Pop Up Settings' : data.title;

			this.prime.loadHtml('settingTitle', document.querySelector('#editor-settings > .active-settings > #' + type + '-settings'));
			$('#editor-settings > .active-settings > #' + type + '-settings').find('h4.settings-title').find('i').attr('class', icon);
			$('#editor-settings > .active-settings > #' + type + '-settings').find('h4.settings-title').find('span').text(this.prime.utils.ucwords(title));
		},

		settingHeader: function(type, set) {
			if ( typeof set === "undefined" || !set.hasOwnProperty('name') ) {
				return;
			}

			var prime = this.prime,
				settings = prime.settings;
			if ( !settings.hasOwnProperty(set.name) ) {
				return;
			}
			
			var obj = settings[set.name],
				title = set.title || obj.title,
				mode = obj.mode || false,
				id = type + '-' + obj.id,
				defaultMode = obj.defaultMode || false;

			prime.loadHtml('settingHeader', document.querySelector('#editor-settings > .active-settings > #' + type + '-settings'), id);
			$('#' + id + ' > span').text(title);
			if ( mode ) {
				$('#' + id + ' > .settings-mode').removeClass('hidden');
			}
			
			$('#' + id).attr('data-type', type);
			$('#' + id).attr('data-for', obj.id);

			// prepare wrapper for tabs & fields
			var prevID = id,
				fieldsID = 'fields-' + type + '-' + obj.id,
				tabsID = fieldsID + '-tabs';

			if ( obj.hasOwnProperty('tabs') ) {
				prime.loadHtml('settingTabs', document.querySelector('#editor-settings > .active-settings > #' + type + '-settings'), tabsID);
				prevID = tabsID;
			}

			prime.loadHtml('settingFields', document.querySelector('#editor-settings > .active-settings > #' + type + '-settings'), fieldsID);

			// show setting header
			$('#' + id).show();
		},

		settingTabs: function(type, set) {
			if ( typeof set === "undefined" || !set.hasOwnProperty('name') ) {
				return;
			}

			var prime = this.prime,
				settings = prime.settings;

			if ( !settings.hasOwnProperty(set.name) ) {
				return;
			}

			var obj = settings[set.name],
				id = 'fields-' + type + '-' + obj.id + '-tabs';
			if ( !obj.hasOwnProperty('tabs') || !$.isArray(obj.tabs) || !obj.tabs.length ) {
				return;
			}

			var output = '<div class="fields-group">';
			obj.tabs.forEach(function(tab){
				var classes = obj.defaultTab == tab.key ? 'setting-tab-link tab-active' : 'setting-tab-link';
				output += '<div class="field-flex">';
					output += '<a href="#' + tab.key + '" class="' + classes + '">' + tab.name + '</a>';
				output += '</div>';
			});
			output += '</div>';
			$('#' + id).append(output);
		},

		settingFields: function(type, set) {
			var prime = this.prime,
				cache = prime.cache,
				settings = prime.settings,
				html = prime.html,
				fields;
			
			if ( typeof set === "undefined" || !set.hasOwnProperty('name') ) {
				return;
			}

			fields = ( settings.hasOwnProperty(set.name) && typeof settings[set.name]['fields'] !== "undefined" ) ? settings[set.name]['fields'] : null;
			if ( fields ) {
				var id = 'fields-' + type + '-' + settings[set.name]['id'];
				fields.forEach(function(field, index){
					var output, options = {
						id: field.id,
						type: field.type,
						label: field.label,
						for: set.name,
						idx: index,
						desc: ( field.hasOwnProperty('desc') ? field['desc'] : '' ),
						classes: ( field.hasOwnProperty('classes') ? field['classes'] : '' ),
						options: field.hasOwnProperty('options') ? field['options'] : null,
						firstOption: field.hasOwnProperty('firstOption') ? field['firstOption'] : null,
						placeholder: ( field.hasOwnProperty('placeholder') ? field['placeholder'] : '' ),
						defaultValue: ( field.hasOwnProperty('defaultValue') ? field['defaultValue'] : '' ),
						mode: ( field.hasOwnProperty('mode') ? field['mode'] : 'none' ),
						reveal: ( field.hasOwnProperty('reveal') ? field['reveal'] : 'none' ),
						groupReveal: ( field.hasOwnProperty('groupReveal') ? field['groupReveal'] : 'none' ),
						negative: ( field.hasOwnProperty('negative') ? field['negative'] : false ),
						useMode: settings[set.name].hasOwnProperty('mode') ? settings[set.name]['mode'] : false,
						useTabs: settings[set.name].hasOwnProperty('tabs') ? settings[set.name]['tabs'] : false
					};

					// mode 
					if ( settings[set.name].hasOwnProperty('defaultMode') ) {
						options.defaultMode = settings[set.name]['defaultMode'];
					}

					// tabs
					if ( settings[set.name].hasOwnProperty('defaultTab') ) {
						options.defaultTab = settings[set.name]['defaultTab'];
					}

					// other options
					if ( field.hasOwnProperty('min') ) {
						options.min = field.min;
					}

					if ( field.hasOwnProperty('multiple') ) {
						options.multiple = field.multiple;
					}

					if ( field.hasOwnProperty('max') ) {
						options.max = field.max;
					}

					if ( field.hasOwnProperty('step') ) {
						options.step = field.step;
					}

					if ( field.hasOwnProperty('dynamic') ) {
						options.dynamic = field.dynamic;
					}
					
					if ( field.hasOwnProperty('condition') ) {
						options.condition = field.condition;
					}

					if ( field.hasOwnProperty('removeIcon') ) {
						options.removeIcon = field.removeIcon;
					}

					if ( typeof options.options === "string" && options.options == 'popupdata' ) {
						options.options = {
							none: '-- No Pop Up Selected--'
						};

						var popups = cache.contents.section, num = 0;
						for ( var pop in popups ) {
							if ( !popups[pop]['settings'].hasOwnProperty('popsetup') ) {
								continue;
							}

							if ( popups[pop]['settings']['popsetup']['popname'] === null ) {
								continue;
							}

							options.options[pop] = popups[pop]['settings']['popsetup']['popname'];
						}
					}

					// create html
					var show = true;
					output = html.getField(type, field.type, options);
					if ( field.hasOwnProperty('condition') && field.condition != type ) {
						show = false;
					}

					if ( show ) {
						$('#' + id).append(output);
					}

				}, this);
			}
		},

		getButtons: function(el, type, alias, hover) {
			var prime = this.prime,
				buttons,
				buttonSettings = this.getButtonSettings(el, type, alias, hover);

			if ( buttonSettings ) {
				this.addButtons(buttonSettings, hover);
			}

			var buttonObj = ( hover ? this.fbuttons : this.buttons );
			buttons = this.getButtonsHtml(buttonObj, type, alias, hover);
			this.buttons = null;
			this.fbuttons = null;
			return buttons;
		},

		getResizer: function(el, type, alias) {
			var prime = this.prime,
				resizer,
				isResizable = this.isResizable(el, type, alias);

			if ( isResizable !== false ) {
				return this.getResizerHtml(isResizable, type, alias);
			}
			return '';
		},

		/**
		 * Get default element buttons
		 * @return {Array} 
		 */
		getDefaultButtons: function() {
			var buttons = [], self = this;
			this.defaultButtons.forEach(function(element){
				buttons.push(Object.assign({}, self.defaultButton, element));
			});
			return buttons;
		},

		createButton: function(obj) {
			var actionclass = obj.hasOwnProperty('actionclass') ? obj.actionclass : 'el-btn-action',
				classes = actionclass + ' btn-' + obj.color + ' ' + obj.classes,
				atts = ' id="el-btn-' + obj.id + '" data-action="' + obj.action + '"',
				icon = obj.icon.indexOf('dashicons') ? '<span class="' + obj.icon + '"></span>' : '<i class="' + obj.icon + '"></i>';

			return '<button type="button" class="btn btn-xs ' + classes + '" title="' + obj.title + '"' + atts + '>\
						' + icon + '\
					</button>';
		},

		createFloatButton: function(obj) {
			var actionclass = obj.hasOwnProperty('actionclass') ? obj.actionclass : 'el-fbtn-action',
				classes = actionclass + ' btn-' + obj.color + ' ' + obj.classes,
				atts = ' id="el-fbtn-' + obj.id + '" data-action="' + obj.action + '"',
				icon = obj.icon.indexOf('dashicons') ? '<span class="' + obj.icon + '"></span>' : '<i class="' + obj.icon + '"></i>';

			return '<button type="button" class="btn btn-xs ' + classes + '" title="' + obj.title + '"' + atts + '>\
						' + icon + '\
					</button>';
		},

		getButtonsHtml: function(buttons, type, alias, hover) {
			var html = '',
				key = ( alias ? alias : type ),
				prime = this.prime,
				cache = prime.cache;

			hover = hover || false;

			if ( hover && (typeof buttons === "undefined" || buttons === null) ) {
				return html;
			}

			// try to load from cache first
			if ( !hover && cache.html.buttons.hasOwnProperty(key) ) {
				return cache.html.buttons[key];
			}

			buttons.forEach(function(btn) {
				if ( hover ) {
					html += this.createFloatButton(btn);
				} else {
					html += this.createButton(btn);
				}
			}, this);

			if ( !hover ) {
				cache.html.buttons[key] = html;
			}
			return html;
		},

		getResizerHtml: function(isResizable, type, alias) {
			var html = '',
				key = ( alias ? alias : type );

			if ( isResizable ) {
				html = this.prime.getHtml('resizer');
			}
			return html;
		},

		columnResizer: function(id, type) {
			var prime = this.prime,
				iframeDom = prime.frameBody,
				elem = iframeDom.find('#' + id),
				selector = (type == 'wsection') ? elem.find(' > .el-content > .el-content-inner > .el-cols') : elem.find(' > .el-cols'),
				html = prime.getHtml('colResizer'),
				resizer = '';

			// clean up other resizers
			$('.el-col-resizer').remove();
			if (selector.find("> .ib2-section-col").length > 1) {
				var loop = selector.find("> .ib2-section-col").length - 1;
				for ( var i = 0; i < loop; i++ ) {
					var firstCol = $(selector.find(" > .ib2-section-col").get(i)),
						secCol = $(selector.find(" > .ib2-section-col").get((i+1))),
						firstW = firstCol.outerWidth(),
						secW = secCol.outerWidth(),
						pos = firstCol.position(),
						left = (pos.left + firstW) - 1;

					resizer = html.replace('{id}', firstCol.attr('id') + '-col-resizer');
					resizer = resizer.replace('{id1}', firstCol.attr('id'));
					resizer = resizer.replace('{id2}', secCol.attr('id'));

					$('.el-selected').append(resizer);
					$('#' + firstCol.attr('id') + '-col-resizer').attr('data-pos', (i+1)).css('left', left);
				}
			}
		},

		/**
		 * Add element button
		 * @param {Object} button
		 */
		addButton: function(button, hover) {
			var buttons = [], 
				self = this,
				pos = button.pos || 'end',
				posid = button.posid || false,
				btnObj = ( hover ? this.fbuttons : this.buttons );

			button = Object.assign({}, self.defaultButton, button);
			if ( $.inArray(pos, ['before', 'after']) > -1 && posid !== false ) {
				btnObj.forEach(function(obj){
					if ( pos == 'before' && posid == obj.id ) {
						buttons.push(button);
					}

					buttons.push(obj);

					if ( pos == 'after' && posid == obj.id ) {
						buttons.push(button);
					}
				});

				if ( hover ) {
					this.fbuttons = buttons;
				} else {
					this.buttons = buttons;
				}
			} else if ( pos == 'first' && btnObj.length > 0 ) {
				if ( hover ) {
					this.fbuttons.unshift(button);
				} else {
					this.buttons.unshift(button);
				}
			} else {
				if ( hover ) {
					this.fbuttons.push(button);
				} else {
					this.buttons.push(button);
				}
			}
		},

		/**
		 * Add multiple element buttons
		 * @param {Array} buttons
		 */
		addButtons: function(buttons, hover) {
			if ( typeof buttons === "undefined" || !buttons.length ) {
				return;
			}

			var self = this;
			buttons.forEach(function(button){
				self.addButton(button, hover);
			});
		},

		getButtonSettings: function(el, type, alias, hover) {
			var element = this.prime.getElementData(type, alias),
				buttons;

			hover = hover || false;
			if ( !hover && this.buttons === null ) {
				this.buttons = this.getDefaultButtons();
			} else if ( hover && this.fbuttons === null ) {
				this.fbuttons = [];
			}

			if ( !hover && element.hasOwnProperty('buttons') ) {
				buttons = element.buttons;
			} else if ( hover && element.hasOwnProperty('floatButtons') ) {
				buttons = element.floatButtons;
			}
			return buttons;
		},

		resizefloatBars: function() {
			var prime = this.prime,
				iframeDom = prime.frameBody,
				left = iframeDom.find('#screen-container').offset().left,
				width = iframeDom.find('#screen-container').outerWidth();

			// infobar
			if ( iframeDom.find('#screen-container').find('.ib2-notification-bar').length ) {
				iframeDom.find('#screen-container')
					.find('.ib2-notification-bar')
					.css({
						'left': left,
						'width': width + 'px' 
					});
			}
		},

		resizeBackgrounds: function() {
			var prime = this.prime,
				selectors = window.FrameDocument.querySelectorAll('.is-background');

			if ( selectors ) {
				selectors.forEach(function(selector){
					var $this = $(selector),
						css = {
							maxWidth: '100%',
							width: '100%',
							height: $this.parent().css('height'),
						};

					$this.css(css);
				});
			}
		},

		resizeSelector: function() {
			var prime = this.prime,
				iframeDom = prime.frameBody,
				iframeDoc = prime.frameDoc,
				options = prime.options,
				elementSelector = options.element.selector,
				elID = prime.cache.activeElement;

			if ( $('.el-selected').is(":visible") && elID ) {
				var target = iframeDom.find('#' + elID);
				if ( target ) {
					if ( !target.hasClass(elementSelector) ) {
						target = target.find(" > ." + elementSelector);
					}
					var offset = target.offset(),
						width = target.outerWidth(),
						height = target.outerHeight();

					// element selected
					$('.el-selected').css({
						top: offset.top  - iframeDoc.scrollTop(),
						left: offset.left - iframeDoc.scrollLeft(),
						width: width + 'px',
						height: height + 'px'
					});

					// reposition resizer
					if ( $('.el-selected').find('.el-resize-handlers').length ) {
						var w = $('.el-selected').innerWidth(), h = $('.el-selected').innerHeight();
						$('.el-n-resize').css({ top: '-5px', left: ((w/2)-4) + 'px' });
						$('.el-s-resize').css({ top: (h-4) + 'px', left: ((w/2)-4) + 'px' });
						$('.el-w-resize').css({ top: ((h/2)-4) + 'px', left: '-6px' });
						$('.el-e-resize').css({ top: ((h/2)-4) + 'px', left: (w-4) + 'px' });
						$('.el-se-resize').css({ top: (h-4) + 'px', left: (w-4) + 'px' });
					}
				}
			}
		},

		resizeHighlight: function() {
			var prime = this.prime,
				interact = prime.interact,
				cache = prime.cache,
				iframeDoc = prime.frameDoc;

			if ( cache.hoveredEl ) {
				var target = interact.tuneElementTarget(cache.hoveredEl),
					offset = target.offset();

				$('.el-highlight').css({
					top: offset.top - iframeDoc.scrollTop(),
					left: offset.left - iframeDoc.scrollLeft(),
					width: target.outerWidth() + 'px',
					height: target.outerHeight() + 'px'
				});
			}
		},

		isResizable: function (el, type, alias) {
			var element = this.prime.getElementData(type, alias);
			if ( element.hasOwnProperty('resizable') ) {
				return element.resizable;
			}

			return false;
		},

		countdownUpdate: function (iframe, id, time, type) {
			var defaultTime = {
				days: 0,
				hours: 0,
				minutes: 0,
				seconds: 0
			};

			time = Object.assign({}, defaultTime, time);
			if ( iframe.find('#' + id + '-tseconds').length ) {
				iframe.find('#' + id + '-tseconds').text(time.seconds);
				iframe.find('#' + id + '-tminutes').text(time.minutes);
				iframe.find('#' + id + '-thours').text(time.hours);
				iframe.find('#' + id + '-tdays').text(time.days);
				
				if ( time.seconds > 1 )
					iframe.find('#' + id + '-tseconds').next('.ib2-digit-txt').text(' seconds ');
				else
					iframe.find('#' + id + '-tseconds').next('.ib2-digit-txt').text(' second ');
					
				if ( time.minutes > 1 )
					iframe.find('#' + id + '-tminutes').next('.ib2-digit-txt').text(' minutes ');
				else
					iframe.find('#' + id + '-tminutes').next('.ib2-digit-txt').text(' minute ');
					
				if ( time.hours > 1 )
					iframe.find('#' + id + '-thours').next('.ib2-digit-txt').text(' hours ');
				else
					iframe.find('#' + id + '-thours').next('.ib2-digit-txt').text(' hour ');
					
				if ( time.days > 1 )
					iframe.find('#' + id + '-tdays').next('.ib2-digit-txt').text(' days ');
				else
					iframe.find('#' + id + '-tdays').next('.ib2-digit-txt').text(' day ');
			}

			if ( type == 'circular' ) {
				iframe.find('#' + id).find('.clock-days').find('.clock-anim').css('stroke-dashoffset', this.getDashArray(time.days, 365)); // days
				iframe.find('#' + id).find('.clock-hours').find('.clock-anim').css('stroke-dashoffset', this.getDashArray(time.hours, 24)); // days
				iframe.find('#' + id).find('.clock-minutes').find('.clock-anim').css('stroke-dashoffset', this.getDashArray(time.minutes, 60)); // days
				iframe.find('#' + id).find('.clock-seconds').find('.clock-anim').css('stroke-dashoffset', this.getDashArray(time.seconds, 60)); // days
			}
		},

		getDashArray: function(num, max) {
			var radius = 54,
				circumference = 2 * Math.PI * radius,
				dashoffset = circumference - ( (max-num) * (circumference/max) );

			//console.log('progress:', progress, '|', 'offset:', dashoffset)
			return dashoffset;
		},
	};

})(PrimeBuilder, window, document, jQuery);