(function (PrimeBuilder, w, d, $) { 
	"use strict";

	function debounce(func, wait, immediate) {
		var timeout;
		return function() {
			var context = this, args = arguments;
			var later = function() {
				timeout = null;
				if (!immediate) func.apply(context, args);
			};
			var callNow = immediate && !timeout;
			clearTimeout(timeout);
			timeout = setTimeout(later, wait);
			if (callNow) func.apply(context, args);
		};
	}

	function scrollingDown() {
		$(window.FrameDocument).find('body').find('.elements-icons').stop().animate({scrollTop:'+=30'}, 300, 'linear', scrollingDown);
	}

	function scrollingUp() {
		$(window.FrameDocument).find('body').find('.elements-icons').stop().animate({scrollTop:'-=30'}, 300, 'linear', scrollingUp);
	}

	function updateEditorByHistory(data) {
		$('body').trigger('restoreHistory', [data]);
	}

	var ballPos = {
		id: '',
		curX: 0,
		curY: 0,
		oldX: 0,
		oldY: 0,
		bounceX: 0,
		bounceY: 0,
		top: 0,
		left: 0
	},
	resizeEl = {
		id: '',
		type: '',
		dir: 'e',
		inc: 'width',
		origWidth: 0,
		origHeight: 0,
		curX: 0,
		curY: 0,
		oldX: 0,
		oldY: 0,
		bounceX: 0,
		top: 0,
		left: 0,
		rect: null,
	},
	resizeCol = {
		col1: '',
		col2: '',
		firstResize: true,
		minLeft: 0,
		offsetX: 0,
		prevLeft: 0,
		nextLeft: 0,
		ghost: null,
		maxLeft: 100,
	};

	/**
	 * Editor Interaction
	 */
	PrimeBuilder.Interact = function (prime) {
		this.prime = prime;
		this.isClicked = false;
		this.oldValue = null;
		this.lastVarWeight = 0;
		this.templateType = 'sales';
		this.templateText = 'Sales';
		
	};

	PrimeBuilder.Interact.prototype = {

		setup: function() {
			var prime = this.prime;
			this.pageSettings();
			this.elementOver();
			this.elementClick();
			this.elementsScroll();
			this.settingTabs();
			this.settingToggle();
			this.closeSettings();
			this.wpMediaHandler();
			this.settingsHandler();
			this.actionButtonHandler();
			this.toggleMode();
			this.handleImageUpload();
			this.removeImage();
			this.imagePositionBall();
			this.elementResize();
			this.columnResize();
			this.parseHTMLForm();
			this.editFormEvents();
			this.tabs();
			this.savePage();
			this.history();
			this.insertElement();
			this.mediaHandler();
			this.disableClicks();
			this.popupEditorClose();
			this.conversionHandler();
			this.templateHandler();
			this.apiConnect();
			this.generateLegal();

			// CONFIRM ON EDITOR EXIT
			$(window).bind('beforeunload', function(e){
				if ( prime.dirty !== false ) {
					return 'Are you sure you want to quit the editor?\nAll unsaved progress will be lost.';
				}
			});
		},

		generateLegal: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui =  prime.gui,
				action = prime.action,
				iframeDom = prime.frameBody;

			$('body').on('submit', 'form#generate-legal-form', function(e){
				var $this = $(this),
					elID = cache.activeElement,
					type = $('#ib2-legal-type').val(),
					name = $('#ib2-legal-name').val(),
					url = $('#ib2-legal-url').val(),
					country = $('#ib2-legal-country').val(),
					content = $('#ib2-' + type + '-html').html();

				if ( confirm("The current text element content will be replaced.\nDo you want to continue?") ) {
					var b_name = name !== '' ? name : '[BUSINESS NAME]',
						b_url = url !== '' ? url : '[URL]',
						b_country = country !== '' ? country : '[COUNTRY]';

					content = content.replace(new RegExp('{%BUSINESS_NAME%}', 'g'), b_name);
					content = content.replace(new RegExp('{%URL%}', 'g'), b_url);
					content = content.replace(new RegExp('{%COUNTRY%}', 'g'), b_country);
					
					iframeDom.find('#' + elID).html(content);
					action.closePopup('legal-generator', true);

					gui.resizeBackgrounds();
					gui.resizeSelector();
				}
				
				e.preventDefault();
			});
		},

		apiConnect: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				action = prime.action;

			$('body').on('submit', 'form#save-api-form', function(e){
				var $this = $(this),
					error = 0,
					provider = $('#api-provider').val();

				// validate
				$this.find('.api-field').each(function(){
					var field = $(this);
					if ( field.is(":visible") ) {
						if ( field.find('.form-control').val() == '' ) {
							error++;
							field.find('.text-danger').text('This field is required');
							field.addClass('has-error');
						} else {
							field.find('.text-danger').text('');
							field.removeClass('has-error');
						}
					}
				});

				if ( error ) {
					return false;
				}

				$this.find('button').button('loading');

				// prepare data
				var fields = {};
				$this.find('.api-field').each(function(){
					var field = $(this);
					if ( field.is(":visible") ) {
						var fieldId = field.find('.form-control').attr('id');
						fields[fieldId] = $.trim(field.find('.form-control').val());
					}
				});
				fields['provider'] = provider;

				$.ajax({
					type: "POST",
					url: ajaxurl,
					data: {
						action: 'ib2_connect_api',
						fields: fields,
					},
					success: function(response) {
						//console.log(response);
						if ( response.success ) {
							gui.updateListOptions(response.lists);
							prime.esplists[provider] = response.lists;
							ib2esp[provider] = response.token;
							$('#esp-connect-button').addClass('hidden');
							$('#api-connect-error').addClass('hidden');
							$('.esp-refresh-btn').removeClass('hidden');
							setTimeout(function(){
								action.closePopup('api-integration', true);
							}, 100);
						} else {
							$('#api-connect-error').removeClass('hidden');
						}
						$this.find('button').button('reset');
					},
					error: function(jqXHR, textStatus, errorThrown) {
						$('#api-connect-error').removeClass('hidden');
						$this.find('button').button('reset');
					}
				});
				e.preventDefault();
			});
		},

		templateHandler: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				action = prime.action;

			$('body').on('click', '.ib2-tmpl-type', function(e){
				var $this = $(this),
					type = $this.attr('data-type'),
					text = $this.text();

				// set active button
				$this.closest('.change-template-type').find('.active').removeClass('active');
				$this.parent().addClass('active');

				prime.loadTemplates(type, text);
				self.templateType = type;
				self.templateText = text;
				e.preventDefault();
			});

			$('body').on('change', '#ib2-tmpl-subtype', function(e){
				var $this = $(this),
					type = self.templateType,
					text = self.templateText;

				prime.loadTemplates(type, text, true);

				e.preventDefault();
			});

			$('body').on('submit', '#template-search-form', function(e){
				var $this = $(this),
					type = self.templateType,
					text = self.templateText;

				prime.loadTemplates(type, text, true);

				e.preventDefault();
				return false;
			});
			
			$('body').on('mouseenter', '.ib2-tmpl-thumb', function(e){
				var $this = $(this);
				if ( !$this.find('.ib2-select-tmpl').length && !$this.find('.ib2-choose-bg').length ) {
					$this.append('<div class="ib2-choose-bg"></div>');
					
					if ( cache.changeTemplate ) {
						$this.append('<a href="' + $('#ib2-admin-url').val() + '?post_id=' + $('#ib2-post-id').val() + '&variant=' + $('#ib2-current-variation').val() + '&ib2action=change_template&template_id=' + $this.data('id') + '" class="btn btn-primary ib2-choose-tmpl">Choose</a>');
					} else {
						$this.append('<a href="#" class="btn btn-primary ib2-choose-tmpl ib2-create-var-link" data-tid="' + $this.data('id') + '">Choose</a>');
					}
					
					var top  = (($this.height() / 2) - ($this.find('.ib2-choose-tmpl').outerHeight() / 2));
					var left = (($this.width() / 2) - ($this.find('.ib2-choose-tmpl').outerWidth() / 2));

					if ( top < 0 ) top = 0;
					if ( left < 0 ) left = 0;
				
					$this.find('.ib2-choose-bg').css({
						'position': 'absolute',
						'background-color': '#FFF',
						'opacity': '0.6',
						'width': $this.width() + 'px',
						'height': $this.height() + 'px',
						'left': 0,
						'top': 0,
						'z-index': '8'
					});
					
					$this.find('.ib2-choose-tmpl').css({
						'position': 'absolute',
						'display': 'block',
						'left': left,
						'top': top,
						'z-index': '10'
					});
				} else {
					$this.find('.ib2-choose-bg').show();
					$this.find('.ib2-choose-tmpl').show();
				}
			});
			
			$('body').on('mouseleave', '.ib2-tmpl-thumb', function(e){
				var $this = $(this);
				$this.find('.ib2-choose-tmpl').remove();
				$this.find('.ib2-choose-bg').remove();
			});

			// VARIANT
			$('body').on('click', '.variant-create', function(e){
				var $this = $(this),
					type = $this.attr('data-type');

				cache.variantType = type;
				if ( type == 'template' ) {
					$('.ib2-tmpls').show();
					return true;
				} else {
					self.savePageData(self);
				}
				e.preventDefault();
			});

			$('body').on('click', '.ib2-create-var-link', function(e){
				var $this = $(this),
					tid = $this.attr('data-tid');

				cache.templateId = tid;
				self.savePageData(self);

				e.preventDefault();
			});

			// SAVE TEMPLATE
			$('body').on('change', '#template-type', function(e){
				var options = '',
					type = $("option:selected", this).val(),
					config = ib2templateconfig;

				options += '<option value=""> -- select page sub-type -- </option>';
				if ( config.hasOwnProperty(type) && config[type] !== null ) {
					var subtypes = config[type];
					for ( var key in subtypes ) {
						options += '<option value="' + key + '">' + subtypes[key] + '</option>';
					}
				}
				$('#template-subtype').html(options);
			});

			$('body').on('template-thumb', function(e){
				var img = $('#template-thumb-url').val();

				if ( img !== '' ) {
					$('#template-thumb-preview').html('<img src="' + img + '" class="img-responsive">');
					$('#remove-template-thumb').show();
				} else {
					$('#template-thumb-preview').html('');
					$('#remove-template-thumb').hide();
				}
			});

			$('body').on('click', '#remove-template-thumb', function(e){
				var $this = $(this);

				$('#template-thumb-url').val('');
				$('body').trigger('template-thumb');
				e.preventDefault();
			});

			$('body').on('submit', 'form#save-template-form', function(e){
				var $this = $(this);
				e.preventDefault();
				if ( $('#template-name').val() == '' ) {
					$('#template-name').focus();
					$('#template-name').closest('.form-group').addClass('has-error');
					$('#template-name').parent().find('.text-danger').text('Please enter a name for your template.');
					return false;
				}
				// check field name...
				$('#template-save').button('loading');
				
				// check if template exist
				if ( !$('#template-overwrite').is(":checked") ) {
					$.ajax({
						type: "GET",
						url: ajaxurl,
						data: {
							action: 'ib2_check_template',
							name: $('#template-name').val(),
							type: $('#template-type').val(),
						},
						success: function(response) {
							if ( response.status == 'exist') {
								$('#tmpl-duplicate-msg').show();
								$('#template-save').button('reset');
							} else {
								// save template
								cache.isSaveTemplate = true;
								self.savePageData(self);
							}
						}
					});
				} else {
					// save template
					cache.isSaveTemplate = true;
					self.savePageData(self);
				}
			});

			$('body').on('click', '.ib2-element-type', function(e){
				var $this = $(this),
					elType = $this.attr('data-type');

				if ( $this.parent().hasClass('active') ) {
					return;
				}

				$this.closest('.change-element-type').find('.active').removeClass('active');
				$this.parent().addClass('active');

				$('.element-cat-title').text(prime.utils.ucwords(elType) + ' Elements');
				$('.element-contents').find('.active').removeClass('active');

				$('.' + elType + '-elements').addClass('active');

				if ( elType == 'saved' ) {
					if ( !cache.savedElementsLoaded ) {
						prime.loadSavedElements();
						cache.savedElementsLoaded = true;
					} else {
						$('.ib2-elements-content').html($('#elements-placeholder').html());
						$('.ib2-element-loader').hide();
						$('.ib2-elements-content').find('.ib2-elem-thumb > img').each(function(){
							$(this).attr('src', $(this).attr('data-src'));
						});
						action.centerPopup('element-manager');
					}
				}

				e.preventDefault();
			});

			$('body').on('mouseenter', '.ib2-elem-thumb', function(e){
				var $this = $(this);
				$this.append('<div class="ib2-choose-bg"></div>');
				$this.append('<a href="#" class="btn btn-primary ib2-choose-elem ib2-add-saved-element" data-id="' + $this.data('id') + '" data-name="' + $this.data('name') + '">Choose</a>');
				
				var top  = (($this.height() / 2) - ($this.find('.ib2-choose-elem').outerHeight() / 2));
				var left = (($this.width() / 2) - ($this.find('.ib2-choose-elem').outerWidth() / 2));

				if ( top < 0 ) top = 0;
				if ( left < 0 ) left = 0;
			
				$this.find('.ib2-choose-bg').css({
					'position': 'absolute',
					'background-color': '#FFF',
					'opacity': '0.6',
					'width': $this.width() + 'px',
					'height': $this.height() + 'px',
					'left': 0,
					'top': 0,
					'z-index': '8'
				});
				
				$this.find('.ib2-choose-elem').css({
					'position': 'absolute',
					'display': 'block',
					'left': left,
					'top': top,
					'z-index': '10'
				});
			});
			
			$('body').on('mouseleave', '.ib2-elem-thumb', function(e){
				var $this = $(this);
				$this.find('.ib2-choose-elem').remove();
				$this.find('.ib2-choose-bg').remove();
			});

			var iframeDom = prime.frameBody;
			$('body').on('submit', 'form#save-element-form', function(e){
				var $this = $(this),
					elID = $('#save-element-id').val();

				e.preventDefault();
				if ( $('#element-name').val() == '' ) {
					$('#element-name').focus();
					$('#element-name').closest('.form-group').addClass('has-error');
					$('#element-name').parent().find('.text-danger').text('Please enter a name for your element.');
					return false;
				}
				// change button state...
				$('#element-save').button('loading');
				
				// check if template exist
				if ( !$('#element-overwrite').is(":checked") ) {
					$.ajax({
						type: "GET",
						url: ajaxurl,
						data: {
							action: 'ib2_check_element',
							name: $('#element-name').val(),
							type: iframeDom.find('#' + elID).attr('data-el'),
						},
						success: function(response) {
							if ( response.status == 'exist') {
								$('#elem-duplicate-msg').show();
								$('#element-save').button('reset');
							} else {
								// save template
								self.saveElement(self, elID);
							}
						}
					});
				} else {
					// save template
					self.saveElement(self, elID);
				}
			});
		},

		saveElement: function(self, id) {
			var prime = self.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				settings = cache.contents,
				cssRules = cache.cssRules,
				videos = cache.videos,
				type = iframeDom.find('#' + id).attr('data-el'),
				data = {};

			if ( type ) {
				var setting = settings[type][id],
					clone = iframeDom.find('#' + id).clone(); 
				
				$('#content-placeholder').html('');
				clone.appendTo('#content-placeholder');

				self.prepareContent(self);
				var content = $('#content-placeholder').html();

				// prepare data
				data.settings = {};
				data.cssRules = {};
				data.videos = {};

				// save main element data
				data.settings[type] = {};
				data.settings[type][id] = settings[type][id];
				if ( cssRules.hasOwnProperty(id) ) {
					data.cssRules[id] = cssRules[id];
				}
				if ( videos.hasOwnProperty(id) ) {
					data.videos[id] = videos[id];
				}

				// save childs data
				if ( $('#content-placeholder').find('.ib2-element').length ) {
					$('#content-placeholder').find('.ib2-element').each(function(i){
						var child = $(this),
							childID = child.attr('id'),
							childType = child.attr('data-el');

						if ( !data.settings.hasOwnProperty(childType) ) {
							data.settings[childType] = {};
						}

						data.settings[childType][childID] = settings[childType][childID];
						if ( cssRules.hasOwnProperty(childID) ) {
							data.cssRules[childID] = cssRules[childID];
						}
						if ( videos.hasOwnProperty(childID) ) {
							data.videos[childID] = videos[childID];
						}
					});
				}

				var configObj = JSON.stringify(data),
					htmlFile = new Blob([content], {type: "text/plain;charset=utf-8"}, 'html.txt'),
					configFile = new Blob([configObj], {type: "text/plain;charset=utf-8"}, 'data.txt'),
					fd = new FormData();

				fd.append('action', 'ib2_save_element');
				fd.append('name', $('#element-name').val());
				fd.append('thumb', $('#template-thumb-url').val());
				fd.append('type', type);
				fd.append('html', htmlFile);
				fd.append('config', configFile);
				fd.append('userid', $('#ib2-user-id').val());

				var sendhtml = $.ajax({
					type: "POST",
					url: ajaxurl,
					data: fd,
					processData: false,
	    			contentType: false,
					success: function(response) {
						$('#element-save').button('reset');
						cache.savedElementsLoaded = false;
					}
				});
			}
		},

		conversionHandler: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody;

			$('body').on('change', '#conversion-type', function(e){
				var opt = $("option:selected", this).val();
				if ( opt == 'wp' ) {
					$('.conversion-wp').removeClass('hidden');
					$('.conversion-ext').addClass('hidden');
				} else {
					$('.conversion-ext').removeClass('hidden');
					$('.conversion-wp').addClass('hidden');
				}
				cache.contents['page']['type']['conversion_type'] = opt;
			});

			$('body').on('change', '#conversion-page', function(e){
				var opt = $("option:selected", this).val();
				cache.contents['page']['type']['conversion_id'] = parseInt(opt);
			});

			// variants
			$('body').on('focusin', ".variant-weights", function(e){
				var $this = $(this),
					oldVal = $this.val();

				self.lastVarWeight = oldVal;
			});

			$('body').on("blur change", ".variant-weights", function(){
				var $this = $(this),
					maxnum = 0;

				$(".variant-weights").each(function(){
					var num = parseInt($(this).val());
					maxnum += num;
				});

				if ( maxnum > 100 ) {
					$this.val(self.lastVarWeight);
					alert("Total weight of all variants cannot exceed 100%");
					$this.focus();
				}
			});
		},

		popupEditorClose: function() {
			var prime = this.prime,
				cache = prime.cache,
				action = prime.action,
				iframeDom = prime.frameBody;

			iframeDom.on('click', '.ib2-close-pop', function(e){
				var popupId = cache.sliderEdit ? cache.sliderEdit : cache.popupId;
				action.closePopupEditor(prime, popupId);

				setTimeout(function(){
					// clean up
					$('.el-highlight').hide();
					$('.el-selected > .el-resize-handlers').html('');
					$('.el-selected').hide();
				}, 1000);
				e.preventDefault();
			});

			$('body').on('click', '.ib2-close-popup', function(e){
				var popupId = cache.sliderEdit ? cache.sliderEdit : cache.popupId;
				action.closePopupEditor(prime, popupId);

				setTimeout(function(){
					// clean up
					$('.el-highlight').hide();
					$('.el-selected > .el-resize-handlers').html('');
					$('.el-selected').hide();
				}, 1000);
				e.preventDefault();
				return false;
			});
		},

		disableClicks: function() {
			var prime = this.prime,
				iframeDom = prime.frameBody;

			iframeDom.find('#screen-container').on("click", "a, button, input", function(e){
				e.preventDefault();
			});
		},

		mediaHandler: function() {
			var prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				action = prime.action;

			$('body').on('mouseenter', '.ib2-img-media', function(e){
				$(this).parent().addClass('ib2-img-media-active');
			});

			$('body').on('mouseout', '.ib2-img-media', function(e){
				$(this).parent().removeClass('ib2-img-media-active');
			});

			$('body').on('fillmedia', function(e, data, settingFor, idx, pid){
				var images = '';
				if ( data.length ) {
					data.forEach(function(img){
						images += '<div class="item" style="width: ' + img.width + 'px; height:100px"><img src="' + img.url + '" class="ib2-img-media" title="Click to insert" data-id="' + pid + '" data-for="' + settingFor + '" data-idx="' + idx + '"></div>';
					});
				} else {
					images += '<p class="help-block"><em>No images found</em></p>';
				}
				$('.media-loader').addClass('hidden');
				$('.media-help').removeClass('hidden');
				$('.ib2-media-images').html(images);
			});

			$('body').on('click', '.ib2-img-media', function(e){
				var url = $(this).attr('src'),
					settingFor = $(this).attr('data-for'),
					idx = $(this).attr('data-idx'),
					pid = $(this).attr('data-id'),
					attachment = {
						url: url,
					};

				if ( url.indexOf('pixabay.com') > -1 ) {
					$('.media-loader').removeClass('hidden');
					$('.media-help').addClass('hidden');
					$.ajax({
						type: "POST",
						url: ajaxurl,
						data: {
							action: 'ib2_download_image',
							img_url: url,
							post_id: $('#ib2-post-id').val()
						},
						success: function(response) {
							if ( response.success ) {
								var attach = {
									url: response.img 
								};
								$('body').trigger("mediaselected", [attach, settingFor, idx, pid]);
							}
							$('.media-loader').addClass('hidden');
							$('.media-help').removeClass('hidden');

							setTimeout(function(){
								action.closePopup('media-manager', true, true);
							}, 1000);
						}
					});
				} else {
					$('body').trigger("mediaselected", [attachment, settingFor, idx, pid]);
					setTimeout(function(){
						action.closePopup('media-manager', true, true);
					}, 1000);
				}
				
				e.preventDefault();
			});

			$('body').on('change', '#ib2-media-cat', function(e){
				var $this = $(this),
					opt = $("option:selected", this).val(),
					settingFor = $(this).attr('data-for'),
					idx = $(this).attr('data-idx'),
					pid = $(this).attr('data-id');

				$('.media-loader').removeClass('hidden');
				$('.media-help').addClass('hidden');
				$('.ib2-media-images').html('');

				if ( opt == 'pixabay' ) {
					$('.search-image-form').removeClass('hidden');
					$('.media-loader').addClass('hidden');
					//$('.media-help').removeClass('hidden');
					$('.ib2-media-images').html('<p class="help-block"><em>No Search Result</em></p>');
				} else {
					$('.search-image-form').addClass('hidden');
					if ( !cache.mediaResults.hasOwnProperty(opt) ) {
						$.ajax({
							type: "GET",
							url: ajaxurl,
							data: {
								action: 'ib2_get_media',
								cat: opt,
							},
							success: function(response) {
								cache.mediaResults[opt] = response;
								$('body').trigger("fillmedia", [response, settingFor, idx, pid]);
							}
						});
					} else {
						$('body').trigger("fillmedia", [cache.mediaResults[opt], settingFor, idx, pid]);
					}
				}
			});

			$('body').on('submit', 'form#ib2-media-form', function(e){
				e.preventDefault();
				var q = $.trim($('#media-keyword').val()),
					settingFor = $('#ib2-media-cat').attr('data-for'),
					idx = $('#ib2-media-cat').attr('data-idx'),
					pid = $('#ib2-media-cat').attr('data-id');

				if ( q == '' ) {
					return;
				}

				$('.media-loader').removeClass('hidden');
				$('.ib2-media-images').html('');
				var btn = $('#search-image-btn').button('loading');
				$.ajax({
					type: "GET",
					url: ajaxurl,
					data: {
						action: 'ib2_search_media',
						provider: $('#ib2-media-cat').val(),
						q: q,
						page: 1
					},
					success: function(response) {
						cache.mediaResults['pixabay'] = response;
						$('body').trigger("fillmedia", [response, settingFor, idx, pid]);
						$('#search-image-btn').button('reset');
					}
				});
				return false;
			});

			// ICONS
			$('body').on('change', '#ib2-icon-cat', function(e){
				var $this = $(this),
					opt = $("option:selected", this).val(),
					content = '';

				prime.faIcons[opt].forEach(function(icon){
					content += '<div class="insert-element">';
						content += '<a href="javascript:void(0)" class="insert-icon-btn" data-icon="' + icon + '">';
						content += '<i class="' + icon + '"></i>';
					content += '</a></div>';
				}, this);
				content += '<div class="clearfix"></div>';

				$('#icon-manager').find('.insert-elements').html(content);
				cache.lastIcons = opt;
			});

			$('body').on('click', '.insert-icon-btn', function(e){
				var $this = $(this),
					elID = cache.activeElement,
					type = cache.activeElementType,
					icon = $this.attr('data-icon');

				if ( type == 'icon' ) {
					$('#icon-icon-shape-icon').val(icon).change();
				} else {
					$('#icon-shape-icon-' + type).val(icon).change();
				}

				action.closePopup('icon-manager', true, true);
				e.preventDefault();
			});

			iframeDom.on('click', '.ib2-navbar-toggle', function(e){
				var $this = $(this);

				if ( $this.parent().find('nav').hasClass('menu-show') ) {
					$this.parent().find('nav').removeClass('menu-show');
				} else {
					$this.parent().find('nav').addClass('menu-show');
				}

				e.preventDefault();
			});
		},

		insertElement: function() {
			var prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				action = prime.action,
				iframeDom = prime.frameBody;

			$('body').on("click", '.insert-element-btn', function(e){
				var $this = $(this),
					parentEl = cache.hoveredEl,
					type = $this.attr('data-el'),
					elem = iframeDom.find('.element-type-' + type),
					pos = $this.attr('data-pos');

				if ( parentEl.find('.ib2-element').length ) {
					pos = (pos == 'before' ? 'afterbegin' : 'beforeend');
				}

				cache.isAdding = true;
				cache.isDragging = true;

				prime.addElement(parentEl, elem, pos);
				action.closePopup('element-manager', true);

				e.preventDefault();
			});

			$('body').on("click", '#desktop-view', function(e){
				if ( !$('#mobile-view').is(":checked") && !$('#tablet-view').is(":checked") ) {
					$('#visibility-alert').removeClass('hidden');

					setTimeout(function(){
						$('#visibility-alert').addClass('hidden');
					}, 3000);

					e.preventDefault();
					return false;
				}

				var elID = cache.activeElement,
					elem = iframeDom.find('#' + elID);

				if ( $(this).is(":checked") ) {
					elem.removeClass('hidden-lg hidden-md');
				} else {
					elem.addClass('hidden-lg hidden-md');
				}
			});

			$('body').on("click", '#mobile-view', function(e){
				if ( !$('#desktop-view').is(":checked") && !$('#tablet-view').is(":checked") ) {
					$('#visibility-alert').removeClass('hidden');

					setTimeout(function(){
						$('#visibility-alert').addClass('hidden');
					}, 3000);

					e.preventDefault();
					return false;
				}

				var elID = cache.activeElement,
					elem = iframeDom.find('#' + elID);

				if ( $(this).is(":checked") ) {
					elem.removeClass('hidden-xs');
				} else {
					elem.addClass('hidden-xs');
				}
			});

			$('body').on("click", '#tablet-view', function(e){
				if ( !$('#desktop-view').is(":checked") && !$('#mobile-view').is(":checked") ) {
					$('#visibility-alert').removeClass('hidden');

					setTimeout(function(){
						$('#visibility-alert').addClass('hidden');
					}, 3000);

					e.preventDefault();
					return false;
				}

				var elID = cache.activeElement,
					elem = iframeDom.find('#' + elID);

				if ( $(this).is(":checked") ) {
					elem.removeClass('hidden-sm');
				} else {
					elem.addClass('hidden-sm');
				}
			});

			this.insertSavedElement();
		},

		insertSavedElement: function() {
			var prime = this.prime,
				gui = prime.gui,
				iframeDom = prime.frameBody,
				iframeHtml = prime.frameHtml,
				cache = prime.cache,
				action = prime.action;

			$('body').on("click", '.ib2-choose-elem', function(e){
				var $this = $(this),
					parentEl = cache.hoveredEl,
					id = $this.attr('data-id'),
					name = $this.attr('data-name'),
					pos = $('.ib2-elements-content').attr('data-pos');

				cache.isAdding = true;
				cache.isDragging = true;

				$.ajax({
					type: "POST",
					url: ajaxurl,
					data: {
						action: 'ib2_load_saved_element',
						element_id: id,
					},
					success: function(response) {
						if ( parentEl && response.success ) {
							var temp = $('#content-placeholder'),
								html = response.html,
								config = JSON.parse(response.config),
								sections = ['section', 'wsection'],
								buildCss = false;

							if ( config && config.hasOwnProperty('settings') ) {
								temp.html(response.html);
								var oldEl = temp.find('.ib2-element').eq(0),
									oldID = oldEl.attr('id'),
									type = response.type,
									newID = prime.generateID(8, 'ib2_el_'),
									setType = config.settings.hasOwnProperty(type) ? config.settings[type] : false,
									settings = setType && setType.hasOwnProperty(oldID) ? setType[oldID] : false;

								if ( settings ) {
									html = html.replace(oldID, newID);
									settings['id'] = newID;

									// save the main element data
									if ( !cache.contents.hasOwnProperty(type) ) {
										cache.contents[type] = {};
									}
									cache.contents[type][newID] = settings;

									// do we have any css
									if ( config.cssRules.hasOwnProperty(oldID) ) {
										cache.cssRules[newID] = config.cssRules[oldID];
										cache.cssRules[newID].forEach(function(rule, i){
											cache.cssRules[newID][i]['selector'] = cache.cssRules[newID][i]['selector'].replace(oldID, newID);
										});
										buildCss = true;
									}

									// do we have any video
									if ( config.videos.hasOwnProperty(oldID) ) {
										cache.videos[newID] = config.videos[oldID];
									}

									if ( oldEl.find('.ib2-element').length ) {
										oldEl.find('.ib2-element').each(function(){
											var child = $(this),
												childID = child.attr('id'),
												childType = child.attr('data-el'),
												childSetType = config.settings.hasOwnProperty(childType) ? config.settings[childType] : false,
												childSettings = childSetType && childSetType.hasOwnProperty(childID) ? childSetType[childID] : false,
												freshID = prime.generateID(9, 'ib2_el_');

											html = html.replace(childID, freshID);
											if ( childSettings ) {
												childSettings['id'] = freshID;

												// save the main element data
												if ( !cache.contents.hasOwnProperty(childType) ) {
													cache.contents[childType] = {};
												}
												cache.contents[childType][freshID] = childSettings;

												// do we have any css
												if ( config.cssRules.hasOwnProperty(childID) ) {
													cache.cssRules[freshID] = config.cssRules[childID];
													cache.cssRules[freshID].forEach(function(rule, i){
														cache.cssRules[freshID][i]['selector'] = cache.cssRules[freshID][i]['selector'].replace(childID, freshID);
													});
													buildCss = true;
												}

												// do we have any video
												if ( config.videos.hasOwnProperty(childID) ) {
													cache.videos[freshID] = config.videos[childID];
												}
											}
										});
									}

									// insert html
									if ( $.inArray(type, sections) > -1 ) {
										parentEl = ( parentEl.parent().attr('id') == 'screen-container' ? parentEl : parentEl.closest('.ib2-section-el') );
										// make sure it's not columns
										if ( parentEl.attr('data-el') == 'columns' ) {
											parentEl = parentEl.parents('.ib2-section-el');
										}
										if ( pos == 'before' ) {
											parentEl.before(html);
										} else {
											parentEl.after(html);
										}
									} else {
										parentEl = ( parentEl.hasClass('ib2-section-content') ? parentEl : parentEl.find('.ib2-section-content').eq(0) );
										if ( pos == 'before' ) {
											parentEl.prepend(html);
										} else {
											parentEl.append(html);
										}
									}

									if ( buildCss ) {
										gui.buildCss(cache.cssRules, true);
									}
									// clean up
									temp.html('');
								}
							}
						}
						cache.isAdding = false;
						cache.isDragging = false;
						action.closePopup('element-manager', true);

						gui.resizeBackgrounds();
						gui.resizeSelector();
					}
				});

				e.preventDefault();
			});
		},

		history: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				iframeDom = prime.frameBody;

			$('body').on('initHistory', function(e){
				prime.history = new SimpleUndo({
					maxLength: 10,
					provider: function(done) {
						if ( cache.popupMode ) {
							return;
						}

						var content = {
							html: $.trim(iframeDom.find('#screen-container').html()),
							contents: JSON.stringify(cache.contents),
							css: JSON.stringify(cache.cssRules),
						};
						done(content);
					},
					onUpdate: function() {
						if ( !prime.history ) return; 
						self.updateHistoryButtons();
					}
				});

				var cacheData = cache.contents,
					cssRules = cache.cssRules,
					initialContent = {
						html: $.trim(iframeDom.find('#screen-container').html()),
						elements: cacheData,
						css: cssRules,
					};
				prime.history.initialize(initialContent);
			});

			$('body').on('click', '#undo-btn', function(e){
				if ( $(this).hasClass('disabled') ) {
					return;
				}
				prime.history.undo(updateEditorByHistory);
				e.preventDefault();
			});

			$('body').on('click', '#redo-btn', function(e){
				if ( $(this).hasClass('disabled') ) {
					return;
				}
				prime.history.redo(updateEditorByHistory);
				e.preventDefault();
			});

			$('body').on('restoreHistory', function(e, data){
				var prime = window.primeObj,
					cache = prime.cache,
					gui = prime.gui,
					iframeDom = prime.frameBody;

				if ( !data || typeof data !== "object" || cache.popupMode ) {
					return;
				}

				// clean up
				$('.el-highlight').hide();
				$('.el-selected > .el-resize-handlers').html('');
				$('.el-selected').hide();

				// close element settings
				// help close picker
				if ( $('.prime-colors').is(":visible") ) {
					prime.picker.pickerClose();
				}

				$('#editor-settings').find('.element-settings').remove();

				iframeDom.find('#screen-container').html(data.html);
				iframeDom.find('.sortable-line').remove();
				iframeDom.find('.sortable-big-line').remove();

				var settingsData = prime.parseJson(data.contents),
					cssData = prime.parseJson(data.css);

				if ( settingsData ) {
					cache.contents = settingsData;
				}

				if ( cssData ) {
					cache.cssRules = data.css;
				}

				gui.buildCss(cssData);

				cache.activeElement = null;
				cache.activeElementType = null;
				cache.hoveredEl = null;

				// remove is-selected class
				iframeDom.find('.is-selected').removeClass('is-selected');

				setTimeout(function(){
					prime.disableSortable();
					prime.sortable();

					// re-calculate layers
					prime.saveLayers();
					// resize backgrounds
					gui.resizeBackgrounds();
				}, 50);
				
			});
		},

		updateHistoryButtons: function() {
			var prime = window.primeObj;
			$('#undo-btn').removeClass('disabled');
			if ( !prime.history.canUndo() ) {
				$('#undo-btn').addClass('disabled');
				
			}

			$('#redo-btn').removeClass('disabled');
			if ( !prime.history.canRedo() ) {
				$('#redo-btn').addClass('disabled');
			}
			prime.dirty = true;
		},

		tabs: function() {
			var prime = this.prime,
				iframeDom = prime.frameBody,
				gui = prime.gui;

			iframeDom.on('click', '.ib2-nav-tabs > li > a', function(e){
				var $this = $(this),
					activeId = $this.attr('href'),
					oldLink = $this.closest('.ib2-nav-tabs').find('.ib2-tabs-active'),
					oldContent = iframeDom.find(activeId).parent().find('.ib2-tabs-active');

				// switch link
				oldLink.removeClass('ib2-tabs-active');
				$this.parent().addClass('ib2-tabs-active');

				// switch content
				oldContent.removeClass('ib2-tabs-active');
				iframeDom.find(activeId).addClass('ib2-tabs-active');

				// resize selector
				gui.resizeSelector();

				// resize all backgrounds
				gui.resizeBackgrounds();

				e.preventDefault();
				e.stopPropagation();
			});
		},

		savePage: function() {
			var self = this;

			$('body').on('click', '.save-page-settings', function(e){
				self.savePageData(self);
				e.preventDefault();
			});

			$('body').on('click', '.publish-page', function(e){
				self.savePageData(self, true);
				e.preventDefault();
			});
		},

		removeDataAttributes: function(target) {
		    var i,
		        $target = $(target),
		        attrName,
		        dataAttrsToDelete = [],
		        dataAttrs = $target.get(0).attributes,
		        dataAttrsLen = dataAttrs.length,
		        skips = [
		        			'data-el', 'data-anchor', 'data-comment', 'data-popup', 
		        			'data-format', 'data-tz', 'data-ride', 'data-slide-to', 
		        			'data-target', 'data-slide', 'data-current-slide',
		        			'data-question'
		        		];

		    for ( i = 0; i < dataAttrsLen; i++ ) {
		    	if ( $.inArray(dataAttrs[i].name, skips) > -1 ) {
		    		continue;
		    	}

		        if ( 'data-' === dataAttrs[i].name.substring(0,5) ) {
		            dataAttrsToDelete.push(dataAttrs[i].name);
		        }
		    }
		    
		    // delete each of the attributes we found above
		    $.each(dataAttrsToDelete, function(index, attrName) {
		        $target.removeAttr(attrName);
		    });
		},

		prepareContent: function(self) {
			var prime = self.prime,
				cache = prime.cache,
				elements = cache.contents;

			$('#content-placeholder').find('.ui-sortable-handle').removeClass('ui-sortable-handle');

			// reset tabs elements
			$('#content-placeholder').find('.ib2-tabs-el').each(function(){
				var $this = $(this);
				$this.find('.ib2-tabs-active').removeClass('ib2-tabs-active');
				$this.find('.ib2-nav-tabs > li').eq(0).addClass('ib2-tabs-active');
				$this.find('.ib2-tab-content > .ib2-tab-pane').eq(0).addClass('ib2-tabs-active');
			});
			// reset questions elements
			$('#content-placeholder').find('.ib2-questions-el').each(function(){
				var $this = $(this);
				$this.find('.ib2-quiz-page').hide();
				$this.find('.ib2-quiz-page').eq(0).show();
			});
			// reset optin3 elements
			$('#content-placeholder').find('.ib2-optslide-el').each(function(){
				var $this = $(this);
				$this.find('.ib2-section-slide').css('height', '0px').hide();
				$this.find('.ib2-section-slide').eq(0).css('height', 'auto').show();
				$this.attr('data-current-slide', '1');
			});
			// reset collapse element
			$('#content-placeholder').find('.ib2-collapse-panel').each(function(){
				var $this = $(this);
				$this.css('max-height', '0px');
			});
			$('#content-placeholder').find('.ib2-collapse-head').removeClass('active');

			// reset countdown reveal position
			$('#content-placeholder').find('.ib2-countdown-el').each(function(){
				var $this = $(this);
				$this.find('> div, > span').each(function(){
					if ( !$(this).hasClass('ib2-countdown-reveal') && !$(this).is(":visible") ) {
						$(this).show();
					}
				});
				$this.find('.ib2-countdown-reveal').hide();
			});

			// remove unnessary data attribute
			var tags = document.querySelector('#content-placeholder').getElementsByTagName("*");
			for ( var tag in tags ) {
				if ( $.isNumeric(tag) ) {
					this.removeDataAttributes(tags[tag]);
				}
			}
			// process delay and animation classes
			$('#content-placeholder').find('.ib2-element').each(function(){
				var el = $(this),
					id = el.attr('id'),
					type = el.attr('data-el');

				var settings = elements[type][id]['settings'],
					effect = ( settings && settings.hasOwnProperty('effect') ? settings.effect : null );

				if ( el.attr('spellcheck') ) {
					el.removeAttr('spellcheck');
				}

				if ( effect ) {
					el.removeClass('ib2-box-animation ib2-box-delay');
					if ( effect.showDelay == 'yes' ) {
						var hour = (parseInt(effect.delayhour) * 3600),
							min = (parseInt(effect.delaymin) * 60),
							sec = parseInt(effect.delaysec),
							delay = (hour + min + sec);

						el.addClass('ib2-box-delay')
							.attr('data-delay', delay);
					}

					if ( effect.animation != null && effect.animation != 'none' && effect.animation != '' ) {
						el.addClass('ib2-box-animation')
							.attr('data-animation', effect.animation);
					}
				}
			});

			$('#content-placeholder').find('.ib2-shortcode-el').each(function(){
				var s = $(this), content = s.find('.current-shortcode').text();
				content = content.replace(/\[/gi, '{%sc_open%}').replace(/\]/gi, '{%sc_close%}');
				s.html(content);
			});
			$('#content-placeholder').find('.ib2-comment-el').each(function(){
				$(this).find('> .el-content').html('{comment}');
			});
			// additional cleanup
			$('#content-placeholder').find('.ib2-popup-bg').remove();
			$('#content-placeholder').find('.is-selected').removeClass('is-selected');
		},

		savePageData(self, publish) {
			var prime = self.prime,
				cache = prime.cache,
				layers = cache.layers,
				action = prime.action,
				iframeDom = prime.frameBody,
				elements = cache.contents,
				page_title = elements['page']['screen-container']['settings']['general']['title'],
				deferreds = [],
				results = [],
				raw = iframeDom.find('#screen-container').html();

			publish = publish || false;
			if ( cache.isSaving ) {
				return;
			}

			// reset active element
			cache.activeElement = null;
			cache.activeElementType = null;	
			cache.hoveredEl = null;
			var isAutosave = cache.isAutosave;

			// clean up
			$('.el-highlight').hide();
			$('.el-selected > .el-resize-handlers').html('');
			$('.el-selected').hide();
			iframeDom.find('.is-selected').removeClass('is-selected');

			// help close picker
			if ( $('.prime-colors').is(":visible") ) {
				prime.picker.pickerClose();
			}

			$('#editor-settings').find('.element-settings').remove();

			cache.isSaving = true;
			$('.save-page-settings').addClass('disabled');
			$('.publish-page').addClass('disabled');
			$('.save-text').removeClass('hidden');
			$('#content-placeholder').html(raw);

			// prepare data
			self.prepareContent(self);

			// fix elements from the evil array
			for ( var elem_type in elements ) {
				if ( Array.isArray(elements[elem_type]) ) {
					elements[elem_type] = {...elements[elem_type]};
				}
			}
			var newHtml = $('#content-placeholder').html(),
				layersObj = JSON.stringify(layers),
				elementsObj = JSON.stringify(elements),
				htmlFile = new Blob([newHtml], {type: "text/plain;charset=utf-8"}, 'html.txt'),
				layersFile = new Blob([layersObj], {type: "text/plain;charset=utf-8"}, 'layers.txt'),
				elementsFile = new Blob([elementsObj], {type: "text/plain;charset=utf-8"}, 'elements.txt'),
				saveMode = cache.isSaveTemplate ? 'template' : 'normal',
				templateData = null,
				ajaxAction = isAutosave ? 'ib2_auto_save' : 'ib2_editor_save',
				fd = new FormData();

			fd.append('action', ajaxAction);
			fd.append('mode', 'html');
			fd.append('html', htmlFile);
			fd.append('layers', layersFile);
			fd.append('elements', elementsFile);
			fd.append('post_id', $('#ib2-post-id').val());
			fd.append('variant_id', $('#ib2-variant-id').val());
			fd.append('savemode', saveMode);
			fd.append('restored', prime.restored);

			if ( saveMode == 'template' ) {
				templateData = {
					name: $('#template-name').val(),
					type: $('#template-type').val(),
					subtype: $('#template-subtype').val(),
					tags: $('#template-tags').val(),
					thumb: ($('#template-thumb-url').val() !== '' ? Base64.encode($('#template-thumb-url').val()) : ''),
				};

				for ( var keyname in templateData ) {
					fd.append('template_' + keyname, templateData[keyname]);
				}
			}

			var sendhtml = $.ajax({
				type: "POST",
				url: ajaxurl,
				data: fd,
				processData: false,
    			contentType: false,
				success: function(response) {
					results.push(response);
				}
			});
			deferreds.push(sendhtml);

			// weight
			var weights = {};
			$('.variant-weights').each(function(){
				weights[$(this).attr('data-variant')] = $(this).val();
			});

			// rest of the data
			var rests = $.ajax({
				type: "POST",
				url: ajaxurl,
				data: {
					action: ajaxAction,
					mode: 'settings',
					savemode: saveMode,
					vals: {
						'videos': cache.videos,
						'cssrules': cache.cssRules,
						'settings': cache.contents['page']['screen-container'],
					},
					template: templateData,
					post_id: $('#ib2-post-id').val(),
					variant_id: $('#ib2-variant-id').val(),
					title: page_title,
					weights: weights,
					restored: prime.restored,
					status: ( publish ? 'publish' : prime.postStatus )
				},
				success: function(response) {
					results.push(response);
				}
			});
			deferreds.push(rests);

			$.when.apply($, deferreds).then(function() {
				cache.isSaving = false;
				cache.isAutosave = false;
				if ( cache.variantType ) {
					var post_id = $('#ib2-post-id').val(),
						mode = cache.variantType,
						url = $('#ib2-admin-url').val();
					
					url += '?action=ib2_new_variant&post_id=' + post_id + '&mode=' + mode;
					if ( mode == 'template' ) url += '&tid=' + cache.templateId;
					if ( mode == 'duplicate' ) url += '&oldvar=' + $('#ib2-current-variation').val();
					window.location.href = url;
				}

				$('.save-text').addClass('hidden');
				$('.save-page-settings').removeClass('disabled');
				$('.publish-page').removeClass('disabled');
        		
        		if ( publish === true ) {
        			prime.postStatus = 'publish';
        			$('.publish-page').hide();
        		}

        		if ( cache.isSaveTemplate ) {
        			cache.isSaveTemplate = false;
        			$('#template-save').button('reset');
        			action.closePopup(cache.currentPopSetting, true);
        		} else {
        			prime.dirty = false;
        			prime.clearLocalData();
        		}
    		});
		},

		editFormEvents: function() {
			var self = this,
				id = 'form-manager',
				prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				gui = prime.gui,
				action = prime.action;

			// edit mode
			$('body').on('click', '.field-edit', function(e){
				var $this = $(this),
					elID = cache.activeElement,
					type = cache.activeElementType,
					data = cache.contents[type][elID],
					inputs = data.inputs,
					idx = parseInt($this.attr('data-idx')),
					field = inputs[idx];

				action.showEditMode('form-manager', field, idx);

				e.preventDefault();
			});

			// select event
			$('body').on('change', '#form-field-type', function(){
				var opt = $("option:selected", this).val();
				if ( opt == 'select' || opt == 'radio' ) {
					$('#' + id).find('.field-options-prop').removeClass('hidden');
					$('#' + id).find('.non-hidden-fields').removeClass('hidden');
					$('#' + id).find('.checked-by-default').addClass('hidden');
				} else if ( opt == 'hidden' ) {
					$('#' + id).find('.field-options-prop').addClass('hidden');
					$('#' + id).find('.non-hidden-fields').addClass('hidden');
					$('#' + id).find('.checked-by-default').addClass('hidden');
				} else if ( opt == 'checkbox' ) {
					$('#' + id).find('.field-options-prop').addClass('hidden');
					$('#' + id).find('.non-hidden-fields').removeClass('hidden');
					$('#' + id).find('.checked-by-default').removeClass('hidden');
				} else {
					$('#' + id).find('.field-options-prop').addClass('hidden');
					$('#' + id).find('.non-hidden-fields').removeClass('hidden');
					$('#' + id).find('.checked-by-default').addClass('hidden');
				}
			});

			// cancel event handler
			$('body').on('click', '.field-cancel-btn', function(e){
				$('#' + id).find('.settings-form-edit').addClass('hidden');
				$('#' + id).find('.form-manager-table').removeClass('hidden');

				$('#' + id)
					.find('.setting-popup-buttons')
						.html('<button type="button" class="btn btn-success add-new-field"><i class="fa fa-plus"></i> Add New Field</button>&nbsp;<button type="button" class="btn btn-default close-setting-popup">Close</button>');

				// switch to wide popup
				$('#' + id).addClass('settings-popup-wide');

				e.preventDefault();
			});

			// update event handler
			$('body').on('click', '.field-update-btn', function(e){
				var elID = cache.activeElement,
					type = cache.activeElementType,
					setting = cache.contents[type][elID]['settings'],
					inputs = cache.contents[type][elID]['inputs'],
					index = parseInt($('#' + id).find('.field-index').val()),
					newname = $.trim($('#' + id).find('.field-key').val()),
					newlabel = $.trim($('#' + id).find('.field-label').val()),
					newicon = $.trim($('#' + id).find('.field-icon').val()),
					newtype = $('#' + id).find('.field-type').val(),
					newtag = ( newtype == 'select' ? 'select' : (newtype == 'textarea' ? 'textarea' : 'input') ),
					options = $.trim($('#' + id).find('.field-options').val()),
					defaultValue = $.trim($('#' + id).find('.field-value').val()),
					errorMsg = '';

				// validation
				if ( newname == '' ) {
					errorMsg = '<p class="text-danger">ERROR: Please enter a key name for the field.</p>'
				}

				if ( newlabel == '' ) {
					errorMsg = '<p class="text-danger">ERROR: Please enter a text label for the field.</p>'
				}

				if ( newtype == ('select' || 'radio') && options == '' ) {
					errorMsg = '<p class="text-danger">ERROR: Please enter at least one option.</p>'
				}

				if ( newtype == 'radio' && defaultValue == '' ) {
					errorMsg = '<p class="text-danger">ERROR: Default value must be set for radio options field.</p>'
				}

				if ( errorMsg != '' ) {
					$('#form-manager').find('.error-msg').html(errorMsg);
					errorMsg = '';
					return false;
				}

				var newinput = {
					name: newname,
					tag: newtag,
					type: newtype,
					value: defaultValue,
					placeholder: newlabel,
					label: newlabel,
					icon: newicon,
					isEmail: newtype == 'email' ? true : false,
					visible: ( $('#' + id).find('.field-hide').is(":checked") ? false : true ),
					required: ( $('#' + id).find('.field-required').is(":checked") ? true : false ),
					checked: ( $('#' + id).find('.field-default').is(":checked") ? true : false ),
					text: '',
					options: {},
				};

				if ( options !== null && options != '' ) {
					var parts = options.split(new RegExp('\\n', 'gi'));
					parts.forEach(function(part){
						var smallparts = part.split(":");
						if ( smallparts.length > 1 ) {
							newinput.options[smallparts[0]] = smallparts[1];
						} else {
							newinput.options[part] = part;
						}
					});
				}

				cache.contents[type][elID]['inputs'][index] = newinput;
				
				// render display
				var newRows = action.formTable(cache.contents[type][elID]['inputs']);

				$('#' + id).find('.form-table-rows').html(newRows);
				$('#' + id).find('.settings-form-edit').addClass('hidden');
				$('#' + id).find('.form-manager-table').removeClass('hidden');

				$('#' + id)
					.find('.setting-popup-buttons')
						.html('<button type="button" class="btn btn-success add-new-field"><i class="fa fa-plus"></i> Add New Field</button>&nbsp;<button type="button" class="btn btn-default close-setting-popup">Close</button>');

				// switch to wide popup
				$('#' + id).addClass('settings-popup-wide');

				// rebuild the form
				gui.buildForm(iframeDom.find('#' + elID).find('.form-fields'), cache.contents[type][elID]['inputs'], setting);

				e.preventDefault();
			});

			// add new field
			$('body').on('click', '.add-new-field', function(e){
				var elID = cache.activeElement,
					type = cache.activeElementType,
					setting = cache.contents[type][elID]['settings'],
					newinput = {
						name: 'custom_field',
						tag: 'input',
						type: 'text',
						value: '',
						placeholder: 'Custom Field',
						label: 'Custom Field',
						icon: 'none',
						isEmail: false,
						visible: true,
						required: false,
						checked: false,
						text: '',
						options: {},
				};

				cache.contents[type][elID]['inputs'].push(newinput);

				// render display
				var newRows = action.formTable(cache.contents[type][elID]['inputs']);
				$('#' + id).find('.form-table-rows').html(newRows);

				// rebuild the form
				gui.buildForm(iframeDom.find('#' + elID).find('.form-fields'), cache.contents[type][elID]['inputs'], setting);

				e.preventDefault();
			});

			// delete fields
			$('body').on('click', '.field-remove', function(e){
				var $this = $(this),
					idx = parseInt($this.attr('data-idx')),
					elID = cache.activeElement,
					type = cache.activeElementType,
					setting = cache.contents[type][elID]['settings'],
					oldInputs = cache.contents[type][elID]['inputs'],
					newinputs = [];

				if ( oldInputs.length <= 1 ) {
					alert('Error: Your form must have at least one field.');
					return false;
				}

				if ( confirm('Are you sure you want to remove this field?') ) {
					oldInputs.forEach(function(input, f){
						if ( f != idx ) {
							newinputs.push(input);
						}
					});
					// update input data
					cache.contents[type][elID]['inputs'] = newinputs;

					// render display
					var newRows = action.formTable(cache.contents[type][elID]['inputs']);
					$('#' + id).find('.form-table-rows').html(newRows);

					// rebuild the form
					gui.buildForm(iframeDom.find('#' + elID).find('.form-fields'), cache.contents[type][elID]['inputs'], setting);
				}
				e.preventDefault();
			});
		},

		parseHTMLForm: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				gui = prime.gui;

			$('body').on("click", '.parse-form-btn', function(e){
				var $this = $(this),
					elID = cache.activeElement,
					type = cache.activeElementType,
					fieldId = $this.attr('data-id'),
					code = $('#' + fieldId).val(),
					setting = cache.contents[type][elID]['settings'],
					form = iframeDom.find('#' + elID).find("form"),
					inputs, 
					htmlform;

				if ( typeof code === "string" && code != '' ) {
					htmlform = $('<div id="html-form">' + code + '</div>');

					// create form view helper
					htmlform.find('script').remove();
					htmlform.find('style').remove();
					htmlform.appendTo('body');

					// create action
					if ( !form.find('.form-fields').find('input[name="action_url"]').length ) {
						form.find('.form-fields').append('<input type="hidden" name="action_url" />');
					}
					if ( !form.find('.form-fields').find('input[name="_method"]').length ) {
						form.find('.form-fields').append('<input type="hidden" name="_method" />');
					}
					form.find('.form-fields').find('input[name="action_url"]').val(htmlform.find('form').attr('action'));
					form.find('.form-fields').find('input[name="_method"]').val(htmlform.find('form').attr('method'));

					// get form data
					inputs = prime.readInputs(htmlform);

					// build the form
					gui.buildForm(form.find('.form-fields'), inputs, setting);
					// remove the form view helper
					htmlform.remove();
					
					// save form data
					prime.cache.contents[type][elID]['inputs'] = inputs;

					// resize selector
					gui.resizeSelector();

					// resize all backgrounds
					gui.resizeBackgrounds();
				}

				e.preventDefault();
			});
		},

		columnResize: function() {
			var self = this,
				prime = this.prime,
				iframeDom = prime.frameBody,
				iframeDoc = prime.frameDoc,
				gui = prime.gui,
				cache = prime.cache;

			$('body').on("mousedown", ".col-resizer", function(e){
				var $this = $(this),
					elID = cache.activeElement,
					firstID = $this.parent().attr('data-col1'),
					secID = $this.parent().attr('data-col2'),
					pos = parseInt($this.parent().attr('data-pos')),
					isOdd = pos % 2 != 0 ? true : false;

				$('.el-highlight').hide();
				e.preventDefault();

				var currentGhost = $this.parent(),
					ghostId = currentGhost.attr('id'),
					//origLeft = parseInt(currentGhost.css('left').replace(/px/gi, '')),
					prevGhost = ( currentGhost.prev().length ? currentGhost.prev() : null ),
					prevLeft = (prevGhost && prevGhost.hasClass('el-col-resizer') && prevGhost.attr('id') != ghostId) ? parseInt(prevGhost.css('left').replace(/px/gi, '')) : 0,
					nextGhost = ( currentGhost.next().length ? currentGhost.next() : null ),
					nextLeft = (nextGhost && nextGhost.hasClass('el-col-resizer') && nextGhost.attr('id') != ghostId) ? parseInt(nextGhost.css('left').replace(/px/gi, '')) : $('.el-selected').get(0).clientWidth;

				resizeCol.col1 = firstID;
				resizeCol.col2 = secID;
				resizeCol.minLeft = prevLeft + 100;
				resizeCol.offsetX = e.offsetX;
				resizeCol.prevLeft = prevLeft;
				resizeCol.nextLeft = nextLeft;
				resizeCol.container = $('.el-selected').get(0);
				resizeCol.ghost = $this.parent().get(0);
				resizeCol.maxLeft = nextLeft - 100;
				resizeCol.totalWidth = iframeDom.find('#' + firstID).get(0).clientWidth + iframeDom.find('#' + secID).get(0).clientWidth;

				$('body').trigger('resizedrag');

				// overlay the iframe
				$('#main-editor').append('<div class="iframe-overlay"></div>');

				if ( isOdd ) {
					iframeDom.find('#' + secID).addClass('ib2-col-resized');
				} else {
					iframeDom.find('#' + firstID).addClass('ib2-col-resized');
				}

				document.onmouseup = self.resizeStop;
				document.onmousemove = self.colResizeDrag;
			});

			$('body').on('resizedrag', function(){
				cache.isDragging = true;
			});

			$('body').on('resizestop', function(){
				cache.isDragging = false;
				prime.saveHistory();
			});

			$('body').on('resizeselector', function(){
				gui.resizeSelector();
				gui.resizeBackgrounds();
			});
		},

		colResizeDrag: function(e) {
			var prime = window.primeObj,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				iframeDoc = prime.frameDoc,
				target = iframeDom.find('#' + resizeCol.col1);

			if ( !cache.isDragging ) {
				return;
			}

			var rect = resizeCol.container.getBoundingClientRect(),
				curX = e.pageX ? e.pageX : e.clientX,
				newLeft = curX - rect.left - resizeCol.offsetX,
				newWidth = 0,
				nextWidth = 0;

			if ( newLeft <= resizeCol.minLeft ) {
				newLeft = resizeCol.minLeft;
			}

			if ( newLeft > resizeCol.maxLeft ) {
				newLeft = resizeCol.maxLeft;
			}

			$(resizeCol.ghost).css('left', newLeft + 'px');

			newWidth = Math.floor(newLeft - resizeCol.prevLeft);
			nextWidth = Math.floor(resizeCol.totalWidth - newWidth);

			target.css('width', newWidth + 'px');
			target.next().css('width', nextWidth + 'px');

			if ( e.target ) {
				var infoPar = $(resizeCol.ghost);
				infoPar.find('.col-size-info').removeClass('hidden');
				infoPar.find('.col1-size-info').text(newWidth + 'px');
				infoPar.find('.col2-size-info').text(nextWidth + 'px');
			}
			$('body').trigger('resizeselector');
		},

		elementResize: function() {
			var self = this,
				prime = this.prime,
				gui = prime.gui,
				cache = prime.cache,
				iframeDom = prime.frameBody;

			$('body').on("mousedown", ".el-resizer", function(e){
				var $this = $(this),
					id = cache.activeElement,
					type = cache.activeElementType,
					dir = $this.attr('data-resize'),
					inc = 'both',
					element = prime.getElementData(type, prime.getElementAlias($('#' + id))),
					target = iframeDom.find('#' + id).hasClass('is-element') ? iframeDom.find('#' + id) : iframeDom.find('#' + id).find('.is-element').eq(0);

				if ( dir == 'w' || dir == 'e' ) {
					inc = 'width';
				} else if ( dir == 'n' || dir == 's' ) {
					inc = 'height';
				}

				$('body').trigger('resizedrag');

				e.preventDefault();

				resizeEl.id = id;
				resizeEl.type = type;
				resizeEl.dir = dir;
				resizeEl.inc = inc;
				resizeEl.oldX = e.clientX;
				resizeEl.oldY = e.clientY;
				resizeEl.origWidth = parseInt(target.css('width'));
				resizeEl.origHeight = target.height();
				resizeEl.bounceX = iframeDom.find('#' + id).parent().width();
				resizeEl.element = element;
				resizeEl.selector = target;

				// overlay the iframe
				$('#main-editor').append('<div class="iframe-overlay"></div>');

				document.onmouseup = self.resizeStop;
				document.onmousemove = self.resizeDrag;
			});

			$('body').on('resizedrag', function(){
				cache.isDragging = true;
			});

			$('body').on('resizestop', function(){
				$('#main-editor').find('.iframe-overlay').remove();
				cache.isDragging = false;
				prime.saveHistory();
			});

			$('body').on('resizeselector', function(){
				gui.resizeSelector();
				gui.resizeBackgrounds();
			});
		},

		resizeDrag: function(e) {
			var prime = window.primeObj,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				iframeDoc = prime.frameDoc,
				baseEl = iframeDom.find('#' + resizeEl.id),
				selector = resizeEl.selector,
				target = selector.find(' > div');

			if ( !cache.isDragging ) {
				return;
			}

			resizeEl.rect = document.querySelector('.el-selected').getBoundingClientRect();

			if ( resizeEl.dir == 's' || resizeEl.dir == 'e' || resizeEl.dir == 'se' ) {
				resizeEl.curX = e.clientX - resizeEl.rect.left;
	    		resizeEl.curY = e.clientY - resizeEl.rect.top;
	    	} else {
	    		resizeEl.curX = resizeEl.rect.width + (resizeEl.rect.left - e.clientX);
	    		resizeEl.curY = resizeEl.rect.height + (resizeEl.rect.top - e.clientY);
	    	}

    		resizeEl.curX = Math.ceil(Math.max(resizeEl.curX, 30));
    		resizeEl.curY = Math.ceil(Math.max(resizeEl.curY, 30));

    		if ( resizeEl.curX > resizeEl.bounceX ) {
    			resizeEl.curX = resizeEl.bounceX;
    		}

    		if ( resizeEl.type == 'image' ) {
    			target = baseEl.find('img');
    		}

    		var isContainer = false, cssKey = 'width';
    		if ( resizeEl.type == 'section' ) {
    			isContainer = true;
    		}

    		// RESIZE WIDTH
    		if ( resizeEl.inc == 'width' || resizeEl.inc == 'both' ) {
    			if ( isContainer ) {
    				// RESIZE DIRECT SECTION WIDTH
    				var maxPopWidth = $(window.FrameDocument).width() - 200;
    				if ( resizeEl.curX >= maxPopWidth ) {
    					resizeEl.curX = maxPopWidth;
    				}

    				if ( selector.find(' > .ib2-section-content').length == 1 ) {
    					target.css('width', '100%');
    				}
    				$('.el-selected').css('width', resizeEl.curX + 'px');
    				baseEl.css('width', resizeEl.curX + 'px');
    				cache.contents[resizeEl.type][resizeEl.id]['width'] = resizeEl.curX;
    			} else {
    				// RESIZE ELEMENT WIDTH
    				// resize element selector width
    				$('.el-selected').css('width', resizeEl.curX + 'px');

    				// resize element's width
    				target.css('width', resizeEl.curX + 'px');

    				// additional resize for specific elements
    				switch (resizeEl.type) {
    					case 'image':
    						var marginTop = parseInt(target.css('marginTop').replace(/px/gi, '')),
    							currentHeight = target.height(),
    							gapHeight = ( marginTop < 0 ) ? (currentHeight - Math.abs(marginTop)) : 0,
    							minGap = (currentHeight - $('.el-selected').height()),
    							prevWidth = resizeEl.curX - 1;

    						if ( marginTop < 0 && resizeEl.origWidth > target.width() ) {
    							var newMarginTop = Math.round(Math.max((target.height() - currentHeight) / 2), 50);
    							if ( newMarginTop <= 0 ) {
    								target.css('marginTop', -newMarginTop + 'px');
    							}
    						}

    						if ( selector.height() > currentHeight ) {
    							if ( marginTop < 0 ) {
    								selector.css('height', gapHeight + 'px');
    							} else {
    								selector.css('height', 'auto');
    							}
    						}

    						selector.css('width', resizeEl.curX + 'px');
    						cache.contents[resizeEl.type][resizeEl.id]['width'] = resizeEl.curX;
    						break;

    					default:
    						cssKey = ( resizeEl.element.hasOwnProperty('useMinWidth') && resizeEl.element.useMinWidth === true ) ? 'minWidth' : 'width';
							resizeEl.element.targetWidth.forEach(function(t){
								var elem = prime.getSettingTarget(resizeEl.id, t);
								elem.css(cssKey, resizeEl.curX + 'px');
								if ( resizeEl.element.hasOwnProperty('resizeParent') && resizeEl.element.resizeParent === true ) {
									elem.parent().css(cssKey, resizeEl.curX + 'px');
								}
							});

							cache.contents[resizeEl.type][resizeEl.id]['width'] = resizeEl.curX;
							if ( cache.contents[resizeEl.type][resizeEl.id]['settings'].hasOwnProperty('boxsize') ) {
								cache.contents[resizeEl.type][resizeEl.id]['settings']['boxsize']['width'] = resizeEl.curX;
								$('#slider-box-width-' + resizeEl.type).val(resizeEl.curX);
								$('#slider-box-width-' + resizeEl.type + '-num').val(resizeEl.curX);
							}
    				}
    			}
    		}

    		// RESIZE HEIGHT
    		if ( resizeEl.inc == 'height' || resizeEl.inc == 'both' ) {
    			if ( cache.popupMode && resizeEl.type == 'section' ) {
    				var maxPopHeight = $(window.FrameDocument).height() - 200;
    				if ( resizeEl.curY >= maxPopHeight ) {
    					resizeEl.curY = maxPopHeight;
    				}
    			}

    			// resize element selector height
    			$('.el-selected').css('height', resizeEl.curY + 'px');

    			// resize element's height
    			switch (resizeEl.type) {
    				case 'image':
    					// onlys resize on 'height' mode
    					if ( resizeEl.inc == 'height' ) {
	    					if ( target.height() > resizeEl.curY ) {
		    					var imgMarginTop = Math.round(Math.max((target.height() - resizeEl.curY) / 2), 50);
		    					target.css('marginTop', -imgMarginTop + 'px');
		    					selector.css('height', resizeEl.curY + 'px');
		    				} else {
		    					var newWidth = Math.ceil(Math.max((resizeEl.origWidth/resizeEl.origHeight*resizeEl.curY), 50));
		    					if ( newWidth < resizeEl.bounceX ) {
		    						target.css({
		    							marginTop: 0 + 'px',
		    							width: newWidth + 'px'
		    						});
		    						selector.css({
		    							width: newWidth + 'px',
		    							height: 'auto'
		    						});
		    					}
		    				}
	    				}
	    				cache.contents[resizeEl.type][resizeEl.id]['height'] = resizeEl.curY;
	    				break;

	    			default:
	    				cssKey = ( resizeEl.element.hasOwnProperty('useMinHeight') && resizeEl.element.useMinHeight === true ) ? 'minHeight' : 'height';
						resizeEl.element['targetHeight'].forEach(function(t){
							var elem = prime.getSettingTarget(resizeEl.id, t);
							elem.css(cssKey, resizeEl.curY + 'px');
							if ( resizeEl.type == 'button' && t == 'button' ) {
								elem.css('line-height', resizeEl.curY + 'px');
							}
							if ( resizeEl.element.hasOwnProperty('resizeParent') && resizeEl.element.resizeParent === true ) {
								elem.parent().css(cssKey, resizeEl.curY + 'px');
							}
						});
						cache.contents[resizeEl.type][resizeEl.id]['height'] = resizeEl.curY;
						if ( cache.contents[resizeEl.type][resizeEl.id]['settings'].hasOwnProperty('boxsize') ) {
							cache.contents[resizeEl.type][resizeEl.id]['settings']['boxsize']['minHeight'] = resizeEl.curY;
							$('#slider-box-min-height-' + resizeEl.type).val(resizeEl.curY);
							$('#slider-box-min-height-' + resizeEl.type + '-num').val(resizeEl.curY);
						}

						if ( cache.contents[resizeEl.type][resizeEl.id]['settings'].hasOwnProperty('minheight') ) {
							cache.contents[resizeEl.type][resizeEl.id]['settings']['minheight']['minHeight'] = resizeEl.curY;
							$('#slider-min-height-' + resizeEl.type).val(resizeEl.curY);
							$('#slider-min-height-' + resizeEl.type + '-num').val(resizeEl.curY);
						}
    			}
    		}

    		if ( resizeEl.type == 'section' && cache.popupMode ) {
    			var action = prime.action;
    			action.centerPopupEdit(iframeDom, resizeEl.id, cache.sliderEdit);
    		}

    		$('body').trigger('resizeselector');
		},

		resizeStop: function(e) {
			document.onmouseup = null;
			document.onmousemove = null;

			if ( resizeCol && resizeCol.hasOwnProperty('ghost') && resizeCol.ghost ) {
				var infoPar = $(resizeCol.ghost);
				if ( infoPar.find('.col-size-info').length ) {
					infoPar.find('.col-size-info').addClass('hidden');
				}
			}

			$('body').trigger('resizestop');
		},

		imagePositionBall: function() {
			var self = this;
			$('body').on("mousedown", ".pos-ball", function(e){
				var $this = $(this),
					id = $this.attr('id');

				e.preventDefault();

				ballPos.id = id;
				ballPos.oldX = e.clientX;
				ballPos.oldY = e.clientY;
				ballPos.bounceX = ($('#' + id).parent().find('img').innerWidth()-10);
				ballPos.bounceY = ($('#' + id).parent().find('img').innerHeight()-10);

				document.onmouseup = self.posBallUp;
				document.onmousemove = self.posBallDrag;
			})
		},

		posBallDrag: function(e) {
			ballPos.curX = ballPos.oldX - e.clientX;
			ballPos.curY = ballPos.oldY - e.clientY;
			ballPos.oldX = e.clientX;
			ballPos.oldY = e.clientY;

			ballPos.top = ($('#' + ballPos.id).position().top - ballPos.curY);
			ballPos.left = ($('#' + ballPos.id).position().left - ballPos.curX);

			if ( ballPos.top < -10 ) ballPos.top = -10;
			if ( ballPos.left < -10 ) ballPos.left = -10;
			if ( ballPos.top > ballPos.bounceY ) ballPos.top = ballPos.bounceY;
			if ( ballPos.left > ballPos.bounceX ) ballPos.left = ballPos.bounceX;

			$('#' + ballPos.id).css({
				top: ballPos.top + 'px',
				left: ballPos.left + 'px',
			});

			var newX = Math.round(ballPos.left/ballPos.bounceX*100),
				newY = Math.round(ballPos.top/ballPos.bounceY*100);

			if ( newX < 0 ) newX = 0;
			if ( newX > 100 ) newX = 100;
			if ( newY < 0 ) newY = 0;
			if ( newY > 100 ) newY = 100;

			$('#' + ballPos.id).closest('.form-group').find('.bg-pos-value').val(newX + '% ' + newY + '%').change();
		},

		posBallUp: function(e) {
			document.onmouseup = null;
			document.onmousemove = null;
		},

		removeImage: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				apply = prime.apply,
				defSettings = prime.settings;

			$('body').on("click", '.remove-media', function(e){
				var $this = $(this),
					pid = $this.closest('.form-group').find('.form-control').attr('id'),
					elID = cache.activeElement,
					type = cache.activeElementType,
					settingFor = $this.data('for'),
					idx = $this.data('idx'),
					setting = defSettings[settingFor] || null;

				if ( setting ) {
					var fields = setting.fields,
						field = fields[idx],
						name = ( field.id == 'bg-image' ? 'background' : 'image' ),
						settingData = cache.contents[type][elID]['settings'][name],
						targetElement = prime.getSettingTarget(elID, settingData.pos),
						value = ( field.action == 'showimage' ? $('#ib2-img-src').attr('src') : '' );

					$('#' + pid).val(value);

					if ( type != 'image' ) {
						$this.addClass('hidden');
					}

					// apply change and update data
					var cacheData = {
						id: elID,
						type: type,
						name: name,
						element: targetElement,
						value: value,
						field: field,
						props: setting.props
					};
					apply.change(cacheData);

					// hide preview
					if ( $('#' + pid).parent().parent().find('.image-preview').length ) {
						$('#' + pid).parent().parent().find('.image-preview').addClass('hidden');
					}

					// remove background placeholder
					targetElement.find(" > .is-background").remove();
					
					// resize selector
					gui.resizeSelector();
					// resize all backgrounds
					gui.resizeBackgrounds();
					// save history
					prime.saveHistory();
				}
				e.preventDefault();	
			});
		},

		handleImageUpload: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				apply = prime.apply,
				defSettings = prime.settings;

			$('body').on("mediaselected", function(e, attachment, settingFor, idx, pid){
				var elID = cache.activeElement,
					type = cache.activeElementType,
					setting = defSettings[settingFor] || null;

				if ( setting ) {
					// process upload
					var fields = setting.fields,
						field = fields[idx],
						name = ( field.id == 'bg-image' ? 'background' : 'image' );

					if ( field.id == 'submit-image' ) {
						name = 'formbutton';
					} else if ( field.id == 'carousel-fields' ) {
						name = 'slides';
					}

					var settingData = cache.contents[type][elID]['settings'][name];
					$('#' + pid).val(attachment.url);

					//if ( $('#' + pid).parent().find('.remove-media').length || elID ) {
					if ( $('#' + pid).parent().find('.remove-media').length ) {
						$('#' + pid).parent().find('.remove-media').removeClass('hidden');

						// apply change and update data
						var cacheData = {
							id: elID,
							type: type,
							name: name,
							element: prime.getSettingTarget(elID, settingData.pos),
							value: attachment.url,
							field: field,
							props: setting.props,
							attachment: attachment
						};

						if ( $('#' + pid).attr('data-num') ) {
							cacheData.num = parseInt($('#' + pid).attr('data-num'));
						}

						if ( $('#' + pid).attr('data-key') ) {
							cacheData.key = $('#' + pid).attr('data-key');
						}
						apply.change(cacheData);

						// set preview and position
						if ( $('#' + pid).parent().parent().find('.image-preview').length ) {
							var preview_url = attachment.url.replace(/^(https?|ftp):/, '');
							$('#' + pid).parent().parent().find('.image-preview').find('img').attr('src', preview_url);
							$('#' + pid).parent().parent().find('.image-preview').removeClass('hidden');
						}
					} else {
						$('body').trigger('template-thumb');
					}

					// resize selector
					gui.resizeSelector();
					// resize all backgrounds
					gui.resizeBackgrounds();
					// save history
					prime.saveHistory();
				}
				
			});
		},

		wpMediaHandler: function() {
			var prime = this.prime,
				gui = prime.gui;

			$("body").on("click", '.wp-media-btn', function(e){
				var $this = $(this),
					mediaType = $this.attr('data-media'),
					multiple = $this.attr('data-multiple'),
					settingFor = $this.attr('data-for'),
					idx = $this.attr('data-idx'),
					pid = $this.attr('data-id');

				gui.displayMedia(mediaType, multiple, settingFor, idx, pid);
				e.preventDefault();
			});
		},

		toggleMode: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				apply = prime.apply,
				defSettings = prime.settings;

			$('body').on('click', '.mode-btn', function(e){
				var $this = $(this),
					elID = cache.activeElement,
					mode = $this.attr('data-mode'),
					parent = $this.closest('.settings-header'),
					parentID = parent.attr('id'),
					settingFor = parent.attr('data-for'),
					setting = defSettings[settingFor] || null;

				if ( $this.hasClass('active') ) {
					return false;
				} 

				$this.parent().find('.active').addClass('btn-default').removeClass('active btn-success');
				$this.addClass('btn-success active').removeClass('btn-default');

				$('#fields-' + parentID).find('.form-group').addClass('hidden');
				$('#fields-' + parentID).find('.mode-' + mode).removeClass('hidden');

				// apply setting
				apply.switchMode(mode, settingFor, setting);

				// resize selector
				//gui.resizeSelector();
				
				e.preventDefault();
			});
		},

		actionButtonHandler: function() {
			var self = this,
				prime = this.prime,
				gui = prime.gui,
				action = prime.action;

			$('body').on('click', '.el-btn-action', function(e){
				var $this = $(this),
					actionname = $this.attr('data-action');

				if ( $this.hasClass('disabled') ) {
					return;
				}

				// clean up
				$('.el-highlight').hide();
				$('.el-selected > .el-resize-handlers').html('');
				$('.el-selected').hide();

				// close element settings
				// help close picker
				if ( $('.prime-colors').is(":visible") ) {
					prime.picker.pickerClose();
				}

				$('#editor-settings').find('.element-settings').remove();

				action.buttons(actionname, $this, e);
				e.preventDefault();
				e.stopPropagation();
			});

			$('body').on('click', '.el-fbtn-action', function(e){
				var $this = $(this),
					actionname = $this.attr('data-action');

				if ( $this.hasClass('disabled') ) {
					return;
				}

				// close element settings
				// help close picker
				if ( $('.prime-colors').is(":visible") ) {
					prime.picker.pickerClose();
				}

				action.buttons(actionname, $this, e);
				e.preventDefault();
				e.stopPropagation();
			});
		},

		settingsHandler: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				iframeDom = prime.frameBody,
				gui = prime.gui,
				apply = prime.apply,
				defSettings = prime.settings;

			$('body').on('focusin', '.setting-input', function(e){
				if ( self.isClicked ) {
					return;
				}

				var $this = $(this),
					oldVal = $this.val();

				self.oldValue = oldVal;

				if ( $('.prime-colors').is(":visible") && !$this.hasClass('color-picker') ) {
					prime.picker.pickerClose();
				}
			});

			var settingChange = function(e){
				if ( self.isClicked ) {
					return;
				}

				var $this = $(this),
					elID = cache.activeElement,
					type = cache.activeElementType,
					settingType = $this.closest('.form-group').attr('data-type'),
					settingFor = $this.closest('.form-group').attr('data-for'),
					settingIdx = $this.closest('.form-group').attr('data-idx'),
					newVal = $this.val(),
					error = false, errorMsg = '';

				// local handler for slider
				if ( settingType == 'slider' || 'border' && newVal != self.oldValue ) {
					self.isClicked = true;
					$this.closest('.fields-group').find('.range-slider, .range-slider-num').val(newVal);
					self.isClicked = false;
				}

				// get settings param
				var setting = defSettings[settingFor] || null;
				if ( setting ) {
					// process setting change
					var fields = setting.fields,
						field = fields[settingIdx],
						settingData = prime.getElementData(type, prime.getElementAlias(iframeDom.find('#' + elID))),
						options = settingData.options || null;

					// required validation
					if ( field.hasOwnProperty('required') && field.required === true && newVal == '' ) {
						newVal = self.oldValue;
						if ( e.type == 'change' ) {
							$this.val(newVal);
							errorMsg = 'Field is required';
							error = true;
						}
						return false;
					}

					// url validation
					// if ( !error && field.type != 'textarea' && field.type != 'htmlform' && field.hasOwnProperty('isURL') && field.isURL === true && newVal != '' ) {
					// 	if ( !PrimeBuilder.Utils.isURL(newVal) ) {
					// 		if ( e.type == 'change' ) {
					// 			errorMsg = 'Please enter a valid URL';
					// 			error = true;
					// 		}
					// 		return false;
					// 	}
					// }
					// 
				
					if ( !error && field.hasOwnProperty('anchor') && field.anchor === true ) {
						newVal = $.trim(newVal.toLowerCase());
						var currentanchor = iframeDom.find('#' + elID).attr('data-anchor') || '';
						if ( $.inArray(e.type, ['change', 'blur']) > -1 && iframeDom.find("[data-anchor='" + newVal + "']").length && newVal !== '' && newVal != currentanchor ) {
							newVal = currentanchor;
							$this.val(newVal);
							errorMsg = 'Please enter a different anchor name';
							error = true;
							return false;
						}

						if ( $.inArray(e.type, ['change', 'blur']) > -1 && newVal != '' && newVal.search(/^[a-zA-Z0-9-_]+$/) == -1 ) {
							newVal = currentanchor;
							$this.val(newVal);
							errorMsg = 'Anchor name contain an invalid character';
							error = true;
							return false;
						}

						if ( !error ) {
							$this.val(newVal);
						}
					}

					if ( error && errorMsg != '' ) {
						//alert(errorMsg);
						$this.val(newVal);
						$this.focus();
						return false;
					}

					if ( field.type == 'color' ) {
						var colorOK = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(newVal);
						if ( newVal != 'transparent' && !colorOK ) {
							$this.val(self.oldValue);
							newVal = self.oldValue;
						}

						var inputColor = prime.utils.inputTextColor(newVal);
						$this.css({
							backgroundColor: newVal,
							color: inputColor
						});
					}

					if ( options ) {
						var option, elem;
						for ( var i = 0; i < options.length; i++ ) {
							if ( options[i].name == settingFor ) {
								option = options[i];
								break;
							}
						}

						if ( typeof option === "undefined" ) {
							return;
						}

						elem = prime.getSettingTarget(elID, option.target);
						if ( field.action == 'showimage' && newVal == '' ) {
							newVal = $('#ib2-img-src').attr('src');
						}

						if ( field.type == 'checkbox' ) {
							var reveal = $this.attr('data-reveal');
							newVal = ( $this.is(":checked") ? "yes" : "no" );
							if ( typeof reveal === "string" && reveal !== 'none' ) {
								if ( $this.is(":checked") ) {
									$this.closest('.settings-fields').find('.' + reveal).removeClass('chk-hidden');
								} else {
									$this.closest('.settings-fields').find('.' + reveal).addClass('chk-hidden');
								}
							}
						}
						
						if ( field.hasOwnProperty('groupReveal') ) {
							var groupClass = field['groupReveal'];

							$this.closest('.settings-fields').find('.' + groupClass).addClass('select-hidden');
							$this.closest('.settings-fields').find('.' + newVal + '-' + groupClass).removeClass('select-hidden');
							if ( groupClass == 'esp-reveal' && typeof ib2esp !== "undefined" ) {
								if ( typeof ib2esp !== "undefined" ) {
									if ( ib2esp.hasOwnProperty(newVal) && ib2esp[newVal] == 'none' ) {
										$this.parent().find('.esp-connect-btn').removeClass('hidden');
										$('.esp-refresh-btn').addClass('hidden');
									} else {
										$this.parent().find('.esp-connect-btn').addClass('hidden');
										$('.esp-refresh-btn').removeClass('hidden');
									}
								}

								if ( prime.esplists.hasOwnProperty(newVal) && prime.esplists[newVal] != 'none' ) {
									gui.updateListOptions(prime.esplists[newVal]);
								} else {
									gui.updateListOptions(false);
								}	
							}
						}

						if ( field.id.indexOf('popup-edit') > -1 ) {
							if ( newVal == 'none' || newVal == '' ) {
								$this.closest('.settings-fields').find('.edit-popup').addClass('hidden');
							} else {
								$this.closest('.settings-fields').find('.edit-popup').removeClass('hidden');
							}
						}

						// font style
						if ( field.type == 'fontstyle' ) {
							field['prop'] = $this.attr('data-prop');
							if ( $this.is(":checked") ) {
								$this.parent().addClass('active');
							} else {
								$this.parent().removeClass('active');
								newVal = $this.attr('data-none');
							}
						}

						if ( field.hasOwnProperty('isURL') && field.isURL === true ) {
							newVal = Base64.encode(newVal);
						}

						if ( field.action == 'backgroundimg' && newVal === '' ) {
							// hide preview
							if ( $this.parent().parent().find('.image-preview').length ) {
								$this.parent().parent().find('.image-preview').addClass('hidden');
							}

							if ( $this.parent().find('.remove-media').length ) {
								$this.parent().find('.remove-media').addClass('hidden');
							}
						}

						// apply change and update data
						var cacheData = {
							id: elID,
							type: type,
							name: option.name,
							element: elem,
							oldValue: self.oldValue,
							value: newVal,
							field: field,
							props: ( setting.hasOwnProperty('props') ? setting.props : null )
						};

						if ( $this.attr('data-num') ) {
							cacheData.num = parseInt($this.attr('data-num'));
						}

						if ( $this.attr('data-key') ) {
							cacheData.key = $this.attr('data-key');
						}

						if ( field.hasOwnProperty('valueMap') ) {
							var values = [];
							field.valueMap.forEach(function(className){
								var mapType = ( field.hasOwnProperty('mapType') ? field.mapType : 'multi-field' );

								if ( mapType == 'single-field' ) {
									values.push($this.closest('.form-group').find('.' + className).val());
								} else if ( mapType == 'multi-field' ) {
									values.push($this.closest('.settings-fields').find('.' + className + ' .form-control').val());
								}
							});
							cacheData.values = values;
						}

						apply.change(cacheData);

						if ( type == 'image' || field.action == 'submitimage' ) {
							$this.parent().find('.remove-media').removeClass('hidden');
						}

						if ( field.hasOwnProperty('onChange') && typeof field['onChange'] === "function" ) {
							field['onChange'](newVal);
						}

						if ( field.action == 'backgroundimg' && newVal !== '' ) {
							if ( $this.parent().parent().find('.image-preview').length && $this.parent().parent().find('.image-preview').hasClass('hidden') ) {
								var preview_url = newVal.replace(/^(https?|ftp):/, '');
								$this.parent().parent().find('.image-preview').find('img').attr('src', preview_url);
								$this.parent().parent().find('.image-preview').removeClass('hidden');
							}

							if ( $this.parent().find('.remove-media').length ) {
								$this.parent().find('.remove-media').removeClass('hidden');
							}
						}

						gui.resizeSelector();
					}
				}
				
				e.preventDefault();
			};

			$('body').on('change', '.setting-input', settingChange);
			$('body').on('keyup', '.setting-input', debounce(settingChange, 500));
		},

		pageSettings: function() {
			var self = this,
				prime = this.prime,
				iframeDom = prime.frameBody,
				gui = prime.gui,
				cache = prime.cache,
				elements = prime.elements;

			$('body').on("click", '.page-settings-btn', function(e){
				self.isClicked = true;

				// clean up
				$('.el-highlight').hide();
				iframeDom.find('.is-selected').removeClass('is-selected');
				$('.el-selected > .el-resize-handlers').html('');
				$('.el-selected').hide();

				// set active element
				cache.activeElement = 'screen-container';
				cache.activeElementType = 'page';

				if ( cache.editorMode == 'desktop' ) {
					$('#main-editor').addClass("edit-settings-on");
				}

				// show settings
				$('#editor-settings').show();

				gui.displaySettings('page', null, elements[0]['options']);
				
				// update with current page settings
				prime.applySettings(elements[0]['options']);

				self.isClicked = false;
				e.preventDefault();
			});

			// permalink
			$('body').on("click", '.change-permalink', function(e){
				var $this = $(this);
				$('#permalink-form').show();
				$('.current-permalink').hide();
				$this.hide();

				e.preventDefault();
			});

			$('body').on("submit", 'form#permalink-form', function(e){
				var $this = $(this);
				var newSlug = $('#ib2-new-slug').val(),
					slug = $('#ib2-current-slug').val();

				if ( slug == newSlug ) {
					$('#permalink-form').hide();
					$('.current-permalink').show();
					$('.change-permalink').show();
				}

				$.ajax({
					type: "POST",
					url: ajaxurl,
					data: {
						action: 'ib2_save_permalink',
						postname: newSlug,
						post_id: $('#ib2-post-id').val(),
					},
					success: function(response) {
						if ( response.success ) {
							prime.permalink = response.permalink;
							prime.postData.post_name = response.post_name;
							prime.postData.permalink = response.permalink;

							$('#ib2-permalink').val(response.permalink);
							$('.ib2-permalink').text(response.permalink);
							$('#ib2-new-slug').val(response.post_name);
							$('#ib2-current-slug').val(response.post_name);

							if ( prime.postStatus == 'publish' ) {
								var newUrl = response.permalink + '?preview=true&variant=' + prime.currentVariation;
								$('.ib2-permalink').attr('href', newUrl);
							}

						}
						$('#permalink-form').hide();
						$('.current-permalink').show();
						$('.change-permalink').show();
					}
				});

				e.preventDefault();
			});
		},

		elementsScroll: function() {
			var prime = this.prime,
				iframeDom = prime.frameBody;

			iframeDom.on("click", ".scroll-elements", function(e){
				e.preventDefault();
			});

			iframeDom.on("mousedown", ".scroll-elements", function(e){
				var $this = $(this),
					dir = $this.attr('data-dir');

				if ( dir == 'down' ) {
					scrollingDown();
				} else {
					scrollingUp();
				}

				e.preventDefault();
			});

			iframeDom.on("mouseup", ".scroll-elements", function(e){
				iframeDom.find('.elements-icons').stop();
				e.preventDefault();
			});
		},

		editorScroll: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				gui = prime.gui,
				action = prime.action,
				options = prime.options,
				elementSelector = options.element.selector,
				iframeDom = prime.frameBody,
				iframeDoc = prime.frameDoc,
				offset,
				target;

			$(window.FrameWindow).on("scroll resize", function(e){
				if ( cache.hoveredEl ) {
					target = self.tuneElementTarget(cache.hoveredEl);
					offset = target.offset();
					$('.el-highlight').css({
						top: offset.top - iframeDoc.scrollTop(),
						left: offset.left - iframeDoc.scrollLeft(),
						width: target.outerWidth() + 'px',
						height: target.outerHeight() + 'px'
					});
				}

				let parentWidth = $('.el-highlight').outerWidth() / 2,
					adderWidth = $('.el-highlight').find('.element-adder').outerWidth() / 2,
					adderLeft = parentWidth - adderWidth;

				$('.el-highlight').find('.element-adder').css('left', adderLeft);

				if ( cache.activeElement && cache.activeElement != 'screen-container' ) {
					gui.resizeSelector();
				}

				if ( cache.popupId || cache.sliderEdit ) {
					var popupId = cache.sliderEdit ? cache.sliderEdit : cache.popupId,
						isSlider = cache.sliderEdit ? true : false;
					action.centerPopupEdit(iframeDom, popupId, isSlider);
				}
			});

			// $(window.FrameWindow).on("resize", function(e){
			// 	gui.adjustEditorSize();
			// });
		},

		elementOver: function() {
			var self = this,
				prime = this.prime,
				cache = prime.cache,
				elements = prime.elements,
				mainSelector = prime.selector,
				options = prime.options,
				cache = prime.cache,
				elementSelector = options.element.selector,
				iframeDom = prime.frameBody,
				iframeDoc = prime.frameDoc;

			iframeDom.on("mousemove", function(e){
				if ( e.target && !(cache.isDragging || cache.isTexting) ) {
					var target = self.getElementTarget(e),
						offset = target.offset(),
						width = target.outerWidth(),
						height = target.outerHeight();

					if ( target.hasClass(elementSelector) ) {
						var display = ( target.hasClass('is-selected') ? 'none' : 'block' ),
							elID = ( target.hasClass('ib2-element') ? target.attr('id') : target.closest('.ib2-element').attr('id') );

						cache.hoveredEl = iframeDom.find('#' + elID);

						// Element Name
						$('.el-name').text(self.getElementName(target));
						// Element Highlight
						$('.el-highlight').css({
							top: offset.top - iframeDoc.scrollTop(),
							left: offset.left - iframeDoc.scrollLeft(),
							width: width + 'px',
							height: height + 'px',
							display: display
						});

						if ( iframeDom.find('#' + elID).parent().attr('id') == 'screen-container' ) {
							if ( cache.popupMode ) {
								$('.el-highlight').find('.section-adder').hide();
							} else {
								$('.el-highlight').find('.section-adder').show();
							}
						} else {
							$('.el-highlight').find('.section-adder').hide();
						}

						let parentWidth = $('.el-highlight').outerWidth() / 2,
							adderWidth = $('.el-highlight').find('.element-adder').outerWidth() / 2,
							adderLeft = parentWidth - adderWidth;

						$('.el-highlight').find('.element-adder').css('left', adderLeft);

						var elWidth = $('.el-highlight').outerWidth();
						if ( elWidth <= 100 ) {
							$('.el-highlight').find('.element-adder').hide();
						} else {
							$('.el-highlight').find('.element-adder').show();
						}

						var alias = prime.getElementAlias(iframeDom.find('#' + elID)),
							buttons = self.getButtons(cache.hoveredEl, cache.hoveredEl.attr('data-el'), alias, true);

						$('.el-fbuttons > .btn-group').html(buttons);

						if ( elWidth <= 100 ) {
							$('.el-fbuttons').css('left', '-75px');
						} else if ( cache.hoveredEl.parent().attr('id') == 'screen-container'  ) {
							$('.el-fbuttons').css('left', '0px').show();
						} else {
							$('.el-fbuttons').css('left', '-20px').show();
						}
					}
				}
			});

			// register element scroll handler
			this.editorScroll();
		},

		elementClick: function() {
			var self = this,
				prime = this.prime,
				elements = prime.elements,
				mainSelector = prime.selector,
				options = prime.options,
				gui = prime.gui,
				cache = prime.cache,
				elementSelector = options.element.selector,
				activeClass = options.element.active,
				iframeDom = prime.frameBody,
				iframeDoc = prime.frameDoc;

			iframeDom.on("click", function(e){
				var hovered = cache.hoveredEl;

				if ( self.isClicked ) {
					return;
				}

				self.isClicked = true;
				// help close picker
				if ( $('.prime-colors').is(":visible") ) {
					prime.picker.pickerClose();
				}

				if ( hovered && e.target && !(cache.isDragging || cache.isTexting) ) {
					var target = hovered;
					if ( target.attr('data-el') ) {
						var elID = target.attr('id'),
							type = target.attr('data-el');

						// set active element
						cache.activeElement = elID;
						cache.activeElementType = type;	
						cache.hoveredEl = null;

						// clean up
						$('.el-highlight').hide();
						iframeDom.find('.is-selected').removeClass('is-selected');
						$('.el-selected > .el-resize-handlers').html('');

						// show settings
						if ( cache.editorMode == 'desktop' ) {
							$('#main-editor').addClass("edit-settings-on");
						}

						if ( !$('#editor-settings').is(":visible") ) {
							$('#editor-settings').show();
						}

						// target found
						var clicked = iframeDom.find('#' + elID);
						if ( !clicked.hasClass(elementSelector) ) {
							clicked = clicked.find(" > ." + elementSelector);
						}

						if ( clicked.hasClass(activeClass) ) {
							return;
						}

						var alias = prime.getElementAlias(iframeDom.find('#' + elID)),
							elemData = prime.getElementData(type, alias);

						gui.displaySettings(type, alias, elemData.options);

						// update with current element settings
						prime.applySettings(elemData.options);

						// show active lines and buttons
						clicked.addClass(activeClass);

						var offset = clicked.offset(),
							width = clicked.outerWidth(),
							height = clicked.outerHeight();

						// element selected
						$('.el-selected').css({
							top: offset.top - iframeDoc.scrollTop(),
							left: offset.left - iframeDoc.scrollLeft(),
							width: width + 'px',
							height: height + 'px',
							display: 'block'
						});

						if ( clicked.hasClass('ib2-slider-el') ) {
							$('.el-selected').css('z-index', 999);
						} else {
							$('.el-selected').css('z-index', 98);
						}

						// show buttons
						var buttons = self.getButtons(clicked, type, alias);
						$('.el-buttons > .btn-group').html(buttons);

						var elWidth = $('.el-highlight').outerWidth(),
							elHeight = $('.el-highlight').outerHeight();
						if ( elWidth <= 100 || elHeight <= 60 ) {
							$('.el-buttons').css({
								'top': '-30px',
								'right': '-2px',
							});
							$('#el-btn-moveup').hide();
							$('#el-btn-movedown').hide();
						} else {
							$('.el-buttons').css({
								'top': '-2px',
								'right': '-2px',
							});
							$('#el-btn-moveup').show();
							$('#el-btn-movedown').show();
						}

						if ( type == 'countdown' && !clicked.find('.ib2-countdown-reveal').length ) {
							$('#el-btn-countdown-normal').hide();
							$('#el-btn-countdown-hidden').hide();
						} else {
							$('#el-btn-countdown-normal').show();
							$('#el-btn-countdown-hidden').show();

							if ( clicked.find('.ib2-countdown-reveal').is(":visible") ) {
								$('#el-btn-countdown-normal').removeClass('disabled');
								$('#el-btn-countdown-hidden').addClass('disabled');
							} else {
								$('#el-btn-countdown-normal').addClass('disabled');
								$('#el-btn-countdown-hidden').removeClass('disabled');
							}
						}

						// show resizable handlers if required
						var resizer = self.getResizer(clicked, type, alias);
						$('.el-selected > .el-resize-handlers').html(resizer);

						if ( $.inArray(type, ['section', 'wsection']) > -1 ) {
							if ( cache.popupMode ) {
								$('#el-btn-clone').hide();
								$('#el-btn-moveup').hide();
								$('#el-btn-movedown').hide();
								$('#el-btn-visibility').hide();
								$('#el-btn-save').hide();
								$('#el-btn-delete').hide();
							} else {
								$('#el-btn-clone').show();
								$('#el-btn-moveup').show();
								$('#el-btn-movedown').show();
								$('#el-btn-visibility').show();
								$('#el-btn-save').show();
								$('#el-btn-delete').show();
							}
						}

						// position the resizers
						var isSlider = clicked.hasClass('ib2-slider-el') ? true : false;
						if ( resizer != '' && isSlider !== true ) {
							$('.el-resizer').hide();

							var w = $('.el-selected').innerWidth(), 
								h = $('.el-selected').innerHeight();

							if ( cache.popupMode === true && type == 'section' ) {
								elemData.resizable = ["s", "e"];
							}

							$('.el-n-resize').css({ top: '-5px', left: ((w/2)-4) + 'px' });
							$('.el-s-resize').css({ top: (h-4) + 'px', left: ((w/2)-4) + 'px' });
							$('.el-w-resize').css({ top: ((h/2)-4) + 'px', left: '-6px' });
							$('.el-e-resize').css({ top: ((h/2)-4) + 'px', left: (w-4) + 'px' });
							$('.el-se-resize').css({ top: (h-4) + 'px', left: (w-4) + 'px' });

							for ( var r = 0; r < elemData.resizable.length; r++ ) {
								$('.el-' + elemData.resizable[r] + '-resize').show();
							}
						}

						// display column resizer?
						if ( type == 'columns' ) {
							gui.columnResizer(elID, type);
						} else {
							$('.el-col-resizer').remove();
						}
					}
				}
				self.isClicked = false;
				e.preventDefault();
				//e.stopPropagation();
			});

			// questions element click and edit
			iframeDom.on('click', '.quiz-text-edit', function(e){
				var $this = $(this);

				$this.addClass("text-editor");
				$this.attr('contenteditable', true);
				$this.focus();
				
				prime.disableSortable();

				cache.isTexting = true;
				e.preventDefault();
			});

			iframeDom.on('blur', '.quiz-text-edit', function(e){
				var $this = $(this);
				$this.attr('contenteditable', false);
				$this.removeAttr('contenteditable');
				$this.removeClass("text-editor");

				setTimeout(function(){
					cache.isTexting = false;
					prime.sortable();
				}, 300);
				e.preventDefault();
			});
		},

		globalTabs: function() {
			var prime = this.prime;
			$("body").on('click', '.global-tab-link', function(e) {
				var $this = $(this), 
					current = $('#editor-settings').find('.tab-settings-active'),
					currentClass = $('#editor-settings').find('.tab-settings-active').attr('data-tab'),
					thisClass = $this.attr('data-tab');

				current.removeClass('tab-settings-active');
				$this.addClass('tab-settings-active');

				$('#editor-settings').find('.' + currentClass).addClass('hidden');
				$('#editor-settings').find('.' + thisClass).removeClass('hidden');

				e.preventDefault();
			});
		},

		settingTabs: function() {
			var prime = this.prime;
			$("body").on('click', '.setting-tab-link', function(e) {
				// help close picker
				if ( $('.prime-colors').is(":visible") ) {
					prime.picker.pickerClose();
				}

				if ( $(this).hasClass('tab-active') ) {
					return false;
				}

				var $this = $(this),
					tabsID = $this.closest('.settings-tabs').attr('id'),
					fieldsID = tabsID.replace("-tabs", ''),
					target = $this.attr('href');

				if ( target ) {
					target = target.replace('#', '');
				}

				$this.closest('.settings-tabs').find('.tab-active').removeClass('tab-active');
				$('#' + fieldsID + ' > div').each(function(){
					// hide current tabs
					if ( $(this).hasClass('group-' + target) ) {
						$(this).removeClass('hidden');
					} else {
						$(this).addClass('hidden');
					}
				});
				$this.addClass('tab-active');

				e.preventDefault();
			});

			// register global tabs here
			this.globalTabs();
		},

		settingToggle: function() {
			var prime = this.prime,
				iframeDom = prime.frameBody,
				gui = prime.gui;

			$("body").on('click', '.settings-toggle', function(e){
				var $this = $(this).closest('.settings-header'),
					settingFor = $this.attr('data-for'),
					type = $this.attr('data-type'),
					settingId = 'fields-' + type + '-' + settingFor,
					settingTabId = 'fields-' + type + '-' + settingFor + '-tabs';

				// help close picker
				if ( $('.prime-colors').is(":visible") ) {
					prime.picker.pickerClose();
				}

				if ( $('#' + settingId).is(":visible") ) {
					$('#' + settingId).hide();
					if ( $('#' + settingTabId).length ) {
						$('#' + settingTabId).hide();
					}

					if (! $this.find('.settings-mode').hasClass('hidden') ) {
						$this.find('.settings-mode').hide();
					}
				} else {
					$('#' + settingId).show();
					if ( $('#' + settingTabId).length ) {
						$('#' + settingTabId).show();
					}

					if (! $this.find('.settings-mode').hasClass('hidden') ) {
						$this.find('.settings-mode').css('display', 'inline-block');
					}
				}
				e.preventDefault();
			});

			// collapse
			iframeDom.on('click', '.ib2-collapse-head', function(e){
				var $this = $(this),
					panel = $this.parent().find('.ib2-collapse-panel');

				if ( $this.hasClass('active') ) {
					panel.css('max-height', '0px');
					$this.removeClass('active');
				} else {
					var panelHeight = panel.find('.ib2-collapse-content').outerHeight();
					panel.css('max-height', panelHeight + 'px');
					$this.addClass('active');

					setTimeout(function(){
						panel.css('max-height', 'none');
					}, 50);
				}
				e.preventDefault();
			});
		},

		closeSettings: function() {
			var prime = this.prime,
				cache = prime.cache,
				action = prime.action,
				gui = prime.gui;

			$("body").on("click", '.close-settings-bar', function(e){
				var $this = $(this);
				// help close picker
				if ( $('.prime-colors').is(":visible") ) {
					prime.picker.pickerClose();
				}

				$('#main-editor').removeClass("edit-settings-on");
				$('#editor-settings').hide();

				// resize Selectors
				gui.resizeSelector();

				// resize Backgrounds
				gui.resizeBackgrounds();

				e.preventDefault();
			});

			$('body').on("click", ".close-setting-popup", function(e){
				action.closePopup(cache.currentPopSetting, true);
				e.preventDefault();
			});

			$('body').on("click", ".setting-popup-overlay", function(e){
				action.closePopup(cache.currentPopSetting, true);
				e.preventDefault();
			});

			$('body').on("tinymcedestroy", function(e){
				action.removeTinyMce();
			});
		},

		getElementTarget: function(event, click) {
			var e = event;
			var target = $(e.target);

			return this.tuneElementTarget(target, click);
		},

		tuneElementTarget: function(target, click) {
			var prime = this.prime,
				options = prime.options,
				elementSelector = options.element.selector;

			if ( target && !target.hasClass(elementSelector) ) {
				if ( target.closest('.' + elementSelector).length ) {
					target = target.closest('.' + elementSelector);
				} else {
					target = target.parent();
				}
			}

			if ( target && click && target.hasClass('el-content') ) {
				target = target.parent();
			}
			return target;
		},

		getElementName: function(target) {
			var prime = this.prime,
				cache = prime.cache,
				utils = prime.utils,
				elName = target.attr('data-el') ? target.attr('data-el') : '';

			if ( elName == '' && target.hasClass('el-content') ) {
				elName = target.parent().attr('data-el') ? target.parent().attr('data-el') : '';
			}

			if ( elName == 'wsection' || elName == 'section' ) {
				elName = ( cache.popupMode ? 'Pop Up' : 'Section' );
			}

			if ( elName == 'wbox' ) {
				elName = 'Background Box';
			}

			if ( elName == 'optin3' ) {
				elName = '3-steps opt-in';
			}

			return utils.ucwords(elName);
		},

		getButtons: function (el, type, alias, hover) {
			var prime = this.prime;
			return prime.getButtonGui(el, type, alias, hover);
		},

		getResizer: function (el, type, alias) {
			var prime = this.prime;
			return prime.getResizerGui(el, type, alias);
		}
	}
})(PrimeBuilder, window, document, jQuery);