(function (PrimeBuilder, w, d, $) { 
	"use strict";

	/**
	 * Color Picker
	 */
	PrimeBuilder.Picker = function (prime) {
		this.prime = prime;
		this.pickerID = null;
	};

	PrimeBuilder.Picker.prototype = {
		colors: ["#5B0F00","#660000","#783F04","#7F6000","#274E13","#0C343D","#1C4587","#073763","#20124D","#4C1130",
			"#5B0F00","#660000","#783F04","#7F6000","#274E13","#0C343D","#1C4587","#073763","#20124D","#4C1130",
			"#85200C","#990000","#B45F06","#BF9000","#38761D","#134F5C","#1155CC","#0B5394","#351C75","#741B47",
			"#A61C00","#CC0000","#E69138","#F1C232","#6AA84F","#45818E","#3C78D8","#3D85C6","#674EA7","#A64D79",
			"#CC4125","#E06666","#F6B26B","#FFD966","#93C47D","#76A5AF","#6D9EEB","#6FA8DC","#8E7CC3","#C27BA0",
			"#DD7E6B","#EA9999","#F9CB9C","#FFE599","#B6D7A8","#A2C4C9","#A4C2F4","#9FC5E8","#B4A7D6","#D5A6BD",
			"#E6B8AF","#F4CCCC","#FCE5CD","#FFF2CC","#D9EAD3","#D0E0E3","#C9DAF8","#CFE2F3","#D9D2E9","#EAD1DC",
			"#980000","#FF0000","#FF9900","#FFFF00","#00FF00","#00FFFF","#4A86E8","#0000FF","#9900FF","#FF00FF",
			"#000000","#222222","#444444","#666666","#888888","#AAAAAA","#CCCCCC","#DDDDDD","#EEEEEE","#FFFFFF"],

		setup: function() {
			$('body').append(this.pickersHtml());

			// events
			this.openPicker();
			this.highlightPicker();
			this.pickerButtons();
			this.selectColor();

			// load iris
			let self = this,
				prime = this.prime;
			$('.custom-color-picker').iris({
				width: 240,
				hide: false,
				change: function(event, ui) {
					let color = ui.color.toString();

					if ( self.pickerID ) {
						$('#' + self.pickerID).val(color);
						let inputColor = prime.utils.inputTextColor(color);
						$('#' + self.pickerID).css({
							backgroundColor: color,
							color: inputColor
						});

						// trigger event change
						$('#' + self.pickerID).trigger("change");
					}
				}
			});
		},

		selectColor: function() {
			let self = this,
				prime = this.prime;

			$('body').on('click', '.picker-selector', function(e){
				let $this = $(this),
					val = $this.attr('data-color');

				if ( self.pickerID ) {
					$('#' + self.pickerID).val(val);
					let inputColor = prime.utils.inputTextColor(val);
					$('#' + self.pickerID).css({
						backgroundColor: val,
						color: inputColor
					});

					// trigger event change
					$('#' + self.pickerID).trigger("change");
				}
				e.preventDefault();
			});
		},

		openPicker: function() {
			let self = this;
			$('body').on("focus", ".color-picker", function(e){
				let $this = $(this),
					parent = $('#editor-settings').position(),
					position = $this.position(),
					pWidth = $('.prime-colors').outerWidth(),
					pHeight = $('.prime-colors').outerHeight(),
					top = position.top,
					left = parent.left;

				self.pickerID = $this.attr('id');

				$('.prime-colors').css({
					top: (top - (pHeight/2) + 10) + 'px',
					left: (left - pWidth - 5) + 'px',
					display: 'block'
				});
			});
		},

		highlightPicker: function() {
			$('body').on('mouseover', '.picker-selector', function(e){
				let $this = $(this);

				$('.color-selected').removeClass('.color-selected');
				$this.addClass('color-selected');
			});

			$('body').on('mouseout', '.picker-selector', function(e){
				let $this = $(this);

				$this.removeClass('color-selected');
			});
		},

		pickerButtons: function() {
			let self = this;
			$('body').on('click', '.picker-toggle', function(e){
				let $this = $(this);

				if ( $('.custom-picker').is(":visible") ) {
					$('.custom-picker').hide();
					$('.color-pickers').show();
				} else {
					$('.custom-picker').show();
					$('.color-pickers').hide();

					var currentColor = $('#' + self.pickerID).val();

					if ( currentColor ) {
						// update iris
						$('.custom-color-picker').iris('color', currentColor);
					}
				}
				e.preventDefault();
			});

			$('body').on('click', '.close-picker', function(e){
				self.pickerClose();
				e.preventDefault();
			});
		},

		pickerClose: function() {
			// reset
			$('.custom-picker').hide();
			$('.color-pickers').show();

			$('.prime-colors').hide();

			this.pickerID = null;
		},

		pickersHtml: function() {
			let output = 
					'<div class="prime-colors popover left">\
						<div class="arrow"></div>\
						<h3 class="popover-title">\
							<button type="button" class="close close-picker" aria-label="Close"><span aria-hidden="true">&times;</span></button>\
							Choose a Color</h3>\
						<div class="popover-content">\
							<div class="color-pickers">\
								<div class="picker-selector transparent-color" data-color="transparent" title="Transparent"></div>';


			this.colors.forEach(function(color){
				output += '<div class="picker-selector" style="background-color: ' + color + '" data-color="' + color + '"></div>';
			});

			output += 			
								'<div class="clearfix"></div>\
								<button type="button" class="btn btn-block btn-default picker-toggle">Advanced Mode</button>\
							</div>\
							<div class="custom-picker">\
								<div class="form-group">\
									<input type="hidden" class="custom-color-picker" />\
								</div>\
								<button type="button" class="btn btn-block btn-default picker-toggle">Simple Mode</button>\
							</div>\
						</div>\
					</div>';

			return output;
		}
	};

})(PrimeBuilder, window, document, jQuery);