PrimeBuilder.Settings = {
	general: {
		mode: false,
		title: 'General',
		id: 'page-general',
		tabs: [{ key: 'basic', name: 'Page Info' }, { key: 'width', name: 'Page Width' }, { key: 'indexing', name: 'Robots' }],
		defaultTab: 'basic',
		fields: [
			{
				id: 'page-title',
				label: 'Page Title',
				type: 'text',
				placeholder: 'e.g. My Home Page',
				action: 'savedata',
				file: 'title',
				mode: "basic",
			},
			{
				id: 'page-desc',
				label: 'Description',
				type: 'textarea',
				action: 'savedata',
				file: 'desc',
				mode: "basic",
			},
			{
				id: 'page-keywords',
				label: 'Keywords',
				type: 'text',
				action: 'savedata',
				file: 'keywords',
				mode: "basic",
			},
			{
				id: 'favicon',
				label: 'Favicon',
				placeholder: 'e.g. https://yourdomain.com/company.ico',
				type: 'upload',
				action: 'savedata',
				file: 'favicon',
				mode: "basic",
			},
			{
				id: 'featured-image',
				label: 'Page Image',
				placeholder: 'e.g. https://yourdomain.com/image.jpg',
				type: 'upload',
				action: 'savedata',
				file: 'featured',
				mode: "basic",
				desc: "If not empty, the image will be used as the primary image when visitors share your page in social sites unless the social metadata feature is disabled."
			},
			{
				id: 'nometasocial',
				label: 'Disable Social Metadata',
				type: 'checkbox',
				action: 'savedata',
				file: 'nometasocial',
				mode: "basic",
			},
			{
				id: 'page-width',
				label: 'Page Width',
				type: 'slider',
				unit: 'px',
				min: 600,
				max: 1600,
				action: 'pagewidth',
				file: 'width',
				mode: "width",
			},
			{
				id: 'noindex',
				label: 'noindex',
				type: 'checkbox',
				action: 'savedata',
				file: 'noindex',
				mode: "indexing",
			},
			{
				id: 'nofollow',
				label: 'nofollow',
				type: 'checkbox',
				action: 'savedata',
				file: 'nofollow',
				mode: "indexing",
			},
			{
				id: 'noarchive',
				label: 'noarchive',
				type: 'checkbox',
				action: 'savedata',
				file: 'noarchive',
				mode: "indexing",
			},
		],
		files: [
			'title',
			'desc',
			'keywords',
			'favicon',
			'noindex',
			'nofollow',
			'noarchive',
			'noodp',
			'noydir',
		]
	},

	pagetypo: {
		mode: false,
		title: 'Typography',
		tabs: [{ key: 'format', name: 'Base' }, , { key: 'sizes', name: 'Sizes' }, { key: 'colors', name: 'Colors' }],
		defaultTab: 'format',
		id: 'page-typography',
		fields: [
			{
				id: 'heading-font',
				label: 'Heading Font Family',
				type: 'fonts',
				options: ib2fonts,
				action: 'css',
				cssSelector: '{elID} h1, {elID} h2, {elID} h3, {elID} h4, {elID} h5, {elID} h6',
				defaultValue: "'Open Sans',sans-serif",
				prop: 'fontFamily@active',
				mode: 'format',
			},
			{
				id: 'page-font',
				label: 'Text Font Family',
				type: 'fonts',
				options: ib2fonts,
				action: 'css',
				cssSelector: '{elID}',
				defaultValue: "'Open Sans',sans-serif",
				prop: 'fontFamily',
				mode: 'format',
			},
			{
				id: 'page-line-height',
				label: 'Text Line Height',
				type: 'slider',
				defaultValue: 1.4,
				min: 1,
				max: 3,
				step: 0.1,
				action: 'css',
				cssSelector: '{elID} p',
				prop: 'lineHeight',
				mode: 'format',
			},
			{
				id: 'page-white-space',
				label: 'Space Between Paragraphs',
				type: 'slider',
				unit: 'px',
				defaultValue: 24,
				min: 1,
				max: 50,
				action: 'css',
				cssSelector: '{elID} p',
				prop: 'paddingBottom',
				mode: 'format',
			},
			{
				id: 'break-word',
				label: 'Enable Word Break',
				type: 'checkbox',
				action: 'breakword',
				file: 'breakWord',
				mode: "format",
			},
			{
				id: 'page-text-size',
				label: 'Base Font Size',
				type: 'slider',
				unit: 'px',
				defaultValue: 14,
				min: 10,
				max: 150,
				action: 'css',
				cssSelector: '{elID}',
				prop: 'fontSize',
				mode: 'sizes',
			},
			{
				id: 'page-h1-size',
				label: 'Heading 1 Size',
				type: 'slider',
				unit: 'px',
				defaultValue: 68,
				min: 10,
				max: 150,
				action: 'css',
				cssSelector: '.ib2-text-el h1',
				prop: 'h1FontSize',
				mode: 'sizes',
			},
			{
				id: 'page-h2-size',
				label: 'Heading 2 Size',
				type: 'slider',
				unit: 'px',
				defaultValue: 42,
				min: 10,
				max: 150,
				action: 'css',
				cssSelector: '.ib2-text-el h2',
				prop: 'h2FontSize',
				mode: 'sizes',
			},
			{
				id: 'page-h3-size',
				label: 'Heading 3 Size',
				type: 'slider',
				unit: 'px',
				defaultValue: 26,
				min: 10,
				max: 150,
				action: 'css',
				cssSelector: '.ib2-text-el h3',
				prop: 'h3FontSize',
				mode: 'sizes',
			},
			{
				id: 'page-h4-size',
				label: 'Heading 4 Size',
				type: 'slider',
				unit: 'px',
				defaultValue: 22,
				min: 10,
				max: 150,
				action: 'css',
				cssSelector: '.ib2-text-el h4',
				prop: 'h4FontSize',
				mode: 'sizes',
			},
			{
				id: 'page-h5-size',
				label: 'Heading 5 Size',
				type: 'slider',
				unit: 'px',
				defaultValue: 18,
				min: 10,
				max: 150,
				action: 'css',
				cssSelector: '.ib2-text-el h5',
				prop: 'h5FontSize',
				mode: 'sizes',
			},
			{
				id: 'page-h6-size',
				label: 'Heading 6 Size',
				type: 'slider',
				unit: 'px',
				defaultValue: 16,
				min: 10,
				max: 150,
				action: 'css',
				cssSelector: '.ib2-text-el h6',
				prop: 'h6FontSize',
				mode: 'sizes',
			},
			{
				id: 'page-text-color',
				label: 'Text Color',
				type: 'color',
				action: 'css',
				cssSelector: '{elID}',
				defaultValue: '#3a3a3a',
				prop: 'color',
				mode: 'colors',
			},
			{
				id: 'page-link-color',
				label: 'Link Color',
				type: 'color',
				action: 'css',
				cssSelector: '{elID} a',
				defaultValue: '#428bca',
				prop: 'color@active',
				mode: 'colors',
			},
			{
				id: 'page-link-hover-color',
				label: 'Link Hover Color',
				type: 'color',
				action: 'css',
				cssSelector: '{elID} a:hover',
				defaultValue: '#2a6496',
				prop: 'color@hover',
				mode: 'colors',
			},
		],
		files: [
			'breakWord',
		],
		props: [
			'fontFamily',
			'color',
			'color@active',
			'color@hover'
		],
		customCSS: [
			'fontFamily',
			'fontSize',
			'lineHeight',
			'paddingBottom',
			'color',
		]
	},

	pageadvanced: {
		mode: false,
		title: 'Manage Data',
		id: 'page-advanced',
		fields: [
			{
				id: 'save-template',
				label: 'Save as Template',
				type: 'savetemplate',
				action: 'none',
				file: 'savetemplate'
			},
		],
		files: [
			'savetemplate'
		]
	},

	popup: {
		mode: false,
		title: 'Pop Up',
		id: 'global-popup',
		fields: [
			{
				id: 'popup-toggle',
				label: 'Enable Pop Up',
				type: 'checkbox',
				action: 'savedata',
				file: 'active',
				reveal: 'show-on-reveal',
			},
			{
				id: 'popup-load-time',
				label: 'Load Timing',
				type: 'select',
				options: {
					pageload: 'Each time the page is loaded',
					firsttime: 'First time visit only',
					daily: 'Once a day',
					weekly: 'Once a week',
					monthly: 'Once a month',
					session: 'Once per browser session',
					unfocus: 'Exit Intent'
				},
				action: 'savedata',
				file: 'time',
				classes: 'show-on-reveal chk-hidden',
			},
			{
				id: 'popup-edit',
				label: 'Pop Up',
				type: 'select',
				action: 'savedata',
				options: 'popupdata',
				defaultValue: 'none',
				file: 'popid',
				classes: 'show-on-reveal chk-hidden',
			}
		],
		files: [
			'active',
			'time',
			'popid'
		]
	},

	popsetup: {
		mode: false,
		title: 'General',
		id: 'general-popup',
		fields: [
			{
				id: 'popup-name',
				label: 'Name',
				type: 'text',
				action: 'savedata',
				file: 'popname',
				required: true,
				desc: 'For identification only. The name will not be shown to your site visitors.'
			},
		],
		files: [
			'popname'
		]
	},

	slider: {
		mode: false,
		title: 'Bottom Slider',
		id: 'bottom-slider',
		fields: [
			{
				id: 'slider-toggle',
				label: 'Enable Bottom Slider',
				type: 'checkbox',
				action: 'savedata',
				file: 'active',
				reveal: 'show-on-reveal',
			},
			{
				id: 'slider-load-time',
				label: 'Load Timing',
				type: 'select',
				options: {
					pageload: 'Each time the page is loaded',
					firsttime: 'First time visit only',
					daily: 'Once a day',
					weekly: 'Once a week',
					monthly: 'Once a month',
					session: 'Once per browser session',
					unfocus: 'Exit Intent'
				},
				action: 'savedata',
				file: 'time',
				classes: 'show-on-reveal chk-hidden',
			},
			{
				id: 'slider-edit',
				label: 'Edit Slider',
				type: 'sliderpop',
				action: 'none',
				file: 'slideredit',
				classes: 'show-on-reveal chk-hidden',
			},
		],
		files: [
			'active',
			'time'
		]
	},
	infobar: {
		mode: false,
		tabs: [{ key: 'setup', name: 'Setup' }, { key: 'style', name: 'Style' }],
		defaultTab: 'setup',
		title: 'Notification Bar',
		id: 'global-infobar',
		fields: [
			{
				id: 'infobar-toggle',
				label: 'Enable Notification Bar',
				type: 'checkbox',
				action: 'infobar',
				file: 'active',
				reveal: 'show-on-reveal',
				mode: 'setup',
			},
			{
				id: 'infobar-load-time',
				label: 'Load Timing',
				type: 'select',
				options: {
					pageload: 'Each time the page is loaded',
					firsttime: 'First time visit only',
					daily: 'Once a day',
					weekly: 'Once a week',
					monthly: 'Once a month',
					session: 'Once per browser session',
					unfocus: 'Exit Intent'
				},
				action: 'savedata',
				file: 'time',
				classes: 'show-on-reveal chk-hidden',
				mode: 'setup',
			},
			{
				id: 'infobar-msg',
				label: 'Message',
				type: 'textarea',
				action: 'infobar',
				file: 'text',
				classes: 'show-on-reveal chk-hidden',
				mode: 'setup',
			},
			{
				id: 'infobar-anchor',
				label: 'Link Text',
				type: 'text',
				action: 'infobar',
				file: 'anchor',
				classes: 'show-on-reveal chk-hidden',
				mode: 'setup',
			},
			{
				id: 'infobar-url',
				label: 'Link URL',
				placeholder: "e.g. https://www.anysite.com/",
				type: 'text',
				action: 'infobar',
				file: 'url',
				classes: 'show-on-reveal chk-hidden',
				mode: 'setup',
			},
			{
				id: 'infobar-font',
				label: 'Font Family',
				type: 'fonts',
				options: ib2fonts,
				action: 'infobar',
				file: 'fontFamily',
				classes: 'show-on-reveal chk-hidden',
				mode: 'style',
			},
			{
				id: 'infobar-text-color',
				label: 'Text Color',
				type: 'color',
				action: 'infobar',
				file: 'color',
				classes: 'show-on-reveal chk-hidden',
				mode: 'style'
			},
			{
				id: 'infobar-bg-color',
				label: 'Background Color',
				type: 'color',
				action: 'infobar',
				file: 'backgroundColor',
				classes: 'show-on-reveal chk-hidden',
				mode: 'style'
			},
			{
				id: 'infobar-btnbg-color',
				label: 'Button Background Color',
				type: 'color',
				action: 'infobar',
				prop: 'backgroundColor@active',
				defaultValue: '#3c78d8',
				classes: 'show-on-reveal chk-hidden',
				mode: 'style'
			},
			{
				id: 'infobar-border-color',
				label: 'Border Color',
				type: 'color',
				action: 'infobar',
				file: 'border',
				classes: 'show-on-reveal chk-hidden',
				mode: 'style'
			},
		],
		files: [
			'active',
			'time',
			'text',
			'anchor',
			'url',
			'fontFamily',
			'color',
			'backgroundColor',
			'backgroundColor@active',
			'border',
		]
	},

	gateway: {
		mode: false,
		title: 'Gateway Page',
		id: 'gateway-page',
		fields: [
			{
				id: 'enable-gateway',
				label: 'Enable Gateway Page',
				type: 'checkbox',
				action: 'savedata',
				file: 'active',
				reveal: 'show-on-reveal',
			},
			{
				id: 'gateway-type',
				label: 'Mode',
				type: 'select',
				action: 'savedata',
				options: {
					welcome: 'Auto-Unlock After First Time Visit',
					launch: 'Unlock Only After Subscribed'
				},
				file: 'type',
				classes: 'show-on-reveal chk-hidden',
				groupReveal: 'gateway-reveal',
			},
			{
				id: 'gateway-posts',
				label: 'Posts/Pages To Be Locked',
				type: 'posts',
				action: 'savedata',
				options: ib2posts,
				multiple: true,
				file: 'posts',
				classes: 'show-on-reveal chk-hidden',
			},
			{
				id: 'gateway-thanks',
				label: 'Opt-In Thank You Page',
				type: 'posts',
				action: 'savedata',
				firstOption: '-- select a post/page --',
				options: ib2posts,
				file: 'thanksId',
				classes: 'show-on-reveal chk-hidden gateway-reveal launch-gateway-reveal',
				onChange: function(newValue) {
					$('#gateway-gateway-url-page').val('Generating URL ...');
					$.ajax({
						type: "GET",
						url: ajaxurl,
						data: {
							action: 'ib2_gate_url',
							thanks_id: newValue,
	  						code: $('#ib2-gate-code').val()
						},
						success: function(response) {
							if ( response.success ) {
								$('#gateway-gateway-url-page').val(response.url);
							}
						}
					});
				}
			},
			{
				id: 'gateway-url',
				label: 'Custom Thank You Page URL',
				type: 'gateway',
				action: 'none',
				file: 'gateway-url',
				placeholder: 'Please select a thank you page first',
				classes: 'show-on-reveal chk-hidden gateway-reveal launch-gateway-reveal',
			},
		],
		files: [
			'active',
			'posts',
			'thanksId'
		]
	},

	rightclick: {
		mode: false,
		title: 'Right Click',
		id: 'disable-right-click',
		fields: [
			{
				id: 'disable-rclick',
				label: 'Disable Right Click',
				type: 'checkbox',
				action: 'savedata',
				file: 'active',
				reveal: 'show-on-reveal',
			},
			{
				id: 'rclick-message',
				label: 'Message',
				type: 'textarea',
				action: 'savedata',
				file: 'text',
				classes: 'show-on-reveal chk-hidden',
				desc: 'Enter some text if you want to display a warning message. Otherwise, leave this blank.',
			},
			{
				id: 'rclick-image',
				label: 'Disable Right Click on Image Only',
				type: 'checkbox',
				action: 'savedata',
				file: 'imageOnly',
				classes: 'show-on-reveal chk-hidden',
			},
		],
		files: [
			'active',
			'text',
			'imageOnly',
		]
	},

	scripts: {
		mode: false,
		title: 'Scripts',
		id: 'page-scripts',
		fields: [
			{
				id: 'head-scripts',
				label: 'Head',
				type: 'textarea',
				action: 'savedata',
				isURL: true,
				file: 'head',
				desc: 'Enter javascript or css code into html document header',
			},
			{
				id: 'body-scripts',
				label: 'Body',
				type: 'textarea',
				action: 'savedata',
				isURL: true,
				file: 'body',
				desc: 'Enter html, javascript or css code right after the opening <code>body</code> tag',
			},
			{
				id: 'footer-scripts',
				label: 'Footer',
				type: 'textarea',
				action: 'savedata',
				isURL: true,
				file: 'footer',
				desc: 'Enter html, javascript or css code right before the closing <code>body</code> tag',
			},
		],
		files: [
			'head',
			'body',
			'footer',
		]
	},

	typography: {
		mode: false,
		title: 'Typography',
		id: 'page-typography',
		fields: [
			{
				id: 'typo-font',
				label: 'Font Family',
				type: 'fonts',
				options: ib2fonts,
				action: 'css',
				cssSelector: '{elID}',
				defaultValue: 'inherit',
				prop: 'fontFamily',
			},
			{
				id: 'typo-line-height',
				label: 'Paragraph Line Height',
				type: 'slider',
				defaultValue: 1.4,
				min: 1,
				max: 3,
				step: 0.1,
				action: 'css',
				cssSelector: '{elID} p',
				prop: 'lineHeight',
			},
			{
				id: 'typo-white-space',
				label: 'Space Between Paragraphs',
				type: 'slider',
				unit: 'px',
				defaultValue: 24,
				min: 1,
				max: 50,
				action: 'css',
				cssSelector: '{elID} p',
				prop: 'paddingBottom',
			},
		],
		props: [
			'fontFamily',
			'lineHeight',
			'paddingBottom',
		],
		customCSS: [
			'fontFamily',
			'lineHeight',
			'paddingBottom',
		]
	},

	textshadow: {
		mode: false,
		title: 'Text Shadow',
		id: 'text-shadow',
		fields: [
			{
				id: 'text-shadow-enable',
				label: 'Enable Text Shadow',
				type: 'checkbox',
				action: 'shadowtext',
				prop: 'textShadowToggle',
				reveal: 'show-on-reveal'
			},
			{
				id: 'text-shadow-color',
				label: 'Shadow Color',
				type: 'color',
				action: 'shadowtext',
				defaultValue: '#4a4a4a',
				prop: 'textShadowColor',
				classes: 'show-on-reveal chk-hidden',
			},
			{
				id: 'text-shadow-blur',
				label: 'Blur',
				type: 'slider',
				unit: 'px',
				min: 0,
				max: 20,
				action: 'shadowtext',
				prop: 'textShadowBlur',
				classes: 'show-on-reveal chk-hidden',
			},
		],
		props: [
			'textShadowToggle',
			'textShadowColor',
			'textShadowBlur',
		]
	},

	sectionsize: {
		mode: false,
		title: 'Size',
		id: 'section-size',
		fields: [
			{
				id: 'size-mode',
				label: 'Section Type',
				type: 'select',
				options: {
					wsection: 'Fluid',
					section: 'Fixed'
				},
				action: 'sectiontoggle',
				file: 'el',
			},
		],
		files: [
			'el',
		]
	},

	anchor: {
		mode: false,
		title: 'Anchor',
		id: 'anchor-el',
		fields: [
			{
				id: 'anchor-name',
				label: 'Anchor Name',
				type: 'text',
				action: 'setanchor',
				anchor: true,
				file: 'anchor',
				placeholder: 'e.g. first-row',
				desc: 'Accepted characters: Letter, number, dash and underscore. No spaces, and must be unique.'
			},
		],
		files: [
			'el',
		]
	},

	background: {
		mode: false,
		tabs: [{ key: 'color', name: 'Color' }, { key: 'image', name: 'Image' }, { key: 'video', name: 'Video' }],
		defaultTab: 'color',
		title: 'Background',
		id: 'background',
		fields: [
			{
				id: 'color',
				label: 'Color',
				type: 'color',
				action: 'bgcolor',
				prop: 'backgroundColor',
				mode: 'color'
			},
			{
				id: 'color1-pos',
				label: 'Color #1 Spread',
				type: 'slider',
				min: 0,
				max: 100,
				action: 'bgcolor',
				defaultValue: 0,
				file: 'color1pos',
				classes: 'gradient-reveal chk-hidden',
				mode: 'color'
			},
			{
				id: 'grad-color',
				label: '2nd Color',
				type: 'color',
				action: 'bgcolor',
				file: 'gradColor',
				defaultValue: '#000000',
				classes: 'gradient-reveal chk-hidden',
				mode: 'color'
			},
			{
				id: 'color2-pos',
				label: 'Color #2 Spread',
				type: 'slider',
				min: 0,
				max: 100,
				action: 'bgcolor',
				defaultValue: 100,
				file: 'color2pos',
				classes: 'gradient-reveal chk-hidden',
				mode: 'color'
			},
			{
				id: 'color-dir',
				label: 'Direction',
				type: 'slider',
				min: 0,
				max: 360,
				action: 'bgcolor',
				defaultValue: 0,
				file: 'colordir',
				classes: 'gradient-reveal chk-hidden',
				mode: 'color'
			},
			{
				id: 'bg-gradient-enable',
				label: 'Enable Gradient',
				type: 'checkbox',
				action: 'bgcolor',
				prop: 'gradient',
				defaultValue:"no",
				reveal: 'gradient-reveal',
				mode: 'color'
			},
			{
				id: 'bg-image',
				label: 'Image URL',
				placeholder: 'e.g. https://yourdomain.com/image.jpg',
				type: 'upload',
				action: 'backgroundimg',
				prop: 'backgroundImage',
				mode: 'image'
			},
			{
				id: 'bg-pos',
				label: 'Position',
				defaultValue: 'no-repeat',
				type: 'bgpos',
				action: 'backgroundimgprop',
				prop: 'backgroundPosition',
				mode: 'image',
				desc: 'Click and drag the blue ball to set the image position',
				classes: 'parallax-reveal scroll-parallax-reveal'
			},
			{
				id: 'bg-tiling',
				label: 'Tile Mode',
				options: {
					'no-repeat': 'No Tile',
					'repeat' : 'Tile',
					'repeat-x': 'Tile Horizontal',
					'repeat-y': 'Tile Vertical'
				},
				defaultValue: 'no-repeat',
				type: 'select',
				action: 'backgroundimgprop',
				prop: 'backgroundRepeat',
				mode: 'image'
			},
			{
				id: 'bg-paralax',
				label: 'Paralax',
				options: {
					'scroll': 'None',
					'fixed' : 'Fixed'
				},
				defaultValue: 'scroll',
				type: 'select',
				action: 'backgroundimgprop',
				prop: 'backgroundAttachment',
				groupReveal: 'parallax-reveal',
				mode: 'image'
			},
			{
				id: 'bg-size',
				label: 'Size',
				options: {
					'auto': 'Default',
					'cover' : 'Cover',
					'contain': 'Contain'
				},
				defaultValue: 'auto',
				type: 'select',
				action: 'backgroundimgprop',
				prop: 'backgroundSize',
				mode: 'image'
			},
			{
				id: 'bg-video',
				label: 'Enable Background Video',
				type: 'checkbox',
				action: 'togglevideo',
				prop: 'backgroundVideo',
				reveal: 'show-on-reveal',
				mode: 'video'
			},
			{
				id: 'yturl',
				label: 'YouTube Video URL',
				type: 'text',
				placeholder: 'e.g. https://youtu.be/ogXKnXmmQzE',
				action: 'savedata',
				file: 'yturl',
				classes: 'show-on-reveal chk-hidden',
				mode: 'video',
				condition: 'page',
				desc: 'If you enter a YouTube video URL here, then the settings below will be ignored. Note: YouTube video background will be displayed only on the actual page.'
			},
			{
				id: 'mp4video',
				label: 'MP4 Video URL',
				type: 'text',
				placeholder: 'e.g. https://yoursite.com/video.mp4',
				action: 'backgroundvid',
				file: 'mp4',
				classes: 'show-on-reveal chk-hidden',
				mode: 'video'
			},
			{
				id: 'webmvideo',
				label: 'WebM Video URL',
				type: 'text',
				placeholder: 'e.g. https://yoursite.com/video.webm',
				action: 'backgroundvid',
				file: 'webm',
				classes: 'show-on-reveal chk-hidden',
				mode: 'video'
			},
			{
				id: 'oggvideo',
				label: 'OGG/OGV Video URL',
				type: 'text',
				placeholder: 'e.g. https://yoursite.com/video.ogv',
				action: 'backgroundvid',
				file: 'ogg',
				classes: 'show-on-reveal chk-hidden',
				mode: 'video'
			},
		],
		props: [
			'backgroundColor',
			'backgroundImage',
			'backgroundPosition',
			'backgroundRepeat',
			'backgroundAttachment',
			'backgroundSize',
			'backgroundVideo'
		],
		files: [
			'backgroundVideoType',
			'yturl',
			'mp4',
			'ogg',
			'webm'
		]
	},
	backgroundcolor: {
		mode: false,
		title: 'Background Color',
		id: 'background-color',
		fields: [
			{
				id: 'bg-color',
				label: 'Color',
				type: 'color',
				action: 'bgcolor',
				prop: 'backgroundColor',
			},
			{
				id: 'bg-color1-pos',
				label: 'Color #1 Spread',
				type: 'slider',
				min: 0,
				max: 100,
				action: 'bgcolor',
				defaultValue: 0,
				file: 'color1pos',
				classes: 'gradient-reveal chk-hidden',
			},
			{
				id: 'bg-grad-color',
				label: '2nd Color',
				type: 'color',
				action: 'bgcolor',
				file: 'gradColor',
				defaultValue: '#000000',
				classes: 'gradient-reveal chk-hidden',
			},
			{
				id: 'bg-color2-pos',
				label: 'Color #2 Spread',
				type: 'slider',
				min: 0,
				max: 100,
				action: 'bgcolor',
				defaultValue: 100,
				file: 'color2pos',
				classes: 'gradient-reveal chk-hidden',
			},
			{
				id: 'bg-color-dir',
				label: 'Direction',
				type: 'slider',
				min: 0,
				max: 360,
				action: 'bgcolor',
				defaultValue: 0,
				file: 'colordir',
				classes: 'gradient-reveal chk-hidden',
			},
			{
				id: 'bg-color-gradient-enable',
				label: 'Enable Gradient',
				type: 'checkbox',
				action: 'bgcolor',
				prop: 'gradient',
				defaultValue:"no",
				reveal: 'gradient-reveal',
			},
		],
		props: [
			'backgroundColor',
		],
	},
	image: {
		mode: false,
		title: 'Image',
		id: 'image-upload',
		fields: [
			{
				id: 'image-url',
				label: 'Image URL',
				placeholder: 'e.g. https://yourdomain.com/image.jpg',
				type: 'upload',
				action: 'showimage',
				attrib: 'src',
				required: true,
			},
		],
		attribs: [
			'src'
		]
	},
	imagestyle: {
		mode: false,
		title: 'Style',
		id: 'image-styles',
		fields: [
			{
				id: 'image-style',
				label: 'Image Style',
				type: 'select',
				options: {
					none: 'None',
					"img-rounded": 'Rounded Corners',
					"img-circle": 'Circle',
					"img-thumbnail": 'Thumbnail'

				},
				action: 'swapclass',
				classname: 'styleclass' 
			},
		],
		cssclasses: [
			'styleclass',
		],
		styleclasses: ['img-rounded', 'img-circle', 'img-thumbnail']
	},
	imagefilter: {
		mode: false,
		title: 'Filter',
		id: 'image-filters',
		fields: [
			{
				id: 'image-filter',
				label: 'Image Filter',
				type: 'select',
				options: {
					none: 'None',
					"img-greyscale": 'Greyscale',
					"img-sepia": 'Sepia',
					"img-blur": 'Blur',
					"img-saturate": 'Saturate',
					"img-hue": 'Hue',
					"img-invert": 'Invert',
					"img-darken": 'Darken',
					"img-contrast": 'Contrast'
				},
				action: 'swapclass',
				classname: 'styleclass' 
			},
		],
		cssclasses: [
			'styleclass',
		],
		styleclasses: ['img-greyscale', 'img-sepia', 'img-blur', 'img-saturate', 'img-hue', 'img-invert', 'img-darken', 'img-contrast']
	},
	imagecaption: {
		mode: false,
		title: 'Caption',
		id: 'image-captions',
		fields: [
			{
				id: 'image-caption',
				label: 'Text',
				type: 'text',
				defaultValue: '',
				action: 'caption',
				textClass: 'img-caption-txt',
				required: false,
			},
			{
				id: 'image-caption-color',
				label: 'Text Color',
				type: 'color',
				action: 'style',
				prop: 'color',
			},
			{
				id: 'image-caption-background-color',
				label: 'Background Color',
				type: 'color',
				action: 'captionstyle',
				prop: 'backgroundColor',
			},
		],
		texts: [
			'img-caption-txt'
		],
		props: [
			'color',
			'backgroundColor'
		]
	},
	video: {
		mode: false,
		title: 'Video',
		id: 'video-main',
		fields: [
			{
				id: 'video-type',
				label: 'Type',
				type: 'select',
				options: {
					youtube: 'YouTube',
					vimeo: 'Vimeo',
					hosted: 'Hosted',
					embed: 'Embed'
				},
				defaultValue: 'youtube',
				action: 'videotoggle',
				file: 'video-type',
				groupReveal: 'video-reveal',
			},
			{
				id: 'youtube-url',
				label: 'YouTube URL',
				type: 'text',
				isURL: true,
				action: 'video',
				classes: 'youtube-video-reveal video-reveal',
				file: 'video-youtube',
				placeholder: 'e.g. https://youtu.be/ogXKnXmmQzE',
			},
			{
				id: 'vimeo-url',
				label: 'Vimeo URL',
				type: 'text',
				isURL: true,
				action: 'video',
				classes: 'vimeo-video-reveal video-reveal',
				file: 'video-vimeo',
				placeholder: 'e.g. https://vimeo.com/283691187',
			},
			{
				id: 'mp4-url',
				label: 'MP4 URL',
				type: 'text',
				isURL: true,
				action: 'video',
				classes: 'hosted-video-reveal video-reveal',
				file: 'video-hosted-mp4',
				placeholder: 'e.g. https://yoursite.com/video.mp4',
			},
			{
				id: 'webm-url',
				label: 'WebM URL',
				type: 'text',
				isURL: true,
				action: 'video',
				classes: 'hosted-video-reveal video-reveal',
				file: 'video-hosted-webm',
				placeholder: 'e.g. https://yoursite.com/video.webm',
			},
			{
				id: 'ogg-url',
				label: 'OGG URL (optional)',
				type: 'text',
				isURL: true,
				action: 'video',
				classes: 'hosted-video-reveal video-reveal',
				file: 'video-hosted-ogg',
				placeholder: 'e.g. https://yoursite.com/video.ogv',
			},
			{
				id: 'splash-url',
				label: 'Splash Image URL',
				type: 'text',
				isURL: true,
				action: 'video',
				classes: 'hosted-video-reveal video-reveal',
				file: 'video-hosted-splash'
			},
			{
				id: 'video-embed',
				label: 'Embed Code',
				type: 'textarea',
				isURL: true,
				action: 'video',
				classes: 'embed-video-reveal video-reveal',
				file: 'video-embed'
			},
			{
				id: 'video-autoplay',
				label: 'Autoplay Video',
				type: 'checkbox',
				action: 'videotoggle',
				classes: 'youtube-video-reveal vimeo-video-reveal hosted-video-reveal video-reveal',
				file: 'video-autoplay'
			},
			{
				id: 'video-controls',
				label: 'Disable Controls',
				type: 'checkbox',
				action: 'videotoggle',
				classes: 'youtube-video-reveal hosted-video-reveal video-reveal',
				file: 'video-controls',
				negative: true,
			},
		],
		files: [
			'video-type',
			'video-youtube',
			'video-vimeo',
			'video-hosted-mp4',
			'video-hosted-webm',
			'video-hosted-ogg',
			'video-hosted-splash',
			'video-embed',
			'video-autoplay',
			'video-controls'
		],
		defaults: {
			'video-type': 'youtube',
			'video-youtube': '',
			'video-vimeo': '',
			'video-hosted-mp4': '',
			'video-hosted-webm': '',
			'video-hosted-ogg': '',
			'video-hosted-splash': '',
			'video-embed': '',
			'video-autoplay': 0,
			'video-controls': 1
		},
	},
	collapse: {
		mode: false,
		title: 'Title',
		id: 'collapse-set',
		fields: [
			{
				id: 'collapse-title',
				label: 'Title',
				type: 'text',
				defaultValue: 'Collapse Title',
				action: 'changetext',
				textClass: 'ib2-collapse-title',
				required: true,
			},
		],
		texts: [
			'ib2-collapse-title'
		]
	},
	icon: {
		mode: false,
		title: 'Icon',
		id: 'icon-set',
		fields: [
			{
				id: 'icon-shape',
				label: 'Current Icon',
				type: 'icon',
				defaultValue: 'fas fa-star',
				action: 'changeicon',
				classname: 'iconclass',
			},
			{
				id: 'icon-color',
				label: 'Icon Color',
				type: 'color',
				action: 'iconstyle',
				defaultValue: '#FFFFFF',
				prop: 'color',
			},
			{
				id: 'icon-size',
				label: 'Icon Size',
				type: 'slider',
				unit: 'px',
				min: 9,
				max: 72,
				action: 'iconstyle',
				defaultValue: 3,
				prop: 'fontSize',
			},
		],
		props: [
			'color',
			'fontSize',
		]
	},

	iconstyle: {
		mode: false,
		title: 'Icon Style',
		id: 'icon-style',
		fields: [
			{
				id: 'icon-bg-color',
				label: 'Background Color',
				type: 'color',
				action: 'style',
				defaultValue: 'transparent',
				prop: 'backgroundColor',
			},
			{
				id: 'bg-shape',
				label: 'Background Shape',
				type: 'select',
				options: {
					sharp: 'Sharp Corners',
					rounded: 'Rounded Corners',
					circle: 'Circle'
				},
				defaultValue: 'sharp',
				action: 'iconshape',
				file: 'iconshape',
			},
		],
		files: [
			'iconshape'
		],
		props: [
			'backgroundColor',
		]
	},

	buttonicon: {
		mode: false,
		title: 'Icon',
		id: 'btn-icon',
		fields: [
			{
				id: 'shape-icon',
				label: 'Current Icon',
				type: 'icon',
				defaultValue: '',
				action: 'changeicon',
				removeIcon: true,
				classname: 'iconclass',
			},
			{
				id: 'icon-pos',
				label: 'Icon Position',
				type: 'select',
				defaultValue: 'before',
				action: 'changeicon',
				options: {
					before: 'Before Text',
					after: 'After Text'
				},
				file: 'iconpos',
			},
		]
	},

	buttontext: {
		mode: false,
		title: 'Text',
		id: 'button-texts',
		fields: [
			{
				id: 'button-text-label',
				label: 'Text Label',
				type: 'text',
				defaultValue: 'Click Here',
				action: 'changetext',
				textClass: 'ib2-btn-txt',
				required: true,
			},
			{
				id: 'button-text-font',
				label: 'Font Family',
				type: 'fonts',
				options: ib2fonts,
				action: 'style',
				prop: 'fontFamily',
			},
			{
				id: 'button-text-style',
				label: 'Font Style',
				type: 'fontstyle',
				action: 'style',
				prop: 'fontStyle',
			},
			{
				id: 'button-text-size',
				label: 'Font Size',
				type: 'slider',
				unit: 'px',
				min: 10,
				max: 150,
				action: 'style',
				prop: 'fontSize',
			},
			{
				id: 'button-text-spacing',
				label: 'Letter Spacing',
				type: 'slider',
				unit: 'px',
				min: 0,
				max: 30,
				action: 'style',
				prop: 'letterSpacing',
			},
		],
		props: [
			'fontFamily',
			'fontWeight',
			'fontStyle',
			'textDecorationLine',
			'fontSize',
			'letterSpacing'
		],
		texts: [
			'ib2-btn-txt'
		]
	},
	buttonstyle: {
		mode: false,
		title: 'Style',
		id: 'button-styles',
		linkedName: 'buttoncolor',
		fields: [
			{
				id: 'style-mode',
				label: 'Button Style',
				type: 'select',
				defaultValue: 'flat',
				action: 'buttonstyle',
				options: {
					flat: 'Flat',
					glossy: 'Glossy'
				},
				file: 'button-type',
			},
		],
		files: [
			'button-type'
		]
	},

	buttoncolor: {
		mode: false,
		tabs: [{ key: 'normal', name: 'Normal' }, { key: 'hover', name: 'Hover' }],
		defaultTab: 'normal',
		title: 'Colors',
		id: 'button-colors',
		linkedName: 'buttonstyle',
		fields: [
			{
				id: 'button-background-color',
				label: 'Background Color',
				type: 'color',
				action: 'buttoncolor',
				defaultValue: '#428bca',
				prop: 'backgroundColor',
				mode: 'normal',
			},
			{
				id: 'button-text-color',
				label: 'Text Color',
				type: 'color',
				action: 'style',
				defaultValue: '#FFFFFF',
				prop: 'color',
				mode: 'normal',
			},
			{
				id: 'button-shadow-color',
				label: 'Text Shadow Color',
				type: 'color',
				action: 'textshadow',
				defaultValue: '#4a4a4a',
				prop: 'textShadow',
				mode: 'normal',
			},
			{
				id: 'button-background-hover-color',
				label: 'Background Color',
				type: 'color',
				action: 'cssvar',
				cssSelector: 'self',
				defaultValue: '#3071a9',
				prop: 'backgroundColor@hover',
				mode: 'hover',
			},
			{
				id: 'button-text-hover-color',
				label: 'Text Color',
				type: 'color',
				action: 'cssvar',
				cssSelector: 'self',
				defaultValue: '#FFFFFF',
				prop: 'color@hover',
				mode: 'hover',
			},
			{
				id: 'button-shadow-hover-color',
				label: 'Text Shadow Color',
				type: 'color',
				action: 'cssvar',
				defaultValue: '#4a4a4a',
				cssSelector: 'self',
				prop: 'textShadow@hover',
				mode: 'hover',
			},
		],
		props: [
			'backgroundColor',
			'backgroundColor@hover',
			'color',
			'color@hover',
			'textShadow',
			'textShadow@hover'
		]
	},
	
	buttonaction: {
		mode: false,
		title: 'Action',
		id: 'click-actions',
		fields: [
			{
				id: 'button-action',
				label: 'Action',
				type: 'select',
				action: 'buttonaction',
				options: {
					none: 'Do Nothing',
					url: 'Go to URL',
					popup: 'Open a Pop Up',
				},
				file: 'button-action',
				groupReveal: 'button-action-reveal',
			},
			{
				id: 'button-url',
				label: 'Destination URL',
				type: 'text',
				action: 'attribute',
				attrib: 'href',
				placeholder: 'e.g. http://yoursite.com/anypage',
				isURL: true,
				desc: 'Enter a full url or an anchor name (e.g. #anchorname)',
				classes: 'url-button-action-reveal button-action-reveal',
			},
			{
				id: 'button-url-target',
				label: 'Open in new window',
				type: 'checkbox',
				action: 'attribute',
				attrib: 'target',
				classes: 'url-button-action-reveal button-action-reveal',
			},
			{
				id: 'button-popup-edit',
				label: 'Pop Up',
				type: 'select',
				action: 'buttonaction',
				options: 'popupdata',
				defaultValue: 'none',
				file: 'popup',
				classes: 'popup-button-action-reveal button-action-reveal',
			},
		],
		attribs: [
			'href',
			'target'
		],
		files: [
			'button-action',
			'popup'
		]
	},

	formstyle: {
		mode: false,
		title: 'Form Style',
		id: 'form-styles',
		fields: [
			{
				id: 'form-style',
				label: 'Form Style',
				type: 'select',
				options: {
					"vertical": 'Full Stack',
					"semi-horizontal": 'Half Stack',
					"horizontal": 'Inline',
					
				},
				action: 'formstyle',
				defaultValue: 'vertical',
				file: 'form-mode',
			},
			{
				id: 'form-field-style',
				label: 'Field Style',
				type: 'select',
				action: 'fieldstyle',
				options: {
					'field-normal': 'Style #1',
					'field-normal-thick': 'Style #2',
					'field-sharp': 'Style #3',
					'field-sharp-thick': 'Style #4'
				},
				classname: 'fieldstyleclass',
			},
			{
				id: 'form-field-size',
				label: 'Field Size',
				type: 'select',
				action: 'fieldsize',
				options: {
					'input-sm': 'Small',
					'input-md': 'Normal',
					'input-lg': 'Big'
				},
				classname: 'fieldclass',
			},
			{
				id: 'field-background-color',
				label: 'Field Background Color',
				type: 'color',
				action: 'style',
				defaultValue: '#FFFFFF',
				prop: 'backgroundColor',
			},
			{
				id: 'field-border-color',
				label: 'Field Border Color',
				type: 'color',
				action: 'style',
				defaultValue: '#cccccc',
				prop: 'borderColor',
			},
		],
		props: [
			'backgroundColor',
			'borderColor'
		],
		files: [
			'form-mode'
		],
		cssclasses: [
			'fieldclass',
			'fieldstyleclass'
		],
		fieldclasses: ['input-sm', 'input-md', 'input-lg'],
		fieldstyleclasses: ['field-normal', 'field-normal-thick', 'field-sharp', 'field-sharp-think']
	},

	formbutton: {
		mode: false,
		tabs: [{ key: 'type', name: 'Type' }, { key: 'text', name: 'Text' }, { key: 'color', name: 'Color' }],
		defaultTab: 'type',
		title: 'Button Style',
		id: 'submit-texts',
		fields: [
			{
				id: 'submit-type',
				label: 'Button Type',
				type: 'select',
				action: 'submitbutton',
				options: {
					css: 'CSS Button',
					image: 'Image',
				},
				typename: 'button-type',
				groupReveal: 'submit-image-reveal',
				mode: 'type'
			},
			{
				id: 'submit-image',
				label: 'Button Image URL',
				placeholder: 'e.g. https://yourdomain.com/button.jpg',
				type: 'upload',
				action: 'submitimage',
				attrib: 'src',
				mode: 'type',
				classes: 'image-submit-image-reveal submit-image-reveal'
			},
			{
				id: 'submit-text-note',
				label: '',
				type: 'desc',
				desc: 'These settings only applies to CSS Button',
				mode: 'text'
			},
			{
				id: 'submit-text-label',
				label: 'Text Label',
				type: 'text',
				defaultValue: 'Click Here',
				action: 'changetext',
				textClass: 'ib2-btn-txt',
				required: true,
				mode: 'text'
			},
			{
				id: 'submit-text-font',
				label: 'Font Family',
				type: 'fonts',
				options: ib2fonts,
				action: 'style',
				prop: 'fontFamily',
				mode: 'text'
			},
			{
				id: 'submit-text-style',
				label: 'Font Style',
				type: 'fontstyle',
				action: 'style',
				prop: 'fontStyle',
				mode: 'text'
			},
			{
				id: 'submit-text-size',
				label: 'Font Size',
				type: 'slider',
				unit: 'px',
				min: 10,
				max: 150,
				action: 'style',
				prop: 'fontSize',
				mode: 'text'
			},
			{
				id: 'submit-text-spacing',
				label: 'Letter Spacing',
				type: 'slider',
				unit: 'px',
				min: 0,
				max: 30,
				action: 'style',
				prop: 'letterSpacing',
				mode: 'text'
			},
			{
				id: 'submit-text-note',
				label: '',
				type: 'desc',
				desc: 'These settings only applies to CSS Button',
				mode: 'color'
			},
			{
				id: 'submit-color-style',
				label: 'Button Color Style',
				type: 'select',
				defaultValue: 'flat',
				action: 'formbuttonstyle',
				options: {
					flat: 'Flat',
					glossy: 'Glossy'
				},
				file: 'submit-color-type',
				mode: 'color',
			},
			{
				id: 'submit-background-color',
				label: 'Background Color',
				type: 'color',
				action: 'buttoncolor',
				defaultValue: '#428bca',
				prop: 'backgroundColor',
				mode: 'color',
			},
			{
				id: 'submit-text-color',
				label: 'Text Color',
				type: 'color',
				action: 'style',
				defaultValue: '#FFFFFF',
				prop: 'color',
				mode: 'color',
			},
			{
				id: 'submit-shadow-color',
				label: 'Text Shadow Color',
				type: 'color',
				action: 'textshadow',
				defaultValue: '#4a4a4a',
				prop: 'textShadow',
				mode: 'color',
			},
			{
				id: 'submit-background-hover-color',
				label: 'Background Color (Hover)',
				type: 'color',
				action: 'cssvar',
				cssSelector: 'self',
				defaultValue: '#3071a9',
				prop: 'backgroundColor@hover',
				mode: 'color',
			},
			{
				id: 'submit-text-hover-color',
				label: 'Text Color (Hover)',
				type: 'color',
				action: 'cssvar',
				cssSelector: 'self',
				defaultValue: '#FFFFFF',
				prop: 'color@hover',
				mode: 'color',
			},
			{
				id: 'submit-shadow-hover-color',
				label: 'Text Shadow Color (Hover)',
				type: 'color',
				action: 'cssvar',
				defaultValue: '#4a4a4a',
				cssSelector: 'self',
				prop: 'textShadow@hover',
				mode: 'color',
			},
		],
		props: [
			'fontFamily',
			'fontWeight',
			'fontStyle',
			'textDecorationLine',
			'fontSize',
			'letterSpacing',
			'backgroundColor',
			'backgroundColor@hover',
			'color',
			'color@hover',
			'textShadow',
			'textShadow@hover'
		],
		attribs: [
			'src'
		],
		texts: [
			'ib2-btn-txt'
		],
		types: [
			'button-type'
		],
		files: [
			'submit-color-type'
		],
		typeSettings: {
			'button-type': {
				mapType: {
					'ib2-form-submit': 'css',
					'ib2-form-image': 'image'
				},
				checkType: 'class',
			}
		}
	},

	formdata: {
		mode: false,
		title: 'ESP/Autoresponder Integration',
		id: 'form-data',
		fields: [
			{
				id: 'esp-integration',
				label: 'Select Provider',
				type: 'esp',
				defaultValue: 'html',
				action: 'switchesp',
				options: {
					ac: 'ActiveCampaign',
					aw: 'Aweber',
					//cm: 'CampaignMonitor',
					//ck: 'ConvertKit',
					//drp: 'Drip',
					gr: 'GetResponse',
					mc: 'MailChimp',
					mp: 'MailPoet',
					html: 'HTML Code',
				},
				file: 'esp-type',
				groupReveal: 'esp-reveal',
			},
			{
				id: 'esp-list',
				label: 'Select List/Campaign/Sequence',
				type: 'list',
				defaultValue: 'none',
				action: 'switchlist',
				file: 'esp-list',
				classes: 'esp-reveal ac-esp-reveal aw-esp-reveal cm-esp-reveal ac-esp-reveal ck-esp-reveal drp-esp-reveal gr-esp-reveal mc-esp-reveal mp-esp-reveal'
			},
			{
				id: 'submit-parse',
				label: 'Form HTML Code',
				type: 'htmlform',
				action: 'savedata',
				isURL: true,
				rows: 5,
				desc: 'Enter the raw html version of the form that you obtain from your provider/autoresponder, and click the "Parse HTML" above.',
				file: 'rawhtml',
				classes: 'esp-reveal html-esp-reveal'
			},
			{
				id: 'confirm-required',
				label: 'Require Email Confirmation',
				type: 'checkbox',
				defaultValue: 'no',
				action: 'optconfirm',
				file: 'double-optin',
				classes: 'esp-reveal mp-esp-reveal'
			},
		],
		files: [
			'esp-type',
			'esp-list',
			'rawhtml'
		]
	},

	webinar: {
		mode: false,
		title: 'Webinar Integration',
		id: 'webinar-data',
		fields: [
			{
				id: 'webinar-key',
				label: 'Webinar',
				type: 'select',
				action: 'webinar',
				options: ib2webinars,
				attrib: '_webinar_key',
			},
			// {
			// 	id: 'webinar-redirect',
			// 	label: 'Thank You Page URL',
			// 	type: 'text',
			// 	action: 'webinar',
			// 	isURL: true,
			// 	placeholder: 'e.g. https://mysite.com/thanks',
			// 	attrib: '_webinar_redirect'
			// }
		],
		attribs: [
			'_webinar_key',
			'_webinar_redirect'
		]
	},

	formredirect: {
		mode: false,
		title: 'Form Redirect',
		id: 'form-redirect',
		fields: [
			{
				id: 'form-redirect-url',
				label: 'Form Redirect URL',
				type: 'text',
				action: 'formredir',
				defaultValue: '',
				isURL: true,
				placeholder: 'e.g. https://mysite.com/thanks',
				attrib: '_webinar_redirect',
				desc: '<strong>Note:</strong> This setting will be ignored if you\'re using the \'HTML Code\' integration without a webinar integration'
			}
		],
		attribs: [
			'_webinar_redirect'
		]
	},

	formgdpr: {
		mode: false,
		title: 'GDPR',
		id: 'form-gdpr',
		fields: [
			{
				id: 'gdpr-enable',
				label: 'Enable GDPR Field',
				type: 'checkbox',
				action: 'gdpr',
				file: 'gdpr',
				reveal: 'show-on-reveal',
			},
			{
				id: 'gdpr-text',
				label: 'GDPR Consent Text',
				type: 'textarea',
				action: 'gdpr',
				defaultValue: 'Yes, I agree to receive transactional and marketing emails from [YOUR BUSINESS].',
				required: true,
				textClass: 'eu-text',
				classes: 'show-on-reveal chk-hidden',
			},
			{
				id: 'gdpr-text-color',
				label: 'Text Color',
				type: 'color',
				action: 'gdpr',
				prop: 'color',
				classes: 'show-on-reveal chk-hidden',
			},
		],
		files: [
			'gdpr'
		],
		texts: [
			'eu-text'
		],
		props: [
			'color'
		]
	},

	facebook: {
		mode: false,
		title: 'Facebook Opt-In',
		id: 'facebook-optin',
		tabs: [{ key: 'setup', name: 'Setup' }, { key: 'style', name: 'Text Style' }],
		defaultTab: 'setup',
		fields: [
			{
				id: 'fb-enable',
				label: 'Enable Facebook Opt-In',
				type: 'checkbox',
				action: 'fbopt',
				file: 'facebook-enable',
				reveal: 'show-on-reveal',
				mode: 'setup'
			},
			{
				id: 'fb-label',
				label: 'FB Button Label',
				type: 'text',
				action: 'fbopt',
				placeholder: 'e.g. Subscribe With Facebook',
				file: 'facebook-label',
				classes: 'show-on-reveal chk-hidden',
				mode: 'setup'
			},
			{
				id: 'fb-cta',
				label: 'FB CTA Text',
				type: 'text',
				action: 'fbopt',
				placeholder: 'e.g. Subscribe With Facebook',
				file: 'facebook-text',
				classes: 'show-on-reveal chk-hidden',
				mode: 'setup'
			},
			{
				id: 'fb-only',
				label: 'Show Facebook Opt-In Only',
				type: 'checkbox',
				action: 'fbopt',
				file: 'facebook-only',
				classes: 'show-on-reveal chk-hidden',
				mode: 'setup'
			},
			{
				id: 'fb-text-font',
				label: 'Font Family',
				type: 'fonts',
				options: ib2fonts,
				action: 'style',
				prop: 'fontFamily',
				classes: 'show-on-reveal chk-hidden',
				mode: 'style'
			},
			{
				id: 'fb-text-color',
				label: 'Color',
				type: 'color',
				action: 'style',
				prop: 'color',
				classes: 'show-on-reveal chk-hidden',
				mode: 'style'
			},
			{
				id: 'fb-text-size',
				label: 'Font Size',
				type: 'slider',
				min: 12,
				max: 72,
				unit: 'px',
				action: 'style',
				prop: 'fontSize',
				classes: 'show-on-reveal chk-hidden',
				mode: 'style'
			},
		],
		files: [
			'facebook-enable',
			'facebook-label',
			'facebook-text',
			'facebook-only',
		],
		props: [
			'fontFamily',
			'color',
			'fontSize',
		]
	},

	countdown: {
		mode: false,
		title: 'Date and Time',
		id: 'countdown-settings',
		fields: [
			{
				id: 'countdown-mode',
				label: 'Mode',
				type: 'select',
				action: 'countdowndate',
				options: {
					date: 'Exact Date',
					evergreen: 'Evergreen',
					cookie: 'Cookie-Based'
				},
				file: 'mode',
				groupReveal: 'count-mode-reveal',
			},
			{
				id: 'countdown-date',
				label: 'Date',
				type: 'date',
				action: 'countdowndate',
				defaultValue: '2020-01-01',
				file: 'exact-date',
				classes: 'date-count-mode-reveal count-mode-reveal',
			},
			{
				id: 'countdown-day',
				label: 'Day',
				type: 'slider',
				min: 0,
				max: 365,
				action: 'countdowndate',
				defaultValue: '0',
				file: 'time-day',
				classes: 'evergreen-count-mode-reveal cookie-count-mode-reveal count-mode-reveal',
			},
			{
				id: 'countdown-hour',
				label: 'Hour',
				type: 'slider',
				min: 0,
				max: 23,
				action: 'countdowndate',
				defaultValue: '0',
				file: 'time-hour'
			},
			{
				id: 'countdown-min',
				label: 'Minute',
				type: 'slider',
				min: 0,
				max: 59,
				action: 'countdowndate',
				defaultValue: '0',
				file: 'time-min'
			},
			{
				id: 'countdown-tz',
				label: 'Timezone',
				type: 'timezone',
				action: 'countdowndate',
				options: ib2timezone,
				defaultValue: 'America/New_York',
				file: 'tz',
				classes: 'date-count-mode-reveal count-mode-reveal',
			},
		],
		files: [
			'mode',
			'exact-date',
			'time-day',
			'time-hour',
			'time-min',
			'tz'
		]
	},

	countdownstyle: {
		mode: false,
		title: 'Style',
		id: 'countdown-styles',
		fields: [
			{
				id: 'countdown-style',
				label: 'Style',
				type: 'select',
				action: 'countdownstyle',
				options: {
					'text': 'Normal Text',
					'fancy-text': 'Fancy Text',
					'flat-box': 'Flat Box',
					'glossy-box': 'Glossy Box',
					'circular': 'Circular'
				},
				file: 'style',
				groupReveal: 'count-style-reveal',
			},
			{
				id: 'countdown-background-color',
				label: 'Background Color',
				type: 'color',
				action: 'css',
				cssSelector: '{elID} > .ib2-digit',
				defaultValue: '#428bca',
				prop: 'backgroundColor',
				classes: 'count-style-reveal flat-box-count-style-reveal glossy-box-count-style-reveal'
			},
			{
				id: 'countdown-border',
				label: 'Border',
				type: 'border',
				action: 'css',
				cssSelector: '{elID} > .ib2-digit',
				prop: 'border',
				valueMap: ['border-width', 'border-style', 'border-color'],
				mapType: 'single-field',
				classes: 'count-style-reveal flat-box-count-style-reveal glossy-box-count-style-reveal'
			},
			{
				id: 'countdown-text-font',
				label: 'Font Family',
				type: 'fonts',
				options: ib2fonts,
				action: 'style',
				prop: 'fontFamily',
			},
			{
				id: 'countdown-text-size',
				label: 'Font Size',
				type: 'slider',
				min: 10,
				max: 100,
				unit: 'px',
				action: 'style',
				prop: 'fontSize'
			},
			{
				id: 'countdown-text-color',
				label: 'Text Color',
				type: 'color',
				action: 'style',
				defaultValue: '#FFFFFF',
				prop: 'color',
			},
			{
				id: 'countdown-shadow-color',
				label: 'Text Shadow Color',
				type: 'color',
				action: 'textshadow',
				defaultValue: '#4a4a4a',
				prop: 'textShadow',
			},
		],
		props: [
			'backgroundColor',
			'border',
			'fontFamily',
			'fontSize',
			'color',
			'textShadow'
		],
		files: [
			'style'
		],
		customCSS: [
			'backgroundColor',
			'border'
		]
	},

	countdowntexts: {
		mode: false,
		title: 'Texts',
		id: 'countdown-texts',
		fields: [
			{
				id: 'submit-text-note',
				label: '',
				type: 'desc',
				desc: 'These settings only applies to "Normal Text" countdown.',
				mode: 'text'
			},
			{
				id: 'countdown-before-text',
				label: 'Before Text',
				type: 'text',
				defaultValue: '',
				action: 'changetext',
				textClass: 'c-before-text',
				required: false,
				classes: 'count-style-reveal text-count-style-reveal'
			},
			{
				id: 'countdown-after-text',
				label: 'After Text',
				type: 'text',
				defaultValue: '',
				action: 'changetext',
				textClass: 'c-after-text',
				required: false,
				classes: 'count-style-reveal text-count-style-reveal'
			},
		],
		texts: [
			'c-before-text',
			'c-after-text',
		]
	},

	expiryaction: {
		mode: false,
		title: 'Expiry Action',
		id: 'expiry-action',
		fields: [
			{
				id: 'expire-action',
				label: 'Action',
				type: 'select',
				action: 'countexpire',
				options: {
					'none': 'Do Nothing',
					'hide': 'Hide Countdown',
					'redirect': 'Redirect to URL',
					'reveal': 'Reveal Hidden Content',
				},
				file: 'end',
				groupReveal: 'expiry-reveal',
			},
			{
				id: 'expire-url',
				label: 'Destination URL',
				type: 'text',
				action: 'savedata',
				placeholder: 'e.g. https://yoursite.com/anypage',
				file: 'url',
				classes: 'expiry-reveal redirect-expiry-reveal',
			},
		],
		files: [
			'end',
			'url'
		]
	},

	social: {
		mode: false,
		title: 'Sharing Sites',
		id: 'social-share',
		fields: [
			{
				id: 'facebook-share',
				label: 'Facebook Share',
				type: 'checkbox',
				action: 'socialshare',
				file: 'share-facebook',
			},
			{
				id: 'twitter-share',
				label: 'Twitter Share',
				type: 'checkbox',
				action: 'socialshare',
				file: 'share-twitter',
			},
			{
				id: 'linkedin-share',
				label: 'LinkedIn Share',
				type: 'checkbox',
				action: 'socialshare',
				file: 'share-linkedin',
			},
			{
				id: 'google-share',
				label: 'Google Share',
				type: 'checkbox',
				action: 'socialshare',
				file: 'share-google',
			},
		],
		files: [
			'share-facebook',
			'share-twitter',
			'share-linkedin',
			'share-google',
		]
	},

	socialurl: {
		mode: false,
		title: 'Custom URL',
		id: 'social-url',
		fields: [
			{
				id: 'custom-share-url',
				label: 'URL to Share',
				type: 'text',
				action: 'savedata',
				file: 'custom-url',
				placeholder: 'e.g. https://mysite.com/page',
				desc: "If you want to share a URL other than this page, then you can enter the URL here. Otherwise, leave this empty.",
			},
		],
		files: [
			'custom-url'
		]
	},

	menutype: {
		mode: false,
		title: 'Menu Setup',
		id: 'menu-type',
		fields: [
			{
				id: 'menu-setup',
				label: 'Choose Menu',
				type: 'select',
				action: 'loadmenu',
				options: ib2navs,
				file: 'menu',
			},
		],
		files: [
			'menu'
		]
	},

	menustyle: {
		mode: false,
		tabs: [{ key: 'main', name: 'Main-Item' }, { key: 'subitem', name: 'Sub-Item' }],
		defaultTab: 'main',
		title: 'Menu Style',
		id: 'menu-styles',
		fields: [
			{
				id: 'menu-style',
				label: 'Style',
				type: 'select',
				action: 'menustyle',
				options: {
					'plain': 'Plain',
					'plain-pipe': 'Plain (/w Pipe)',
					'flat-box': 'Flat Box',
					'glossy': 'Glossy Box',
				},
				file: 'style',
				groupReveal: 'menu-style-reveal',
				mode: 'main'
			},
			{
				id: 'nav-text-color',
				label: 'Link Color (Normal)',
				type: 'color',
				action: 'cssvar',
				cssSelector: '> li > a',
				defaultValue: '#428bca',
				prop: 'color',
				mode: 'main'
			},
			{
				id: 'nav-hover-color',
				label: 'Link Color (Hover)',
				type: 'color',
				action: 'cssvar',
				cssSelector: '> li > a',
				defaultValue: '#2a6496',
				prop: 'color@hover',
				mode: 'main'
			},
			{
				id: 'nav-bg-color',
				label: 'Background Color (Normal)',
				type: 'color',
				action: 'cssvar',
				cssSelector: 'self',
				defaultValue: '#e5e5e5',
				prop: 'backgroundColor',
				mode: 'main',
				classes: 'menu-style-reveal flat-box-menu-style-reveal glossy-menu-style-reveal'
			},
			{
				id: 'nav-bg-hover-color',
				label: 'Background Color (Hover)',
				type: 'color',
				action: 'cssvar',
				cssSelector: '> li',
				defaultValue: '#cccccc',
				prop: 'backgroundColor@hover',
				mode: 'main',
				classes: 'menu-style-reveal flat-box-menu-style-reveal glossy-menu-style-reveal'
			},
			{
				id: 'subnav-text-color',
				label: 'Link Color (Normal)',
				type: 'color',
				action: 'cssvar',
				cssSelector: 'ul li a',
				defaultValue: '#428bca',
				prop: 'sub-color',
				mode: 'subitem'
			},
			{
				id: 'subnav-hover-color',
				label: 'Link Color (Hover)',
				type: 'color',
				action: 'cssvar',
				cssSelector: 'ul li a',
				defaultValue: '#2a6496',
				prop: 'sub-color@hover',
				mode: 'subitem'
			},
			{
				id: 'subnav-bg-color',
				label: 'Background Color (Normal)',
				type: 'color',
				action: 'cssvar',
				cssSelector: 'ul',
				defaultValue: '#e5e5e5',
				prop: 'sub-backgroundColor',
				mode: 'subitem',
				classes: 'menu-style-reveal flat-box-menu-style-reveal glossy-menu-style-reveal'
			},
			{
				id: 'subnav-bg-hover-color',
				label: 'Background Color (Hover)',
				type: 'color',
				action: 'cssvar',
				cssSelector: 'ul li',
				defaultValue: '#cccccc',
				prop: 'sub-backgroundColor@hover',
				mode: 'subitem',
				classes: 'menu-style-reveal flat-box-menu-style-reveal glossy-menu-style-reveal'
			},
		],
		files: [
			'style'
		],
		props: [
			'color',
			'color@hover',
			'backgroundColor',
			'backgroundColor@hover',
			'sub-color',
			'sub-color@hover',
			'sub-backgroundColor',
			'sub-backgroundColor@hover'
		],
		customCSS: [
			'color',
			'backgroundColor',
			'sub-color',
			'sub-backgroundColor',
		]
	},
	line: {
		mode: false,
		title: 'Style',
		id: 'line-style',
		fields: [
			{
				id: 'line-border',
				label: 'Line Style',
				type: 'border',
				action: 'border',
				prop: 'borderTop',
				valueMap: ['border-width', 'border-style', 'border-color'],
				mapType: 'single-field'
			},
		],
		props: [
			'borderTop'
		]
	},

	comment: {
		mode: false,
		title: 'Setup',
		id: 'comment-setup',
		fields: [
			{
				id: 'comment-provider',
				label: 'Comment Provider',
				type: 'select',
				action: 'comment',
				options: {
					'facebook': 'Facebook',
					'disqus': 'Disqus',
				},
				file: 'comment',
			},
		],
		files: [
			'comment'
		]
	},

	shortcode: {
		mode: false,
		title: 'Shortcode',
		id: 'shortcode-text',
		fields: [
			{
				id: 'shortcode-content',
				label: 'Shortcode Content',
				type: 'textarea',
				action: 'shortcode',
				file: 'shortcode',
				desc: 'Enter your shortcode here. You can enter more than one shortcode.'
			},
		],
		files: [
			'shortcode'
		]
	},

	html: {
		mode: false,
		title: 'HTML',
		id: 'html-text',
		fields: [
			{
				id: 'html-content',
				label: 'HTML Code',
				type: 'textarea',
				action: 'html',
				file: 'code',
				desc: 'Enter your html code here. <strong>Important:</strong> Only HTML and css code allowed. Do NOT enter javascript snippet here.'
			},
		],
		files: [
			'code'
		]
	},

	slides: {
		mode: false,
		title: 'Carousel',
		id: 'image-carousel',
		fields: [
			{
				id: 'carousel-fields',
				label: '',
				type: 'dynamic',
				action: 'carousel',
				dynamic: [
					{
						label: 'Image URL #{n}',
						placeholder: 'e.g. https://yourdomain.com/image{n}.jpg',
						type: 'upload',
						dynamic: 'imageurl',
						dynkey: 'slides',
					},
					{
						label: 'Title #{n}',
						type: 'text',
						dynamic: 'title',
						dynkey: 'slides',
					},
					{
						label: 'URL #{n} (optional)',
						type: 'text',
						isURL: true,
						dynamic: 'desturl',
						dynkey: 'slides',
					},
				]
			}
		],
		dynamics: [
			'slides',
		]
	},

	questions: {
		mode: false,
		title: 'Question Setup',
		id: 'tabbed-content',
		fields: [
			{
				id: 'question-num',
				label: '# of Questions',
				type: 'slider',
				min: 1,
				max: 5,
				action: 'question',
				file: 'questions',
			},
		],
		files: [
			'questions'
		]
	},

	tabs: {
		mode: false,
		title: 'Tabs',
		id: 'tabbed-content',
		fields: [
			{
				id: 'tab-title',
				label: '',
				type: 'dynamic',
				action: 'tab',
				dynamic: [
					{
						label: 'Tab Title #{n}',
						type: 'text',
						dynamic: 'title',
						dynkey: 'tabs',
					},
				]
			}
		],
		dynamics: [
			'tabs',
		]
	},

	tabstyle: {
		mode: false,
		title: 'Style',
		id: 'tabs-style',
		fields: [
			{
				id: 'tab-link-color',
				label: 'Tab Head Text Color',
				type: 'color',
				action: 'css',
				cssSelector: '{elID} > .ib2-nav-tabs > li > a',
				defaultValue: '#555555',
				prop: 'color',
				important: true,
			},
			{
				id: 'tab-active-background',
				label: 'Tab Head Background Color (active)',
				type: 'color',
				action: 'css',
				cssSelector: '{elID} > .ib2-nav-tabs > li.ib2-tabs-active > a, {elID} > .ib2-nav-tabs > li.ib2-tabs-active > a:hover',
				defaultValue: '#FFFFFF',
				prop: 'backgroundColor@active',
			},
			{
				id: 'tab-hover-background',
				label: 'Tab Head Background Color (hover)',
				type: 'color',
				action: 'css',
				cssSelector: '{elID} > .ib2-nav-tabs > li > a:hover',
				defaultValue: '#e6e6e6',
				prop: 'backgroundColor@hover',
			},
			{
				id: 'panel-background',
				label: 'Panel Background Color',
				type: 'color',
				action: 'css',
				cssSelector: '{elID} > .ib2-tab-content',
				defaultValue: '#ffffff',
				prop: 'backgroundColor',
			},
			{
				id: 'panel-border',
				label: 'Border Color',
				type: 'color',
				action: 'css',
				cssSelector: '{elID} > .ib2-tab-content, {elID} > .ib2-nav-tabs, {elID} .ib2-nav-tabs > li.ib2-tabs-active > a',
				defaultValue: '#dddddd',
				prop: 'borderColor',
			},
		],
		props: [
			'color',
			'backgroundColor@active',
			'backgroundColor@hover',
			'backgroundColor',
			'borderColor'
		],
		customCSS: [
			'color',
			'backgroundColor',
			'borderColor'
		]
	},

	dateformat: {
		mode: false,
		title: 'Setup',
		id: 'date-setup',
		fields: [
			{
				id: 'date-format',
				label: 'Format',
				type: 'select',
				action: 'datesetup',
				options: ib2dates,
				file: 'format',
			},
			{
				id: 'date-tz',
				label: 'Timezone',
				type: 'timezone',
				action: 'datesetup',
				options: ib2timezone,
				defaultValue: 'America/New_York',
				file: 'tz',
			},
		],
		files: [
			'format',
			'tz'
		]
	},

	map: {
		mode: false,
		title: 'Setup',
		id: 'map-setup',
		fields: [
			{
				id: 'map-address',
				label: 'Address',
				type: 'text',
				action: 'map',
				file: 'address',
				defaultValue: '',
			},
			{
				id: 'map-type',
				label: 'Map Type',
				type: 'select',
				action: 'map',
				options: {
					'q': 'Roadmap',
					'w': 'Satellite'
				},
				defaultValue: 'q',
				file: 'maptype',
			},
			{
				id: 'map-zoom',
				label: 'Zoom',
				type: 'slider',
				min: 1,
				max: 20,
				action: 'map',
				defaultValue: 1,
				file: 'mapzoom',
			},
		],
		files: [
			'address',
			'maptype',
			'mapzoom'
		]
	},

	datestyle: {
		mode: false,
		title: 'Style',
		id: 'date-style',
		fields: [
			{
				id: 'date-font',
				label: 'Font Family',
				type: 'fonts',
				options: ib2fonts,
				action: 'style',
				prop: 'fontFamily',
			},
			{
				id: 'date-color',
				label: 'Color',
				type: 'color',
				action: 'style',
				prop: 'color',
			},
			{
				id: 'date-size',
				label: 'Font Size',
				type: 'slider',
				unit: 'px',
				min: 10,
				max: 150,
				action: 'style',
				prop: 'fontSize',
			},
		],
		files: [
			'format',
			'tz',
		],
		props: [
			'fontFamily',
			'color',
			'fontSize'
		]
	},

	progress: {
		mode: false,
		title: 'Style',
		id: 'progress-bar',
		fields: [
			{
				id: 'progress-color',
				label: 'Color',
				type: 'color',
				action: 'style',
				prop: 'backgroundColor',
			},
			{
				id: 'progress-size',
				label: 'Bar Width (in percent)',
				type: 'slider',
				unit: '%',
				defaultValue: 50,
				min: 0,
				max: 100,
				action: 'style',
				prop: 'width',
			},
			{
				id: 'progress-text',
				label: 'Text',
				type: 'text',
				defaultValue: '50%',
				action: 'progress',
				textClass: 'progress-bar',
			},
			{
				id: 'progress-striped',
				label: 'Add Stripes',
				type: 'checkbox',
				action: 'progress',
				classname: 'progress-bar-striped',
				reveal: 'show-on-reveal',
			},
			{
				id: 'progress-animate',
				label: 'Animate Stripes',
				type: 'checkbox',
				action: 'progress',
				classname: 'active',
				classes: 'show-on-reveal chk-hidden',
			},
		],
		cssclasses: [
			'progress-bar-striped',
			'active',
		],
		props: [
			'backgroundColor',
			'width'
		],
		texts: [
			'progress-bar'
		]
	},

	separator: {
		mode: false,
		tabs: [{ key: 'topsep', name: 'Top' }, { key: 'botsep', name: 'Bottom' }],
		defaultTab: 'topsep',
		title: 'Separator Shape',
		id: 'separator-style',
		fields: [
			{
				id: 'top-separator',
				label: 'Shape',
				options: {
					'normal': 'Normal',
					'triangle' : 'Triangle',
					'small-triangle' : 'Small Triangle',
					'big-triangle' : 'Big Triangle',
					'triangle-asym' : 'Triangle Asymmetrical',
					'slanted' : 'Slanted',
					'slanted-opac' : 'Slanted Opacity',
					'curve' : 'Curve',
					'wavy' : 'Wavy',
					'waves' : 'Waves',
					'waves-opac' : 'Waves Opacity',
					'spikes' : 'Spikes',
				},
				defaultValue: 'normal',
				type: 'select',
				action: 'separator',
				prop: 'top-separator',
				mode: 'topsep',
			},
			{
				id: 'top-separator-color',
				label: 'Fill Color',
				type: 'color',
				action: 'sepcolor',
				defaultValue: '#FFFFFF',
				prop: 'top-color',
				mode: 'topsep',
			},
			{
				id: 'top-sep-height',
				label: 'Height',
				type: 'slider',
				unit: 'px',
				defaultValue: 100,
				min: 50,
				max: 500,
				action: 'sepheight',
				prop: 'top-height',
				mode: 'topsep',
			},
			{
				id: 'top-sep-flip',
				label: 'Flip',
				type: 'checkbox',
				action: 'sepflip',
				prop: 'top-flip',
				defaultValue: "no",
				mode: 'topsep',
			},

			{
				id: 'bot-separator',
				label: 'Shape',
				options: {
					'normal': 'Normal',
					'triangle' : 'Triangle',
					'small-triangle' : 'Small Triangle',
					'big-triangle' : 'Big Triangle',
					'triangle-asym' : 'Triangle Asymmetrical',
					'slanted' : 'Slanted',
					'slanted-opac' : 'Slanted Opacity',
					'curve' : 'Curve',
					'wavy' : 'Wavy',
					'waves' : 'Waves',
					'waves-opac' : 'Waves Opacity',
					'spikes' : 'Spikes',
				},
				defaultValue: 'normal',
				type: 'select',
				action: 'separator',
				prop: 'separator',
				mode: 'botsep',
			},
			{
				id: 'bot-separator-color',
				label: 'Fill Color',
				type: 'color',
				action: 'sepcolor',
				defaultValue: '#FFFFFF',
				prop: 'color',
				mode: 'botsep',
			},
			{
				id: 'bot-sep-height',
				label: 'Height',
				type: 'slider',
				unit: 'px',
				defaultValue: 100,
				min: 50,
				max: 500,
				action: 'sepheight',
				prop: 'height',
				mode: 'botsep',
			},
			{
				id: 'bot-sep-flip',
				label: 'Flip',
				type: 'checkbox',
				action: 'sepflip',
				prop: 'flip',
				defaultValue: "no",
				mode: 'botsep',
			},
		],
		props: [
			'topseparator',
			'topcolor',
			'topheight',
			'topflip',
			'separator',
			'color',
			'height',
			'flip'
		]
	},
	minheight: {
		mode: false,
		title: 'Height',
		id: 'height',
		fields: [
			{
				id: 'min-height',
				label: 'Min-Height',
				type: 'slider',
				unit: 'px',
				min: 20,
				max: 1000,
				action: 'style',
				prop: 'minHeight',
			},
		],
		props: [
			'minHeight',
		]
	},
	height: {
		mode: false,
		title: 'Height',
		id: 'height',
		fields: [
			{
				id: 'elem-height',
				label: 'Height',
				type: 'slider',
				unit: 'px',
				min: 20,
				max: 1000,
				action: 'style',
				prop: 'height',
			},
		],
		props: [
			'height',
		]
	},
	boxsize: {
		mode: false,
		title: 'Box Size',
		id: 'box-size',
		fields: [
			{
				id: 'box-width',
				label: 'Width',
				type: 'slider',
				unit: 'px',
				min: 50,
				max: 1200,
				action: 'style',
				prop: 'width',
			},
			{
				id: 'box-min-height',
				label: 'Min-Height',
				type: 'slider',
				unit: 'px',
				min: 20,
				max: 1000,
				action: 'style',
				prop: 'minHeight',
			},
		],
		props: [
			'width',
			'minHeight',
		]
	},
	margin: {
		mode: false,
		title: 'Margin',
		id: 'margin',
		fields: [
			{
				id: 'margin-top',
				label: 'Margin Top',
				type: 'slider',
				unit: 'px',
				min: -300,
				max: 300,
				action: 'style',
				prop: 'marginTop',
			},
			{
				id: 'margin-bottom',
				label: 'Margin Bottom',
				type: 'slider',
				min: -300,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'marginBottom',
			}
		],
		props: [
			'marginTop',
			'marginBottom',
		]
	},
	padding: {
		mode: true,
		defaultMode: 'single',
		title: 'Padding',
		id: 'padding',
		modeAction: 'style',
		fields: [
			{
				id: 'padding-all',
				label: 'Padding',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'padding',
				mode: 'all'
			},
			{
				id: 'padding-top',
				label: 'Padding Top',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'paddingTop',
				mode: 'single'
			},
			{
				id: 'padding-right',
				label: 'Padding Right',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'paddingRight',
				mode: 'single'
			},
			{
				id: 'padding-bottom',
				label: 'Padding Bottom',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'paddingBottom',
				mode: 'single'
			},
			{
				id: 'padding-left',
				label: 'Padding Left',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'paddingLeft',
				mode: 'single'
			},
		],
		props: {
			all: 'paddingTop',
			single: [
				'paddingTop',
				'paddingRight',
				'paddingBottom',
				'paddingLeft'
			]
		}
	},
	colpadding: {
		mode: true,
		defaultMode: 'single',
		title: 'Column Padding',
		id: 'column-padding',
		modeAction: 'style',
		fields: [
			{
				id: 'col-padding-all',
				label: 'Padding',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'padding',
				mode: 'all'
			},
			{
				id: 'col-padding-top',
				label: 'Padding Top',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'paddingTop',
				mode: 'single'
			},
			{
				id: 'col-padding-right',
				label: 'Padding Right',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'paddingRight',
				mode: 'single'
			},
			{
				id: 'col-padding-bottom',
				label: 'Padding Bottom',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'paddingBottom',
				mode: 'single'
			},
			{
				id: 'col-padding-left',
				label: 'Padding Left',
				type: 'slider',
				min: 0,
				max: 300,
				unit: 'px',
				action: 'style',
				prop: 'paddingLeft',
				mode: 'single'
			},
		],
		props: {
			all: 'paddingTop',
			single: [
				'paddingTop',
				'paddingRight',
				'paddingBottom',
				'paddingLeft'
			]
		}
	},
	border: {
		mode: true,
		defaultMode: 'single',
		title: 'Border',
		id: 'border',
		modeAction: 'style',
		fields: [
			{
				id: 'border-all',
				label: 'Border',
				type: 'border',
				action: 'border',
				prop: 'border',
				mode: 'all',
				valueMap: ['border-width', 'border-style', 'border-color'],
				mapType: 'single-field'
			},
			{
				id: 'border-top',
				label: 'Border Top',
				type: 'border',
				action: 'border',
				prop: 'borderTop',
				mode: 'single',
				valueMap: ['border-width', 'border-style', 'border-color'],
				mapType: 'single-field'
			},
			{
				id: 'border-right',
				label: 'Border Right',
				type: 'border',
				action: 'border',
				prop: 'borderRight',
				mode: 'single',
				valueMap: ['border-width', 'border-style', 'border-color'],
				mapType: 'single-field'
			},
			{
				id: 'border-bottom',
				label: 'Border Bottom',
				type: 'border',
				action: 'border',
				prop: 'borderBottom',
				mode: 'single',
				valueMap: ['border-width', 'border-style', 'border-color'],
				mapType: 'single-field'
			},
			{
				id: 'border-left',
				label: 'Border Left',
				type: 'border',
				action: 'border',
				prop: 'borderLeft',
				mode: 'single',
				valueMap: ['border-width', 'border-style', 'border-color'],
				mapType: 'single-field'
			},
		],
		props: {
			all: 'borderTop',
			single: [
				'borderTop',
				'borderRight',
				'borderBottom',
				'borderLeft'
			]
		}
	},
	boxshadow: {
		mode: false,
		title: 'Shadow',
		id: 'boxshadow',
		fields: [
			{
				id: 'shadow-type',
				label: 'Shadow Mode',
				type: 'select',
				action: 'shadow',
				prop: 'boxShadow',
				defaultValue: 'none',
				classes: 'shadow-mode',
				options: {
					initial: 'Drop Shadow',
					inset: 'Inner Shadow',
					none: 'None',
				},
				valueMap: ['shadow-mode', 'shadow-h', 'shadow-v', 'shadow-blur', 'shadow-spread', 'shadow-color', 'shadow-alpha'],
				mapType: 'multi-field'
			},
			{
				id: 'horizontal-shadow',
				label: 'X-Offset',
				type: 'slider',
				min: 0,
				max: 100,
				unit: 'px',
				defaultValue: 0,
				action: 'shadow',
				prop: 'boxShadowH',
				classes: 'shadow-h',
				valueMap: ['shadow-mode', 'shadow-h', 'shadow-v', 'shadow-blur', 'shadow-spread', 'shadow-color', 'shadow-alpha'],
				mapType: 'multi-field'
			},
			{
				id: 'vertical-shadow',
				label: 'Y-Offset',
				type: 'slider',
				min: 0,
				max: 100,
				unit: 'px',
				defaultValue: 0,
				action: 'shadow',
				prop: 'boxShadowV',
				classes: 'shadow-v',
				valueMap: ['shadow-mode', 'shadow-h', 'shadow-v', 'shadow-blur', 'shadow-spread', 'shadow-color', 'shadow-alpha'],
				mapType: 'multi-field'
			},
			{
				id: 'blur-shadow',
				label: 'Blur',
				type: 'slider',
				min: 0,
				max: 100,
				unit: 'px',
				defaultValue: 4,
				action: 'shadow',
				prop: 'boxShadowBlur',
				classes: 'shadow-blur',
				valueMap: ['shadow-mode', 'shadow-h', 'shadow-v', 'shadow-blur', 'shadow-spread', 'shadow-color', 'shadow-alpha'],
				mapType: 'multi-field'
			},
			{
				id: 'spread-shadow',
				label: 'Spread',
				type: 'slider',
				min: 0,
				max: 100,
				unit: 'px',
				defaultValue: 2,
				action: 'shadow',
				prop: 'boxShadowSpread',
				classes: 'shadow-spread',
				valueMap: ['shadow-mode', 'shadow-h', 'shadow-v', 'shadow-blur', 'shadow-spread', 'shadow-color', 'shadow-alpha'],
				mapType: 'multi-field'
			},
			{
				id: 'shadow-color',
				label: 'Color',
				type: 'color',
				action: 'shadow',
				defaultValue: '#000000',
				prop: 'boxShadowColor',
				classes: 'shadow-color',
				valueMap: ['shadow-mode', 'shadow-h', 'shadow-v', 'shadow-blur', 'shadow-spread', 'shadow-color', 'shadow-alpha'],
				mapType: 'multi-field'
			},
			{
				id: 'thick-shadow',
				label: 'Thickness',
				type: 'slider',
				min: 0,
				max: 1,
				step: 0.1,
				unit: 'px',
				defaultValue: 0.1,
				action: 'shadow',
				prop: 'boxShadowAlpha',
				classes: 'shadow-alpha',
				valueMap: ['shadow-mode', 'shadow-h', 'shadow-v', 'shadow-blur', 'shadow-spread', 'shadow-color', 'shadow-alpha'],
				mapType: 'multi-field'
			},
		],
		props: ['boxShadow', 'boxShadowH', 'boxShadowV', 'boxShadowBlur', 'boxShadowSpread', 'boxShadowColor', 'boxShadowAlpha']
	},
	corner: {
		mode: true,
		defaultMode: 'single',
		title: 'Corners',
		id: 'corner',
		modeAction: 'style',
		fields: [
			{
				id: 'corner-all',
				label: 'Corner Shape',
				type: 'slider',
				min: 0,
				max: 100,
				unit: 'px',
				action: 'style',
				prop: 'borderRadius',
				mode: 'all'
			},
			{
				id: 'corner-top-left',
				label: 'Top Left Corner Shape',
				type: 'slider',
				min: 0,
				max: 100,
				unit: 'px',
				action: 'style',
				prop: 'borderTopLeftRadius',
				mode: 'single'
			},
			{
				id: 'corner-top-right',
				label: 'Top Right Corner Shape',
				type: 'slider',
				min: 0,
				max: 100,
				unit: 'px',
				action: 'style',
				prop: 'borderTopRightRadius',
				mode: 'single'
			},
			{
				id: 'corner-bottom-left',
				label: 'Bottom Left Corner Shape',
				type: 'slider',
				min: 0,
				max: 100,
				unit: 'px',
				action: 'style',
				prop: 'borderBottomLeftRadius',
				mode: 'single'
			},
			{
				id: 'corner-bottom-right',
				label: 'Bottom Right Corner Shape',
				type: 'slider',
				min: 0,
				max: 100,
				unit: 'px',
				action: 'style',
				prop: 'borderBottomRightRadius',
				mode: 'single'
			},
		],
		props: {
			all: 'borderTopLeftRadius',
			single: [
				'borderTopLeftRadius',
				'borderTopRightRadius',
				'borderBottomLeftRadius',
				'borderBottomRightRadius'
			]
		}
	},
	opacity: {
		title: 'Opacity',
		id: 'opacity',
		fields: [
			{
				id: 'opacity',
				label: 'Opacity',
				type: 'slider',
				min: 0,
				max: 1,
				step: 0.1,
				unit: 'px',
				action: 'opacity',
				prop: 'opacity',
			},
		],
		props: [
			'opacity'
		]
	},
	effect: {
		title: 'Effect',
		tabs: [{ key: 'anim', name: 'Animation' }, { key: 'delay', name: 'Show Delay' }],
		defaultTab: 'anim',
		id: 'effect',
		fields: [
			{
				id: 'animation',
				label: 'Animation',
				type: 'select',
				action: 'animation',
				file: 'animation',
				defaultValue: 'none',
				options: {
					none: 'None',
					pulsate: 'Flash',
					pulse: 'Pulse',
					explode: 'Rubber Band',
					shake: 'Shake',
					headShake: 'Head Shake',
					swing: 'Swing',
					puff: 'Tada',
					wobble: 'Wobble',
					jello: 'Jello',
					bounceIn: 'Bounce In',
					drop: 'Bounce In Down',
					bounceInLeft: 'Bounce In Left',
					bounceInRight: 'Bounce In Right',
					bounceInUp: 'Bounce In Up',
					fadeIn: 'Fade In',
					blind: 'Fade In Down',
					fadeInDownBig: 'Fade In Down Big',
					slide: 'Fade In Left',
					fadeInLeftBig: 'Fade In Left Big',
					fadeInRight: 'Fade In Right',
					fadeInRightBig: 'Fade In Right Big',
					clip: 'Fade In Up',
					fadeInUpBig: 'Fade In Up Big',
					fold: 'Flip In X',
					flipInY: 'Flip In Y',
					highlight: 'Light Speed In',
					rotateIn: 'Rotate In',
					rotateInDownLeft: 'Rotate In Down Left',
					rotateInDownRight: 'Rotate In Down Right',
					rotateInUpLeft: 'Rotate In Up Left',
					rotateInUpRight: 'Rotate In Up Right',
					hinge: 'Hinge',
					jackInTheBox: 'Jack In The Box',
					rollIn: 'Roll In',
					scale: 'Zoom In',
					zoomInDown: 'Zoom In Down',
					zoomInLeft: 'Zoom In Left',
					zoomInRight: 'Zoom In Right',
					zoomInUp: 'Zoom In',
					slideInDown: 'Slide In Down',
					slideInLeft: 'Slide In Left',
					slideInRight: 'Slide In Right',
					slideInUp: 'Slide In Up',
				},
				mode: 'anim'
			},
			{
				id: 'show-delay',
				label: 'Enable Delay',
				type: 'checkbox',
				action: 'savedata',
				prop: 'showDelay',
				reveal: 'show-on-reveal',
				mode: 'delay'
			},
			{
				id: 'delay-hour',
				label: 'Hour',
				type: 'slider',
				min: 0,
				max: 23,
				action: 'savedata',
				file: 'delayhour',
				classes: 'show-on-reveal chk-hidden',
				mode: 'delay'
			},
			{
				id: 'delay-min',
				label: 'Minute',
				type: 'slider',
				min: 0,
				max: 59,
				action: 'savedata',
				file: 'delaymin',
				classes: 'show-on-reveal chk-hidden',
				mode: 'delay'
			},
			{
				id: 'delay-sec',
				label: 'Second',
				type: 'slider',
				min: 0,
				max: 59,
				action: 'savedata',
				file: 'delaysec',
				classes: 'show-on-reveal chk-hidden',
				mode: 'delay'
			},
		],
		props: [
			'showDelay'
		],
		files: [
			'animation',
			'delayhour',
			'delaymin',
			'delaysec'
		]
	}
};