PrimeBuilder.Utils = {
  matches: function (el, selector) {
  	const matches = el && ( el.matches || el.webkitMatchesSelector || el.mozMatchesSelector || el.msMatchesSelector || el.oMatchesSelector );
  	return !!matches && matches.call(el, selector);
	},
	contains: function (parent, node) {
		while ( node && (node = node.parentNode) )
      if ( node === parent ) return true;
    	
    	return false;
  },
  isDocument: function (obj) {
  	return obj != null && obj.nodeType == obj.DOCUMENT_NODE
  },
  ucwords: function (str) {
		str = str.toLowerCase();
		return str.replace(/(^([a-zA-Z\p{M}]))|([ -][a-zA-Z\p{M}])/g,
			function(s) {
	  			return s.toUpperCase();
		});
	},
  trim: function(str) {
    return str.replace(/^\s+|\s+$/gm,'');
  },
  camelToDash: function(str) {
    let newStr = str.replace(/\W+/g, '-')
                  .replace(/([a-z\d])([A-Z])/g, '$1-$2');
    return newStr.toLowerCase();
  },
  dashToCamel: function(str) {
    return str.replace(/-([a-z])/g, function (g) { return g[1].toUpperCase(); });
  },
  slugify: function (text) {
    return text.toString().toLowerCase()
      .replace(/\s+/g, '-')           // Replace spaces with -
      .replace(/[^\w\-]+/g, '')       // Remove all non-word chars
      .replace(/\-\-+/g, '-')         // Replace multiple - with single -
      .replace(/^-+/, '')             // Trim - from start of text
      .replace(/-+$/, '');            // Trim - from end of text
  },
  rgbToHex: function(rgb) {
    if ( rgb.substr(0, 1) === "#" ) {
      return rgb;
    }

    if ( rgb == 'transparent' ) {
      return rgb;
    }

    let parts = rgb.substring(rgb.indexOf("(")).split(",");
    let r = parseInt(this.trim(parts[0].substring(1)), 10),
        g = parseInt(this.trim(parts[1]), 10),
        b = parseInt(this.trim(parts[2]), 10);
        
    if ( typeof parts[3] !== "undefined" ) {
      let alpha = parseFloat(this.trim(parts[3]));
      if ( alpha === 0 ) {
        return 'transparent';
      }
    }

    return ('#' + this.componentToHex(r) + this.componentToHex(g) + this.componentToHex(b));
  },
  componentToHex: function (c) {
    let hex = c.toString(16);
    return hex.length == 1 ? "0" + hex : hex;
  },
  hexToRgb: function(hex) {
    if ( hex == 'transparent' ) {
      return { r: 0, g: 0, b: 0 };
    }

    // Expand shorthand form (e.g. "03F") to full form (e.g. "0033FF")
    let shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
    hex = hex.replace(shorthandRegex, function(m, r, g, b) {
        return r + r + g + g + b + b;
    });

    let result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? {
        r: parseInt(result[1], 16),
        g: parseInt(result[2], 16),
        b: parseInt(result[3], 16)
    } : null;
  },
  getGradientColors: function(color) {
    let num = 35, r, g, b, origColor = color;
    
    if ( origColor == 'transparent' ) {
      return false;
    }

    if ( color.substr(0, 1) === "#" ) {
      color = this.hexToRgb(color);
      r = color.r;
      g = color.g;
      b = color.b;
    } else {
      color = /(.*?)rgb\((\d+),\s*(\d+),\s*(\d+)\)/i.exec(color);

      if ( !color ) {
        return false;
      }

      r = color[2];
      g = color[3];
      b = color[4];
    }
    
    let hsp = Math.sqrt( // HSP equation from http://alienryderflex.com/hsp.html
      0.299 * (r * r) +
      0.587 * (g * g) +
      0.114 * (b * b)
    );

    if ( hsp > 127.5 ) {
      num = -28;
    }

    r = r + num; g = g + num; b = b + num;
    if ( r > 255 ) r = 255; if ( r < 0 ) r = 0;
    if ( g > 255 ) r = 255; if ( g < 0 ) g = 0;
    if ( b > 255 ) r = 255; if ( b < 0 ) b = 0;
    
    let newColor = "#" + this.componentToHex(r) + this.componentToHex(g) + this.componentToHex(b),
        darkerColor = ( hsp > 127.5 ) ? this.getDarkerColor(newColor) : this.getDarkerColor(origColor);
    return ( hsp > 127.5 ) ? [origColor, newColor, darkerColor] : [newColor, origColor, darkerColor];
  },
  getDarkerColor: function(color) {
    let num = -28, r, g, b;
    
    if ( color.substr(0, 1) === "#" ) {
      color = this.hexToRgb(color);
      r = color.r;
      g = color.g;
      b = color.b;
    } else {
      color = /(.*?)rgb\((\d+),\s*(\d+),\s*(\d+)\)/i.exec(color),
      r = color[2];
      g = color[3];
      b = color[4];
    }

    r = r + num; g = g + num; b = b + num;
    if ( r > 255 ) r = 255; if ( r < 0 ) r = 0;
    if ( g > 255 ) r = 255; if ( g < 0 ) g = 0;
    if ( b > 255 ) r = 255; if ( b < 0 ) b = 0;
    
    return "#" + this.componentToHex(r) + this.componentToHex(g) + this.componentToHex(b);
  },
  inputTextColor: function(color) {
    if ( typeof color === 'undefined' ) {
      return color;
    }

    if ( color == 'transparent' || $.trim(color) === '' ) {
      return '#fefefe';
    }

    if ( color.substr(0, 1) === "#" ) {
        color = this.hexToRgb(color);
    }

    if ( typeof color === "string" && color.match(/^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/) ) {
      color = color.match(/^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/);
      r = color[1];
      g = color[2];
      b = color[3];
    } else if ( typeof color === "object" ) {
      r = color.r;
      g = color.g;
      b = color.b;
    }

    let hsp = Math.sqrt( // HSP equation from http://alienryderflex.com/hsp.html
      0.299 * (r * r) +
      0.587 * (g * g) +
      0.114 * (b * b)
    );

    if ( hsp > 127.5 ) {
      return '#555';
    } else {
      return '#fefefe';
    }
  },
  isURL: function(str) {
    var urlregex =  /^(?:(?:https?|ftp):\/\/)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,})))(?::\d{2,5})?(?:\/\S*)?$/;
    return urlregex.test(str);
  },
  isTransparent: function(color) {
    if ( typeof color === "string" && color != '' ) {
      if ( color == 'transparent' ) {
        return true;
      }

      if ( color.substr(0, 1) === "#" ) {
        return false;
      }
      if ( color.match(/^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/) ) {
        color = color.match(/^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/);
        if ( typeof color[4] !== "undefined" && parseInt(color[4]) == 0 ) {
          return true;
        }
      }
    }
    return false;
  }
};