<?php
require_once( IB2_INC . 'libs/Ib2Oauth1.php' );
require_once( IB2_INC . 'libs/Ib2ApiRequest.php' );

function ib2_api_user_agent() {
	$agent = sanitize_text_field(get_bloginfo('name'));
	$agent = str_replace(' ', '-', strtolower($agent));
	return $agent . ' - ' . get_bloginfo('url');
}

function ib2_aw_auth($auth_code) {
	$app_id = IB2_AW_APPID;
	$access_token_url = 'https://auth.aweber.com/1.0/oauth/access_token';
	if ( empty($auth_code) ) {
		return new WP_Error('authcode_missing', 'Auth Code is Missing.');
	}

	$parts = explode("|", $auth_code);
	if ( !$parts || !is_array($parts) ) {
		return new WP_Error('authcode_error', 'Invalid Auth Code.');
	}
	list($consumer_key, $consumer_secret, $request_token, $request_token_secret, $verifier, $blank) = $parts;

	$config = array(
        'request_method' => 'POST',
        'url' => $access_token_url,
        'consumer_key' => $consumer_key,
        'consumer_secret' => $consumer_secret,
        'request_token' => $request_token,
        'token_secret' => $request_token_secret,
    );

    $oauth1 = new Ib2Oauth1($config, array('oauth_verifier' => $verifier));
    $signed = $oauth1->getSigned();

    if ( !$signed ) {
    	return new WP_Error('oauth_failed', 'Failed to create Oauth1 signature.');
    }

    $http = new Ib2ApiRequest();
	$http->setHeaders(array(
		'Content-Type' => 'application/x-www-form-urlencoded',
		'User-Agent' => ib2_api_user_agent(),
		'Expect' => ''
	));
	$response = $http->call($access_token_url, $signed, 'POST');
	if ( is_wp_error($response) ) {
		return $response;
	}

	if ( !is_array($response) ) {
		if ( is_string($response) ) {
			parse_str($response, $arr);
			$response = $arr;
		} else {
			return new WP_Error('oauth1_invalid', 'Invalid Response', $response);
		}
	}

	$access_token = isset($response['oauth_token']) ? $response['oauth_token'] : '';
	$token_secret = isset($response['oauth_token_secret']) ? $response['oauth_token_secret'] : '';

	return array('consumer_key' => $consumer_key, 'consumer_secret' => $consumer_secret, 'access_token' => $access_token, 'token_secret' => $token_secret);
}

function ib2_aw_account() {
    if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['aw']) || empty($esp['aw']) ) {
		return new WP_Error('noapikey', 'API Token Not Found.');
	}

	$api_key = $esp['aw'];
	$consumer_key = trim($api_key['consumer_key']);
	$consumer_secret = trim($api_key['consumer_secret']);
	$access_token = trim($api_key['access_token']);
	$token_secret = trim($api_key['token_secret']);
	$config = array(
        'request_method' => 'GET',
        'url' => 'https://api.aweber.com/1.0/accounts',
        'consumer_key' => $consumer_key,
        'consumer_secret' => $consumer_secret,
        'access_token' => $access_token,
        'token_secret' => $token_secret,
    );

    $oauth1 = new Ib2Oauth1($config);
    $signed = $oauth1->getSigned();

    if ( !$signed ) {
    	return new WP_Error('oauth_failed', 'Failed to create Oauth1 signature.');
    }

    //$cert = IB2_INC . 'certs/cacert.pem';
    $http = new Ib2ApiRequest();
	$http->setHeaders(array(
		'User-Agent' => ib2_api_user_agent(),
		'Expect' => ''
	));
	$response = $http->call('https://api.aweber.com/1.0/accounts', $signed);
	if ( is_wp_error($response) || !is_array($response) || ( is_array($response) && !isset($response['entries']) ) ) {
		return $response;
	}

	$entries = $response['entries'][0];
    $esp['aw']['account_id'] = $entries['id'];
    $esp['aw']['lists_collection_link'] = $entries['lists_collection_link'];
    update_option('ib2_connected_esp', $esp);
    return $entries;
}

function ib2_aw_fetch_list($next_collection_link = '') {
	if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['aw']) || empty($esp['aw']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = $esp['aw'];
	$consumer_key = trim($api_key['consumer_key']);
	$consumer_secret = trim($api_key['consumer_secret']);
	$access_token = trim($api_key['access_token']);
	$token_secret = trim($api_key['token_secret']);
	$account_id = isset($api_key['account_id']) ? $api_key['account_id'] : false;
	$lists_collection_link = isset($api_key['lists_collection_link']) ? $api_key['lists_collection_link'] : false;

	if ( !$account_id ) {
		$account = ib2_aw_account();
		if ( is_wp_error($account) || !is_array($account) || ( is_array($account) && !isset($account['id']) ) ) {
			return new WP_Error('noaccountid', 'Cannot find Aweber account id.', $account);
		}
		$account_id = $account['id'];
		$lists_collection_link = $entries['lists_collection_link'];
	}

	$api_url = ( !empty($next_collection_link) ? $next_collection_link : $lists_collection_link );
	$config = array(
        'request_method' => 'GET',
        'url' => $api_url,
        'consumer_key' => $consumer_key,
        'consumer_secret' => $consumer_secret,
        'access_token' => $access_token,
        'token_secret' => $token_secret,
    );

    $oauth1 = new Ib2Oauth1($config);
    $signed = $oauth1->getSigned();

    if ( !$signed ) {
    	return new WP_Error('oauth_failed', 'Failed to create Oauth1 signature.');
    }

    $http = new Ib2ApiRequest();
    $http->setHeaders(array(
		'User-Agent' => ib2_api_user_agent(),
		'Expect' => ''
	));

    $response = $http->call($api_url, $signed);
	if ( is_wp_error($response) ) {
		return $response;
	}

 	$lists = array();
	if ( is_array($response) && isset($response['entries']) ) {
        $data = $response['entries'];
        if ( count($data) ) {
            foreach ($data as $list) {
                $lists[$list['id']] = $list['name'];
            }
        }

        if ( !empty($response['next_collection_link']) && filter_var($response['next_collection_link'], FILTER_VALIDATE_URL) ) {
        	$nexts = ib2_aw_fetch_list($response['next_collection_link']);
        	if ( !is_wp_error($nexts) && is_array($nexts) ) {
        		$lists = array_merge($lists, $nexts);
        	}
        }
    }
    return $lists;
}

function ib2_get_aw_subscriber($list, $email) {
	if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['aw']) || empty($esp['aw']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = $esp['aw'];
	$consumer_key = trim($api_key['consumer_key']);
	$consumer_secret = trim($api_key['consumer_secret']);
	$access_token = trim($api_key['access_token']);
	$token_secret = trim($api_key['token_secret']);
	$account_id = isset($api_key['account_id']) ? $api_key['account_id'] : false;

	if ( !$account_id ) {
		$account = ib2_aw_account();
		if ( is_wp_error($account) || !is_array($account) || ( is_array($account) && !isset($account['id']) ) ) {
			return new WP_Error('noaccountid', 'Cannot find Aweber account id.', $account);
		}
		$account_id = $account['id'];
	}

	$api_url = "https://api.aweber.com/1.0/accounts/{$account_id}/lists/{$list}/subscribers";
	$config = array(
        'request_method' => 'GET',
        'url' => $api_url,
        'consumer_key' => $consumer_key,
        'consumer_secret' => $consumer_secret,
        'access_token' => $access_token,
        'token_secret' => $token_secret,
    );

	$params = array(
		'ws.op' => 'find',
		'email' => $email
	);

    $oauth1 = new Ib2Oauth1($config, $params);
    $signed = $oauth1->getSigned();

    if ( !$signed ) {
    	return new WP_Error('oauth_failed', 'Failed to create Oauth1 signature.');
    }

    $http = new Ib2ApiRequest();
    $http->setHeaders(array(
		'User-Agent' => ib2_api_user_agent(),
		'Expect' => ''
	));

    $response = $http->call($api_url, $signed);
	if ( is_wp_error($response) ) {
		return $response;
	}

 	if ( is_array($response) && isset($response['entries']) && count($response['entries']) > 0 ) {
        return $response['entries'][0];
    }
    return false;
}

function ib2_aw_subscribe($list, $data) {
	if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['aw']) || empty($esp['aw']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = $esp['aw'];
	$consumer_key = trim($api_key['consumer_key']);
	$consumer_secret = trim($api_key['consumer_secret']);
	$access_token = trim($api_key['access_token']);
	$token_secret = trim($api_key['token_secret']);
	$account_id = isset($api_key['account_id']) ? $api_key['account_id'] : false;

	if ( !$account_id ) {
		$account = ib2_aw_account();
		if ( is_wp_error($account) || !is_array($account) || ( is_array($account) && !isset($account['id']) ) ) {
			return new WP_Error('noaccountid', 'Cannot find Aweber account id.', $account);
		}
		$account_id = $account['id'];
	}

	$api_url = "https://api.aweber.com/1.0/accounts/{$account_id}/lists/{$list}/subscribers";
	$config = array(
        'request_method' => 'POST',
        'url' => $api_url,
        'consumer_key' => $consumer_key,
        'consumer_secret' => $consumer_secret,
        'access_token' => $access_token,
        'token_secret' => $token_secret,
    );

    // prepare data
    $first_name = isset($data['firstname']) ? trim($data['firstname']) : '';
    $last_name = isset($data['lastname']) ? trim($data['lastname']) : '';
    $email = $data['email'];
    $fields = !empty($data['fields']) ? $data['fields'] : null;

    if ( $subscriber = ib2_get_aw_subscriber($list, $email) ) {
    	return true;
    }

    $postdata = array(
    	'ws.op' => "create",
        'email' => $email,
        'last_followup_message_number_sent' => 0,
        'ad_tracking' => 'instabuilder'
    );

    if ( !empty($data['ip']) ) {
        $postdata['ip_address'] = $data['ip'];
    }

    if ( !empty($first_name) ) {
    	$fullname = $first_name . ' ' . $last_name;
        $postdata['name'] = trim($fullname);
    }

    if ( $fields ) {
    	$custom_fields = ib2_aw_fields($list);
    	$aw_fields = array();
    	if ( $custom_fields ) {
    		foreach ( $custom_fields as $cf ) {
    			if ( isset($cf['name']) && isset($fields[$cf['name']]) ) {
    				$cf_val = $fields[$cf['name']];
    				$aw_fields[$cf['name']] = $cf_val;
    			}
        	}
    	}

    	if ( !empty($aw_fields) ) {
    		$postdata['custom_fields'] = $aw_fields;
    	}
    }

    $oauth1 = new Ib2Oauth1($config, $postdata);
    $signed = $oauth1->getSigned();

    if ( !$signed ) {
    	return new WP_Error('oauth_failed', 'Failed to create Oauth1 signature.');
    }

    $http = new Ib2ApiRequest();
    $http->setHeaders(array(
		'User-Agent' => ib2_api_user_agent(),
		'Expect' => ''
	));

    $response = $http->call($api_url, $signed, 'POST');
	return $response;
}

function ib2_aw_fields($list) {
    if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['aw']) || empty($esp['aw']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = $esp['aw'];
	$consumer_key = trim($api_key['consumer_key']);
	$consumer_secret = trim($api_key['consumer_secret']);
	$access_token = trim($api_key['access_token']);
	$token_secret = trim($api_key['token_secret']);
	$account_id = isset($api_key['account_id']) ? $api_key['account_id'] : false;

	if ( !$account_id ) {
		$account = ib2_aw_account();
		if ( is_wp_error($account) || !is_array($account) || ( is_array($account) && !isset($account['id']) ) ) {
			return new WP_Error('noaccountid', 'Cannot find Aweber account id.', $account);
		}
		$account_id = $account['id'];
	}

	$api_url = "https://api.aweber.com/1.0/accounts/{$account_id}/lists/{$list}/custom_fields";
	$config = array(
        'request_method' => 'GET',
        'url' => $api_url,
        'consumer_key' => $consumer_key,
        'consumer_secret' => $consumer_secret,
        'access_token' => $access_token,
        'token_secret' => $token_secret,
    );

	$oauth1 = new Ib2Oauth1($config);
    $signed = $oauth1->getSigned();

    if ( !$signed ) {
    	return new WP_Error('oauth_failed', 'Failed to create Oauth1 signature.');
    }

    $http = new Ib2ApiRequest();
    $http->setHeaders(array(
		'User-Agent' => ib2_api_user_agent(),
		'Expect' => ''
	));

    $response = $http->call($api_url, $signed);
	if ( is_wp_error($response) || !is_array($response) || ( is_array($response) && !isset($response['entries']) ) ) {
		return $response;
	}

	return $response['entries'];
}

function ib2_ac_fetch_list() {
	if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['ac']) || empty($esp['ac']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = trim($esp['ac']);
	list($url, $key) = explode('|', $api_key);

	$api_key = $key;
	$api_url = $url;

	$http = new Ib2ApiRequest($api_url);
	$http->setHeaders(array(
		'Content-Type' => 'application/x-www-form-urlencoded'
	));

	$params = array(
	    'api_key' => $api_key,
	    'api_action' => 'list_paginator',
	    'api_output' => 'json',
	    'somethingthatwillneverbeused' => '',
	    'sort' => '',
	    'offset' => 0,
	    'limit' => 100,
	    'filter' => 0,
	    'public' => 0,
	);

	$query = "";
	foreach( $params as $key => $value ) $query .= urlencode($key) . '=' . urlencode($value) . '&';
	$query = rtrim($query, '& ');

	$response = $http->call('/admin/api.php?' . $query);
	if ( is_wp_error($response) ) {
		return $response;
	}

	$is_error = ( $response['result_code'] ? false : true );
 	$lists = array();
	if ( $response && !$is_error ) {
        $data = ( is_array($response) && isset($response['rows']) ) ? $response['rows'] : array();
        if ( count($data) ) {
            foreach ($data as $list) {
                $lists[$list['id']] = $list['name'];
            }
        }
    }
    return $lists;
}

function ib2_ac_subscribe($list, $data) {
	if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['ac']) || empty($esp['ac']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = trim($esp['ac']);
	list($url, $key) = explode('|', $api_key);

	$api_key = $key;
	$api_url = $url;

	$http = new Ib2ApiRequest($api_url);
	$http->setHeaders(array(
		'Content-Type' => 'application/x-www-form-urlencoded'
	));

	$params = array(
	    'api_key' => $api_key,
	    'api_action' => 'contact_add',
	    'api_output' => 'json',
	);

	$query = "";
	foreach( $params as $key => $value ) $query .= urlencode($key) . '=' . urlencode($value) . '&';
	$query = rtrim($query, '& ');

	// prepare data
    $first_name = isset($data['firstname']) ? trim($data['firstname']) : '';
    $last_name = isset($data['lastname']) ? trim($data['lastname']) : '';
    $email = $data['email'];
    $fields = !empty($data['fields']) ? $data['fields'] : null;

    $postdata = [
        'email' => $email,
        'tags' => 'api,instabuilder',
        'p[' . $list . ']' => $list,
        'status[' . $list . ']' => 1,
        'instantresponders[' . $list . ']' => 1,
    ];

    if ( !empty($first_name) ) {
        $postdata['first_name'] = $first_name;
    }

    if ( !empty($last_name) ) {
        $postdata['last_name'] = $last_name;
    }

	$response = $http->call('/admin/api.php?' . $query, $postdata, 'POST');
	return $response;
}

function ib2_mc_fetch_list() {
	if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['mc']) || empty($esp['mc']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = trim($esp['mc']);
	list($num, $dc) = explode('-', $api_key);
	$api_url = 'https://'.$dc.'.api.mailchimp.com/3.0';

	$http = new Ib2ApiRequest($api_url);
	$http->setHeaders(array(
		'Authorization' => "apikey {$api_key}"
	));

	$response = $http->call('/lists', array('count' => 1000));
	if ( is_wp_error($response) ) {
		return $response;
	}

 	$lists = array();
	if ( $response ) {
        $data = ( is_array($response) && isset($response['lists']) ) ? $response['lists'] : array();
        if ( count($data) ) {
            foreach ($data as $list) {
                $lists[$list['id']] = $list['name'];
            }
        }
    }
    return $lists;
}

function ib2_mc_subscribe($list, $data) {
	if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['mc']) || empty($esp['mc']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = trim($esp['mc']);
	list($num, $dc) = explode('-', $api_key);
	$api_url = 'https://'.$dc.'.api.mailchimp.com/3.0';

	$http = new Ib2ApiRequest($api_url);
	$http->setHeaders(array(
		'Authorization' => "apikey {$api_key}",
		'Content-Type' => 'application/json',
		'Accept' => 'application/json'
	));

	// prepare data
    $first_name = isset($data['firstname']) ? trim($data['firstname']) : '';
    $last_name = isset($data['lastname']) ? trim($data['lastname']) : '';
    $email = $data['email'];
    $fields = !empty($data['fields']) ? $data['fields'] : null;

    $postdata = [
        'email_address' => $email,
        'email_type' => 'html',
        'status' => 'subscribed',
        'ip_signup' => $data['ip']
    ];

    if ( !empty($first_name) ) {
        $postdata['merge_fields'] = [
            'FNAME' => $first_name,
            'LNAME' => ( !empty($last_name) ? $last_name : $first_name )
        ];
    }

    if ( $fields ) {
        foreach ( $fields as $field => $value ) {
            $postdata['merge_fields'][strtoupper($field)] = $value;
        }
    }

	$response = $http->call("/lists/{$list}/members", $postdata, 'POST');
	return $response;
}

function ib2_gr_fetch_list() {
	if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['gr']) || empty($esp['gr']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = trim($esp['gr']);
	$api_url = 'https://api.getresponse.com/v3';

	$http = new Ib2ApiRequest($api_url);
	$http->setHeaders(array(
		'X-Auth-Token' => "api-key {$api_key}",
		'Accept' => 'application/json'
	));

	$response = $http->call('/campaigns');
	if ( is_wp_error($response) ) {
		return $response;
	}
	$lists = array();
	if ( $response ) {
        $data = ( is_array($response) ) ? $response : array();
        if ( count($data) ) {
            foreach ($data as $list) {
                $lists[$list['campaignId']] = $list['name'];
            }
        }
    }
    return $lists;
}

function ib2_gr_subscribe($list, $data) {
	if ( !$esp = get_option('ib2_connected_esp') ) {
		return new WP_Error('nooption', 'ESP Option Not Found.');
	}

	if ( !isset($esp['gr']) || empty($esp['gr']) ) {
		return new WP_Error('noapikey', 'API Key Not Found.');
	}

	$api_key = trim($esp['gr']);
	$api_url = 'https://api.getresponse.com/v3';

	$http = new Ib2ApiRequest($api_url);
	$http->setHeaders(array(
		'X-Auth-Token' => "api-key {$api_key}",
		'Content-Type' => 'application/json',
		'Accept' => 'application/json'
	));

	// prepare data
    $first_name = isset($data['firstname']) ? trim($data['firstname']) : '';
    $last_name = isset($data['lastname']) ? trim($data['lastname']) : '';
    $email = $data['email'];
    $fields = !empty($data['fields']) ? $data['fields'] : null;

    $postdata = [
        'email' => $email,
        'dayOfCycle' => 0,
        'campaign' => ['campaignId' => $list],
        'ipAddress' => $data['ip'],
    ];

    if ( !empty($first_name) ) {
        $fullname = $first_name . ' ' . $last_name;
        $postdata['name'] = trim($fullname);
    }

    if ( $fields ) {
    	$custom_fields = ib2_gr_fields();
    	$gr_fields = array();
    	if ( $custom_fields ) {
    		foreach ( $custom_fields as $cf ) {
    			if ( isset($cf['customFieldId']) && isset($fields[$cf['customFieldId']]) ) {
    				$cf_val = $fields[$cf['customFieldId']];
    				$gr_fields[] = array(
	                    'customFieldId' => $cf['customFieldId'],
	                    'value' => [$cf_val],
	                );
    			}
        	}
    	}

    	if ( !empty($gr_fields) ) {
    		$postdata['customFieldValues'] = $gr_fields;
    	}
    }

	$response = $http->call("/contacts", $postdata, 'POST');
	return $response;
}

function ib2_gr_fields() {
   if ( !$esp = get_option('ib2_connected_esp') ) {
		return false;
	}

	if ( !isset($esp['gr']) || empty($esp['gr']) ) {
		return false;
	}

	$api_key = trim($esp['gr']);
	$api_url = 'https://api.getresponse.com/v3';

	$http = new Ib2ApiRequest($api_url);
	$http->setHeaders(array(
		'X-Auth-Token' => "api-key {$api_key}",
		'Accept' => 'application/json'
	));

    $endpoint = '/custom-fields?fields=name&sort[name]=desc&page=1&perPage=100';
    $response = $http->call($endpoint);
	if ( is_wp_error($response) ) {
		return false;
	}
	return $response;
}

function ib2_mailpoet() {
	if ( version_compare(PHP_VERSION, '5.6.0', '<') ) {
		return false;
	}

	$mailpoet = is_plugin_active('mailpoet/mailpoet.php');
	if ( $mailpoet ) {
		// check version
		$file = WP_CONTENT_DIR . '/plugins/mailpoet/mailpoet.php';
		if ( !file_exists($file) ) {
			return false;
		}
		$data = get_plugin_data($file);
		if ( !isset($data['Version']) ) {
			return false;
		}
		if ( version_compare($data['Version'], '3.0.0', '<') ) {
			return false;
		}
	}
	return $mailpoet;
}

function ib2_mp_fetch_list() {
	$mailpoet = ib2_mailpoet();
	if ( !$mailpoet ) {
		return false;
	}

	if ( !class_exists('\MailPoet\API\API') ) {
		return false;
	}

	$response = \MailPoet\API\API::MP('v1')->getLists();
	$lists = array();
	if ( $response ) {
        $data = ( is_array($response) ) ? $response : array();
        if ( count($data) ) {
            foreach ($data as $list) {
                $lists[$list['id']] = $list['name'];
            }
        }
    }
    return $lists;
}

function ib2_mp_subscribe($list, $data) {
	$mailpoet = ib2_mailpoet();
	if ( !$mailpoet ) {
		return false;
	}

	if ( !class_exists('\MailPoet\API\API') ) {
		return false;
	}

	// prepare data
    $first_name = isset($data['firstname']) ? trim($data['firstname']) : '';
    $last_name = isset($data['lastname']) ? trim($data['lastname']) : '';
    $email = $data['email'];
    $fields = !empty($data['fields']) ? $data['fields'] : null;
    $confirm = $data['confirm'];
    $lists = array($list);
    $options = array(
    	'send_confirmation_email' => $confirm,
  		'schedule_welcome_email' => true
    );

    $postdata = [
        'email' => $email,
    ];

    if ( !empty($first_name) ) {
        $postdata['first_name'] = $first_name;
    }

    if ( !empty($last_name) ) {
        $postdata['last_name'] = $last_name;
    }

    if ( $fields ) {
    	$custom_fields = ib2_mp_fields();
    	if ( $custom_fields ) {
    		foreach ( array_keys($custom_fields) as $cf ) {
    			if ( isset($fields[$cf]) ) {
    				$cf_val = $fields[$cf];
    				$postdata[$cf] = $cf_val;
    			}
        	}
    	}
    }

	try {
  		$subscriber = \MailPoet\API\API::MP('v1')->addSubscriber($postdata, $lists, $options);
  		return $subscriber;
	} catch(Exception $exception) {
  		// return $exception->getMessage();
  		return false;
	}
}

function ib2_mp_fields() {
    $mailpoet = ib2_mailpoet();
	if ( !$mailpoet ) {
		return false;
	}

	if ( !class_exists('\MailPoet\API\API') ) {
		return false;
	}

    $response = \MailPoet\API\API::MP('v1')->getSubscriberFields();
    return $response;
}