<?php

class Ib2Oauth1 {
	protected $config;
	protected $requestData = array();
	protected $payload = array();
	protected $signed;

	public function __construct($config, array $requestData = array()) 
	{
		$this->config = array(
            'version' => '1.0',
            'request_method' => 'POST',
            'url' => null,
            'consumer_key' => null,
            'consumer_secret' => null,
            'access_token' => null,
            'authorized_token' => null,
            'request_token' => null,
            'token_secret' => null,
        );

        foreach ($config as $key => $value) {
            $this->config[$key] = $value;
        }

		$this->requestData = $requestData;

		$this->prepare();
		$this->signRequest();
	}

	public function getSigned() {
		return $this->signed;
	}

	protected function prepare()
    {
        $this->signed = $this->mergeOAuthData();
    }

    protected function mergeOAuthData()
    {
    	$requestData = $this->requestData;
        $oauthData = $this->getOAuthRequestData();
        return array_merge($requestData, $oauthData);
    }

    protected function getOAuthRequestData()
    {
        $token = $this->getHighestPriorityToken();
        $ts = $this->generateTimestamp();
        $nonce = $this->generateNonce($ts);
        return array(
            'oauth_token' => $token,
            'oauth_consumer_key' => $this->config['consumer_key'],
            'oauth_version' => '1.0',
            'oauth_timestamp' => $ts,
            'oauth_signature_method' => 'HMAC-SHA1',
            'oauth_nonce' => $nonce
        );
    }

    protected function signRequest()
    {
        $base = $this->createSignatureBase();
        $key = $this->createSignatureKey();
        $this->signed['oauth_signature'] = $this->createSignature($base, $key);
        ksort($this->signed);
    }

    public function createSignature($sigBase, $sigKey)
    {
        return base64_encode(hash_hmac('sha1', $sigBase, $sigKey, true));
    }

    private function createSignatureKey()
    {
        return $this->config['consumer_secret'] . '&' . $this->config['token_secret'];
    }

    private function createSignatureBase()
    {
    	$url = $this->config['url'];
    	$data = $this->signed;
        $method = rawurlencode(strtoupper($this->config['request_method']));
        $query = parse_url($url, PHP_URL_QUERY);
        if ($query) {
            $parts = explode('?', $url, 2);
            $url = array_shift($parts);
            $items = explode('&', $query);
            foreach ($items as $item) {
                list($key, $value) = explode('=', $item);
                $data[rawurldecode($key)] = rawurldecode($value);
            }
        }
        $url = rawurlencode($url);
        $data = rawurlencode($this->collapseDataForSignature($data));
        return $method . '&' . $url . '&' . $data;
    }

    private function collapseDataForSignature($data)
    {
        ksort($data);
        $collapse = '';
        foreach ($data as $key => $val) {
            if (!empty($collapse)) {
                $collapse .= '&';
            }
            $collapse .= $key . '=' . rawurlencode($val);
        }
        return $collapse;
    }

    private function getHighestPriorityToken()
    {
        if (!empty($this->config['access_token'])) {
            return $this->config['access_token'];
        }
        if (!empty($this->config['authorized_token'])) {
            return $this->config['authorized_token'];
        }
        if (!empty($this->config['request_token'])) {
            return $this->config['request_token'];
        }

        // Return no token, new user
        return '';
    }

    private function generateTimestamp()
    {
        return time();
    }

    private function generateNonce($timestamp = false)
    {
        if (!$timestamp) {
            $timestamp = $this->generateTimestamp();
        }
        return md5($timestamp.'-'.rand(10000, 99999).'-'.uniqid());
    }
}